Package: meteoland
Type: Package
Title: Landscape Meteorology Tools
Version: 2.2.7
Date: 2026-05-08
Authors@R: c(
	person('Miquel', 'De Cáceres', role=c('aut','cre'),
	email='miquelcaceres@gmail.com', comment = c(ORCID = "0000-0001-7132-2080")),
	person('Víctor', 'Granda', role='aut', comment = c(ORCID = "0000-0002-0469-1991")),
	person('Nicolas', 'Martin', role='aut', comment = c(ORCID = "0000-0001-7574-0108")),
	person('Antoine', 'Cabon', role='aut', comment = c(ORCID = "0000-0001-6426-1726")))
Description: Functions to estimate weather variables at any position of a landscape [De Caceres et al. (2018) <doi:10.1016/j.envsoft.2018.08.003>].
License: GPL (>= 2)
URL: https://emf-creaf.github.io/meteoland/,
        https://github.com/emf-creaf/meteoland
BugReports: https://github.com/emf-creaf/meteoland/issues
LazyData: TRUE
Depends: R (>= 4.1.0)
Imports: methods, sf, stars, stats, Rcpp, units, lifecycle, cli, dplyr,
        tidyr, rlang, assertthat, purrr, ncdfgeom, ncmeta, lubridate,
        cubelyr
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), meteospain, worldmet,
        tibble
LinkingTo: Rcpp
Encoding: UTF-8
NeedsCompilation: yes
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/roxygen2/version: 8.0.0
Packaged: 2026-05-08 13:08:14 UTC; miquel
Author: Miquel De Cáceres [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7132-2080>),
  Víctor Granda [aut] (ORCID: <https://orcid.org/0000-0002-0469-1991>),
  Nicolas Martin [aut] (ORCID: <https://orcid.org/0000-0001-7574-0108>),
  Antoine Cabon [aut] (ORCID: <https://orcid.org/0000-0001-6426-1726>)
Maintainer: Miquel De Cáceres <miquelcaceres@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-08 14:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-08 16:11:37 UTC; unix
Archs: meteoland.so.dSYM

# Generated by roxygen2: do not edit by hand

export(add_topo)
export(complete_meteo)
export(create_meteo_interpolator)
export(defaultInterpolationParams)
export(get_interpolation_params)
export(humidity_dewtemperature2relative)
export(humidity_relative2dewtemperature)
export(humidity_relative2specific)
export(humidity_specific2relative)
export(interpolate_data)
export(interpolation_cross_validation)
export(interpolation_dewtemperature)
export(interpolation_precipitation)
export(interpolation_temperature)
export(interpolation_wind)
export(interpolator_calibration)
export(meteospain2meteoland)
export(penman)
export(penmanmonteith)
export(precipitation_concentration)
export(precipitation_rainfall_erosivity)
export(radiation_dateStringToJulianDays)
export(radiation_daylength)
export(radiation_daylengthseconds)
export(radiation_directDiffuseDay)
export(radiation_directDiffuseInstant)
export(radiation_julianDay)
export(radiation_netRadiation)
export(radiation_outgoingLongwaveRadiation)
export(radiation_potentialRadiation)
export(radiation_skyLongwaveRadiation)
export(radiation_solarConstant)
export(radiation_solarDeclination)
export(radiation_solarElevation)
export(radiation_solarRadiation)
export(radiation_sunRiseSet)
export(read_interpolator)
export(set_interpolation_params)
export(summarise_interpolated_data)
export(summarise_interpolator)
export(utils_airDensity)
export(utils_atmosphericPressure)
export(utils_averageDailyVP)
export(utils_averageDaylightTemperature)
export(utils_latentHeatVaporisation)
export(utils_latentHeatVaporisationMol)
export(utils_psychrometricConstant)
export(utils_saturationVP)
export(utils_saturationVaporPressureCurveSlope)
export(with_meteo)
export(worldmet2meteoland)
export(write_interpolator)
importFrom(Rcpp,evalCpp)
importFrom(Rcpp,sourceCpp)
importFrom(cubelyr,tbl_cube)
importFrom(dplyr,.data)
importFrom(lifecycle,deprecated)
importFrom(methods,.valueClassTest)
importFrom(rlang,":=")
importFrom(stats,aggregate)
importFrom(stats,complete.cases)
importFrom(stats,cor)
useDynLib(meteoland, .registration = TRUE)

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(meteoland)
library(stars)
library(dplyr)

## ----data_preparation, echo=FALSE---------------------------------------------
unformatted_meteo <- tibble(
  date = rep(seq(from = as.Date("2022-12-01"), to = as.Date("2022-12-05"), by = 1), 3),
  station = rep(letters[1:3], each = 5),
  latitude = rep(c(41.35, 40.11, 42.00), each = 5),
  longitude = rep(c(-0.33, 0.12, 1.12), each = 5),
  min_temp = rnorm(15, 10, 6),
  max_temp = min_temp + rnorm(15, 10, 3),
  rh = rnorm(15, 50, 25)
) |>
  dplyr::mutate(
    rh = dplyr::if_else(rh > 100, 100, rh),
    rh = dplyr::if_else(rh < 20, 20, rh)
  )

## ----meteoland_meteo_example--------------------------------------------------
meteoland_meteo_example

## ----meteo_names--------------------------------------------------------------
names(meteoland_meteo_example)

## ----unformatted_meteo--------------------------------------------------------
unformatted_meteo

## ----unformatted_transformation-----------------------------------------------
ready_meteo <- unformatted_meteo |>
  # convert names to correct ones
  dplyr::mutate(
    MinTemperature = min_temp,
    MaxTemperature = max_temp,
    MeanRelativeHumidity = rh
  ) |>
  # transform to sf (WGS84)
  sf::st_as_sf(
    coords = c("longitude", "latitude"),
    crs = sf::st_crs(4326)
  )

ready_meteo

## ----meteospain, eval = FALSE-------------------------------------------------
# library(meteospain)
# get_meteo_from(
#   "meteogalicia",
#   meteogalicia_options('daily', as.Date("2022-12-01"), as.Date("2022-12-05"))
# ) |>
#   meteospain2meteoland()

## ----worldmet, eval = FALSE---------------------------------------------------
# library(worldmet)
# worldmet::importNOAA("081120-99999", year = 2022) |>
#   worldmet2meteoland()

## ----raster_meteo_preparing, echo = FALSE-------------------------------------
raster_meteo_reference <- interpolate_data(
  raster_to_interpolate_example, meteoland_interpolator_example, verbose = FALSE
)

## -----------------------------------------------------------------------------
raster_meteo_reference

## ----raster_to_points---------------------------------------------------------
points_meteo_reference <- names(raster_meteo_reference) |>
  # for each variable
  purrr::map(
    # take the variable raster
    ~ raster_meteo_reference[.x] |>
      # convert to sf
      sf::st_as_sf(as_points = TRUE, na.rm = FALSE) |>
      # pivot the data for dates to be in one column
      tidyr::pivot_longer(cols = -geometry, names_to = "dates", values_to = .x) |>
      # convert to tibble to fasten the process
      dplyr::as_tibble() |>
      # convert to date and create stationID
      dplyr::mutate(
        dates = as.Date(dates),
        stationID = as.character(geometry)
      )
  ) |>
  # join all variables
  purrr::reduce(dplyr::left_join) |>
  # create the points sf object
  sf::st_as_sf()

points_meteo_reference

## ----raster_interpolator------------------------------------------------------
with_meteo(points_meteo_reference) |>
  create_meteo_interpolator()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(meteoland)
library(stars)
library(dplyr)

## ----points_to_interpolate_example--------------------------------------------
points_to_interpolate_example

## ----raster_to_interpolate_example--------------------------------------------
raster_to_interpolate_example

## ----meteoland_meteo_example--------------------------------------------------
meteoland_meteo_example

## ----meteo_names--------------------------------------------------------------
names(meteoland_meteo_example)

## ----quick_interpolation------------------------------------------------------
# creating the interpolator object
interpolator <- with_meteo(meteoland_meteo_example) |>
  create_meteo_interpolator()

# performing the interpolation
points_interpolated <- points_to_interpolate_example |>
  interpolate_data(interpolator)
points_interpolated

## ----non_mandatory_vars_in_meteo, error=TRUE----------------------------------
try({
meteo_without_temp <- meteoland_meteo_example
meteo_without_temp[["MinTemperature"]] <- NULL
meteo_without_temp[["MaxTemperature"]] <- NULL
with_meteo(meteo_without_temp)
})

## ----interpolatior_params-----------------------------------------------------
# parameters
get_interpolation_params(interpolator)

## ----interpolated_data--------------------------------------------------------
# interpolated meteo for the first location
points_interpolated[["interpolated_data"]][1]

## ----long---------------------------------------------------------------------
tidyr::unnest(points_interpolated, cols = "interpolated_data")

## ----interpolator_class-------------------------------------------------------
class(interpolator)

## ----interpolator_description-------------------------------------------------
interpolator

## ----get_interpolation_params-------------------------------------------------
get_interpolation_params(interpolator)

## ----set_interpolation_params-------------------------------------------------
# wind_height parameter
get_interpolation_params(interpolator)$wind_height

# set a new wind_height parameter and check
interpolator <- set_interpolation_params(interpolator, params = list(wind_height = 5))
get_interpolation_params(interpolator)$wind_height

## ----writing_interpolator-----------------------------------------------------
temporal_folder <- tempdir()
write_interpolator(interpolator, file.path(temporal_folder, "interpolator.nc"))
# file should exists now
file.exists(file.path(temporal_folder, "interpolator.nc"))

## ----reading_interpolator-----------------------------------------------------
file_interpolator <- read_interpolator(file.path(temporal_folder, "interpolator.nc"))
# the read interpolator should be identical to the one we have already
identical(file_interpolator, interpolator)

## ----interpolator_calibration-------------------------------------------------
# min temperature N and alpha before calibration
get_interpolation_params(interpolator)$N_MinTemperature
get_interpolation_params(interpolator)$alpha_MinTemperature

# calibration
interpolator <- interpolator_calibration(
  interpolator,
  variable = "MinTemperature",
  N_seq = c(10, 20),
  alpha_seq = c(1, 10),
  update_interpolation_params = TRUE
)

# parameters after calibration
get_interpolation_params(interpolator)$N_MinTemperature
get_interpolation_params(interpolator)$alpha_MinTemperature

## ----preparing_interpolator---------------------------------------------------
interpolator <- with_meteo(meteoland_meteo_example) |>
  create_meteo_interpolator() |>
  interpolator_calibration(
    variable = "MinTemperature",
    N_seq = c(10, 20),
    alpha_seq = c(1, 10),
    update_interpolation_params = TRUE
  ) |>
  interpolator_calibration(
    variable = "MaxTemperature",
    N_seq = c(10, 20),
    alpha_seq = c(1, 10),
    update_interpolation_params = TRUE
  ) |>
  interpolator_calibration(
    variable = "DewTemperature",
    N_seq = c(10, 20),
    alpha_seq = c(1, 10),
    update_interpolation_params = TRUE
  ) |>
  write_interpolator(
    filename = file.path(temporal_folder, "interpolator.nc"),
    .overwrite = TRUE
  )

## ----cross_validation---------------------------------------------------------
cross_validation <- interpolation_cross_validation(interpolator, verbose = FALSE)
cross_validation$errors
cross_validation$station_stats
cross_validation$dates_stats
cross_validation$r2

## ----summarise_interpolated_data----------------------------------------------
summarise_interpolated_data(
  points_interpolated,
  fun = "mean",
  frequency = "week"
)

## ----erosivity_one_location---------------------------------------------------
precipitation_rainfall_erosivity(
  points_interpolated$interpolated_data[[1]],
  longitude = sf::st_coordinates(points_interpolated$geometry[[1]])[,1],
  scale = 'month'
)

## ----erosivity_mutate---------------------------------------------------------
points_interpolated |>
  mutate(erosivity = precipitation_rainfall_erosivity(
    interpolated_data,
    longitude = sf::st_coordinates(geometry)[,1],
    scale = 'month'
  ))

## ----interpolation_piped------------------------------------------------------
points_interpolated <- points_to_interpolate_example |>
  interpolate_data(interpolator) |>
  summarise_interpolated_data(
    fun = "mean",
    frequency = "week"
  ) |>
  summarise_interpolated_data(
    fun = "max",
    frequency = "month"
  ) |>
  mutate(
    monthly_erosivity = precipitation_rainfall_erosivity(
      interpolated_data,
      longitude = sf::st_coordinates(geometry)[,1],
      scale = 'month'
    )
  )

points_interpolated

## ----raster_to_interpolate----------------------------------------------------
raster_to_interpolate_example

## ----raster_interpolation-----------------------------------------------------
raster_interpolated <- raster_to_interpolate_example |>
  interpolate_data(interpolator)

raster_interpolated

## ----raster_temporal_agg------------------------------------------------------
summarise_interpolated_data(
  raster_interpolated,
  fun = "mean",
  frequency = "week"
)

## ----raster_piped-------------------------------------------------------------
monthly_mean_temperature <- raster_to_interpolate_example |>
  interpolate_data(interpolator, variables = "Temperature") |>
  summarise_interpolated_data(
    fun = "max",
    frequency = "month",
    variable = "MeanTemperature"
  )

plot(monthly_mean_temperature)

#' Interpolation core function
#'
#' Interpolation core process
#'
#' This function takes the points to be interpolated, the interpolator object
#' and an optional vector of dates to perform the interpolation. It uses the
#' vectorised C++ methods, so no need to loop through points or dates.
#'
#' @param sf sf object with the points topography
#' @param interpolator interpolator object
#' @param dates vector with dates (dates must be inside the interpolator date
#'   range)
#' @param variables vector with variable names to be interpolated. NULL (default),
#' will interpolate all variables. Accepted names are "Temperature", "Precipitation",
#' "RelativeHumidity", "Radiation" and "Wind"
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @param ignore_convex_hull_check Logical indicating if convex hull check for points
#' must be honoured. This is useful (and needed) for the
#' \code{\link{interpolation_cross_validation}} function, because when removing stations
#' from the vertices of the convex hull, the latter is reduced and the station removed can
#' be outside the buffered convex hull, triggering an error (100 % of points lay outside
#' the convex hull buffer). It can also be useful to users interpolating on a few points close 
#' but outside of the convex hull.
#'
#' @family interpolator functions
#' @return A tibble for each point, with the dates as rows and the interpolated
#'   data as columns
#'
#' @noRd
.interpolation_point <- function(
    sf, interpolator, dates = NULL, variables = NULL, verbose,
    ignore_convex_hull_check = FALSE
) {
  ## debug
  # browser()

  ## assertions
  # crs assertion. CRS of sf must be changed in the parent env before calling
  # .interpolation_point method, but we check here just in case
  assertthat::assert_that(
    identical(sf::st_crs(interpolator), sf::st_crs(sf)),
    msg = "CRS of sf must be the same as the interpolator"
  )

  ## dates checks
  if (is.null(dates)) {
    dates <- stars::st_get_dimension_values(interpolator, 'date')
  } else {
    dates_ref <- stars::st_get_dimension_values(interpolator, 'date')
    dates_index <- dplyr::between(dates, as.Date(min(dates_ref)), as.Date(max(dates_ref)))
    dates_inside <- dates[dates_index]

    # No dates in range
    if (length(dates_inside) < 1) {
      cli::cli_abort(c(
        "{.arg dates} supplied are outside the {.arg interpolator} date range. No possible interpolation.",
        "i" = "{.arg interpolator} date range is from {dates_ref[1]} to {dates_ref[length(dates_ref)]}"
      ))
    }

    # some dates in range, not all
    if (length(dates_inside) < length(dates)) {
      cli::cli_warn(c(
        "Some {.arg dates} are outside the {.arg interpolator} date range, only dates inside will be used",
        "x" = "Used {.arg dates} {?is/are} {as.character(dates_inside)}"
      ))
    }

    # set correct dates
    dates <- dates_inside
  }

  ## checks point
  # we need to check if points are inside the convex hull. Some points outside is ok (convex hull
  # can left some near border points outside), but a large proportion of points outside can be
  # a sign of wrong interpolator or wrong points used, so:
  #
  #   - warning if some points are outside (< 10%)
  #   - error if more points are outside (>= 10%)
  #   - also, in any case, indicate the points inside and outside
  #
  # Buffer is done with `dist = 10000` for a 10km distance. This is because sf::st_buffer
  # with dist without units, independently of the crs is taken as meters (due to s2 pacakge).
  # As sf imports s2, both packages will be installed, so dist is always to be in meters.
  interpolator_convex_hull <-
    stars::st_get_dimension_values(interpolator, "station") |>
    sf::st_union() |>
    sf::st_convex_hull() |>
    sf::st_buffer(dist = 10000)

  sf_contained <- sf::st_contains(interpolator_convex_hull, sf, sparse = FALSE)
  if (any(!sf_contained) & !ignore_convex_hull_check) {

    # if less than 10% go ahead
    if (sum(!sf_contained)/length(sf_contained) < 0.1) {
      cli::cli_warn(c(
        "Some points are outside the convex hull of the {.arg interpolator} object.",
        "x" = "Index{?es} of outside point{?s} {?is/are} {.val {as.character(which(!sf_contained))}}"
      ))
    } else {
      cli::cli_abort(c(
        "More than 10% of the points in {.arg spatial_data} fall outside the convex hull of the interpolator object.",
        "x" = "Aborting interpolation",
        "i" = "{.arg spatial_data} index{?es} outside {?is/are} {.val {as.character(which(!sf_contained))}}"
      ))
    }
  }

  .verbosity_control(
    cli::cli_alert_info("Starting interpolation..."),
    verbose
  )
  ## extraction and calculations
  # filtered interpolator, just once
  filtered_interpolator <- interpolator |> dplyr::filter(as.Date(date) %in% as.Date(dates))
  tmin_interpolator <- t(filtered_interpolator[["MinTemperature"]])
  tmax_interpolator <- t(filtered_interpolator[["MaxTemperature"]])
  # constants and helpers
  DOY <- as.numeric(format(dates, "%j"))
  J <- radiation_dateStringToJulianDays(as.character(dates))
  latrad <- sf::st_coordinates(sf::st_transform(sf, 4326))[,2] * (pi/180)
  slorad <- sf[["slope"]] * (pi/180)
  asprad <- sf[["aspect"]] * (pi/180)

  .as_interpolator_res_array <- function(vector, ref_dim) {
    return(array(vector, dim = ref_dim))
  }

  # default NULL objects to substitute for calculations
  tmean <- NULL
  tmin <- NULL
  tmax <- NULL
  prec <- NULL
  rhmean <- NULL
  rhmin <- NULL
  rhmax <- NULL
  rad <- NULL
  wind <- NULL
  pet <- NULL

  # Logical vector excluding points with no elevation values (e.g. incomplete rasters)
  target_filter <- !is.na(sf$elevation)
  dim_out <- c(length(latrad), ncol(tmin_interpolator))
  
  # temperature (needed also if interpolating relative humidity and radiation)
  if (is_null_or_variable(variables, c("Temperature", "RelativeHumidity", "Radiation", "PET"))) {

    if (is_null_or_variable(variables, c("RelativeHumidity", "Radiation", "PET"))) {
      .verbosity_control(
        cli::cli_alert_info("Temperature interpolation is needed also..."),
        verbose
      )
    }

    .verbosity_control(
      cli::cli_ul("Interpolating temperature..."),
      verbose
    )

    tmin <- array(NA, dim_out)
    tmax <- array(NA, dim_out)

    tmin[target_filter,] <- .interpolateTemperatureSeriesPoints(
      Xp = sf::st_coordinates(sf)[target_filter,1], Yp = sf::st_coordinates(sf)[target_filter,2],
      Zp = sf$elevation[target_filter],
      X = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,1],
      Y = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,2],
      Z = interpolator$elevation[1,],
      T = tmin_interpolator,
      iniRp = get_interpolation_params(interpolator)$initial_Rp,
      alpha = get_interpolation_params(interpolator)$alpha_MinTemperature,
      N = get_interpolation_params(interpolator)$N_MinTemperature,
      iterations = get_interpolation_params(interpolator)$iterations,
      debug = get_interpolation_params(interpolator)$debug
    )

    tmax[target_filter, ] <- .interpolateTemperatureSeriesPoints(
      Xp = sf::st_coordinates(sf)[target_filter,1], Yp = sf::st_coordinates(sf)[target_filter,2],
      Zp = sf$elevation[target_filter],
      X = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,1],
      Y = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,2],
      Z = interpolator$elevation[1,],
      T = tmax_interpolator,
      iniRp = get_interpolation_params(interpolator)$initial_Rp,
      alpha = get_interpolation_params(interpolator)$alpha_MaxTemperature,
      N = get_interpolation_params(interpolator)$N_MaxTemperature,
      iterations = get_interpolation_params(interpolator)$iterations,
      debug = get_interpolation_params(interpolator)$debug
    )
    
    # Swap values if tmin > tmax
    swap_filter <- (tmin > tmax)
    swap_filter[!target_filter, ] <- FALSE
    swap_filter[is.na(swap_filter)] <- FALSE
    if(sum(swap_filter)>0) {
      tmax_tmp <- tmax[swap_filter]
      tmax[swap_filter] <- tmin[swap_filter]
      tmin[swap_filter] <- tmax_tmp
    }
    
    # Estimate tmean
    tmean <- 0.606*tmax+0.394*tmin
  }

  # precipitation (needed also if interpolating radiation)
  if (is_null_or_variable(variables, c("Precipitation", "Radiation", "PET"))) {
    if (is_null_or_variable(variables, c("Radiation", "PET"))) {
      .verbosity_control(
        cli::cli_alert_info("Precipitation interpolation is needed also..."),
        verbose
      )
    }

    .verbosity_control(
      cli::cli_ul("Interpolating precipitation..."),
      verbose
    )
    
    prec <- array(NA, dim = dim_out)
    prec[target_filter,] <- .interpolatePrecipitationSeriesPoints(
      Xp = sf::st_coordinates(sf)[target_filter,1], Yp = sf::st_coordinates(sf)[target_filter,2],
      Zp = sf$elevation[target_filter],
      X = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,1],
      Y = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,2],
      Z = interpolator$elevation[1,],
      P = t(filtered_interpolator[["Precipitation"]]),
      Psmooth = t(filtered_interpolator[["SmoothedPrecipitation"]]),
      iniRp = get_interpolation_params(interpolator)$initial_Rp,
      alpha_event = get_interpolation_params(interpolator)$alpha_PrecipitationEvent,
      alpha_amount = get_interpolation_params(interpolator)$alpha_PrecipitationAmount,
      N_event = get_interpolation_params(interpolator)$N_PrecipitationEvent,
      N_amount = get_interpolation_params(interpolator)$N_PrecipitationAmount,
      iterations = get_interpolation_params(interpolator)$iterations,
      popcrit = get_interpolation_params(interpolator)$pop_crit,
      fmax = get_interpolation_params(interpolator)$f_max,
      debug = get_interpolation_params(interpolator)$debug
    )
  }

  # relative humidity (needed also if interpolating radiation)
  if (is_null_or_variable(variables, c("RelativeHumidity", "Radiation", "PET"))) {
    if (is_null_or_variable(variables, c("Radiation", "PET"))) {
      .verbosity_control(
        cli::cli_alert_info("Relative humidity interpolation is needed also..."),
        verbose
      )
    }
    .verbosity_control(
      cli::cli_ul("Interpolating relative humidity..."),
      verbose
    )
    # relative humidity, depends on if we have it or not
    # If we dont, estimate VP assuming that dew-point temperature is equal to Tmin
    if (all(is.na(filtered_interpolator[["RelativeHumidity"]]))) {

      rhmean <- .relativeHumidityFromMinMaxTemp(tmin, tmax) |>
        .as_interpolator_res_array(dim_out)
      VP <- .temp2SVP(tmin) |> #kPA
        .as_interpolator_res_array(dim_out)
      rhmax <- rep(100, length(rhmean)) |>
        .as_interpolator_res_array(dim_out)
      rhmin <- pmax(0, .relativeHumidityFromDewpointTemp(tmax, tmin)) |>
        .as_interpolator_res_array(dim_out)

    } else {

      TdewM <- .dewpointTemperatureFromRH(
        0.606 * filtered_interpolator[["MaxTemperature"]] +
          0.394 * filtered_interpolator[["MinTemperature"]],
        filtered_interpolator[["RelativeHumidity"]]
      )

      tdew <- array(NA, dim = dim_out)
      tdew[target_filter, ] <- .interpolateTdewSeriesPoints(
        Xp = sf::st_coordinates(sf)[target_filter,1], Yp = sf::st_coordinates(sf)[target_filter,2],
        Zp = sf$elevation[target_filter],
        X = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,1],
        Y = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,2],
        Z = interpolator$elevation[1,],
        T = t(TdewM),
        iniRp = get_interpolation_params(interpolator)$initial_Rp,
        alpha = get_interpolation_params(interpolator)$alpha_DewTemperature,
        N = get_interpolation_params(interpolator)$N_DewTemperature,
        iterations = get_interpolation_params(interpolator)$iterations,
        debug = get_interpolation_params(interpolator)$debug
      )

      rhmean <- .relativeHumidityFromDewpointTemp(tmean, tdew) |>
        .as_interpolator_res_array(dim_out)
      VP <- .temp2SVP(tdew) |>
        .as_interpolator_res_array(dim_out) #kPa
      rhmax = pmin(100, .relativeHumidityFromDewpointTemp(tmin, tdew)) |>
        .as_interpolator_res_array(dim_out)
      rhmin = pmax(0, .relativeHumidityFromDewpointTemp(tmax, tdew)) |>
        .as_interpolator_res_array(dim_out)
    }
  }

  # radiation
  if (is_null_or_variable(variables, c("Radiation", "PET"))) {

    if (is_null_or_variable(variables, c("PET"))) {
      .verbosity_control(
        cli::cli_alert_info("Radiation calculation is needed also..."),
        verbose
      )
    }
    .verbosity_control(
      cli::cli_ul("Calculating radiation..."),
      verbose
    )
    diffTemp <- abs(tmax - tmin)
    diffTempMonth <- array(NA, dim = dim_out)
    diffTempMonth[target_filter,] <- .interpolateTemperatureSeriesPoints(
      Xp = sf::st_coordinates(sf)[target_filter,1], Yp = sf::st_coordinates(sf)[target_filter,2],
      Zp = sf$elevation[target_filter],
      X = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,1],
      Y = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,2],
      Z = interpolator$elevation[1,],
      T = abs(t(filtered_interpolator[["SmoothedTemperatureRange"]])),
      iniRp = get_interpolation_params(interpolator)$initial_Rp,
      alpha = get_interpolation_params(interpolator)$alpha_MinTemperature,
      N = get_interpolation_params(interpolator)$N_MinTemperature,
      iterations = get_interpolation_params(interpolator)$iterations,
      debug = get_interpolation_params(interpolator)$debug
    )

    rad <- array(NA, dim = dim_out)
    points_wo_precip <- numeric(0)
    for (i in which(target_filter)) {
      if (any(is.na(prec[i,]))) {
        points_wo_precip <- c(points_wo_precip, i)
      }
      rad[i,] <- .radiationSeries(
        latrad[i], sf$elevation[i], slorad[i], asprad[i],
        J,
        diffTemp[i,], diffTempMonth[i,],
        VP[i,], prec[i,]
      )
    }
    if (length(points_wo_precip) > 0) {
      cli::cli_warn(c(
        "Some/All dates for points {.val {points_wo_precip}} have missing precipitation values, assuming clear days when interpolating radiation for these days"
      ))
    }
  }

  # Wind
  if (is_null_or_variable(variables, c("Wind", "PET"))) {

    if (is_null_or_variable(variables, c("PET"))) {
      .verbosity_control(
        cli::cli_alert_info("Wind interpolation is needed also..."),
        verbose
      )
    }
    .verbosity_control(
      cli::cli_ul("Interpolating wind..."),
      verbose
    )
    wind_speed <- array(NA, dim_out)
    wind_direction <- array(NA, dim_out)
    wind <- .interpolateWindStationSeriesPoints(
      Xp = sf::st_coordinates(sf)[target_filter,1], Yp = sf::st_coordinates(sf)[target_filter,2],
      WS = t(filtered_interpolator[["WindSpeed"]]),
      WD = t(filtered_interpolator[["WindDirection"]]),
      X = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,1],
      Y = sf::st_coordinates(stars::st_get_dimension_values(interpolator, "station"))[,2],
      iniRp = get_interpolation_params(interpolator)$initial_Rp,
      alpha = get_interpolation_params(interpolator)$alpha_Wind,
      N = get_interpolation_params(interpolator)$N_Wind,
      iterations = get_interpolation_params(interpolator)$iterations
    )
    wind_speed[target_filter, ] <- wind[["WS"]]
    wind_direction[target_filter, ] <- wind[["WD"]]
    wind <- list("WS" = wind_speed, "WD" = wind_direction)
  }

  # PET
  if (is_null_or_variable(variables, c("PET"))) {
    .verbosity_control(
      cli::cli_ul("Calculating PET..."),
      verbose
    )

    pet <- array(NA, dim = dim_out)

    params <- get_interpolation_params(interpolator)
    #Sets defaults if missing
    if(!("wind_height" %in% names(params))) params$wind_height = 10
    if(!("wind_roughness_height" %in% names(params))) params$wind_roughness_height = 0.001
    if(!("penman_albedo" %in% names(params))) params$penman_albedo = 0.25
    if(!("penman_windfun" %in% names(params))) params$penman_windfun = "1956"
    for (i in which(target_filter)) {
      pet[i,] <- .penmanpoint(
        latrad[i], sf$elevation[i], slorad[i], asprad[i], J,
        tmin[i,], tmax[i,], rhmin[i,], rhmax[i,], rad[i,],
        wind$WS[i,], 
        params$wind_height,
        params$wind_roughness_height, 
        params$penman_albedo,
        params$penman_windfun
      )
    }
  }

  # return the res df
  res <- purrr::map(
    1:length(latrad),
    ~ dplyr::tibble(
      dates = dates,
      DOY = DOY,
      MeanTemperature = as.vector(null2na(tmean[.x,])),
      MinTemperature = as.vector(null2na(tmin[.x,])),
      MaxTemperature = as.vector(null2na(tmax[.x,])),
      Precipitation = as.vector(null2na(prec[.x,])),
      MeanRelativeHumidity = null2na(rhmean[.x,]),
      MinRelativeHumidity = null2na(rhmin[.x,]),
      MaxRelativeHumidity = null2na(rhmax[.x,]),
      Radiation = null2na(rad[.x,]),
      WindSpeed = as.vector(null2na(wind$WS[.x,])),
      WindDirection = as.vector(null2na(wind$WD[.x,])),
      PET = null2na(pet[.x,])
    )
  )

  .verbosity_control(
    cli::cli_alert_success("Interpolation done..."),
    verbose
  )

  return(res)
}

#' Dispatcher of spatial data to interpolation points
#'
#' Dispatcher of different kinds (stars, sf) of spatial data to interpolation
#' point function
#'
#' This function takes the spatial data, checks its class (stars or sf), apply
#' some assertions specific of the class and pass the spatial data to the
#' \code{\linl{.interpolation_point}} function
#'
#' @param spatial_data stars or sf object with the spatial data
#' @param interpolator meteoland interpolator object, as created by
#'   \code{\link{create_meteo_interpolator}}
#' @param dates vector with dates to interpolate (must be within the
#'   interpolator date range). Default to NULL (all dates present in the
#'   interpolator object)
#' @param variables vector with variable names to be interpolated. NULL (default),
#' will interpolate all variables. Accepted names are "Temperature", "Precipitation",
#' "RelativeHumidity", "Radiation" and "Wind"
#' @param ignore_convex_hull_check Logical indicating whether errors in convex hull checks should be ignored. 
#' Checking for points to be inside the convex hull will normally raise an error
#' if >10% of points are outside. Setting \code{ignore_convex_hull_check = TRUE} means
#' that a warning is raised but interpolation is performed, which can be useful to users interpolating 
#' on a few points close but outside of the convex hull.
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#'
#' @return the same as \code{\link{.interpolation_point}} function.
#'
#' @noRd
.interpolation_spatial_dispatcher <- function(
    spatial_data, interpolator, dates = NULL, variables = NULL, 
    ignore_convex_hull_check = FALSE, verbose
) {

  # sf
  if (inherits(spatial_data, "sf")) {
    # assertions
    assertthat::assert_that(
      inherits(sf::st_geometry(spatial_data), "sfc_POINT"),
      msg = paste0(
        "Spatial data must inhertis from sfc_POINT, not ",
        class(sf::st_geometry(spatial_data))[1]
      )
    )

    # interpolation
    return(
      spatial_data |>
        sf::st_transform(sf::st_crs(interpolator)) |>
        .interpolation_point(interpolator, dates, variables, 
                             ignore_convex_hull_check = ignore_convex_hull_check, 
                             verbose = verbose)
    )

  }

  # stars
  if (inherits(spatial_data, 'stars')) {
    return(
      spatial_data |>
        .stars2sf() |>
        sf::st_transform(sf::st_crs(interpolator)) |>
        .interpolation_point(interpolator, dates, variables, 
                             ignore_convex_hull_check = ignore_convex_hull_check, 
                             verbose = verbose)
    )
  }

  cli::cli_abort(c(
    "No compatible {.arg spatial_data} found.",
    "x" = "{.arg spatial_data} must be an {.pkg sf} or a {.pkg stars} object"
  ))
}

#' Binding interpolation results to spatial data provided
#'
#' Binding interpolation results to spatial data provided
#'
#' This function takes the results list of the interpolation and join it with
#' the provided spatial data (raster or sf)
#'
#' @param res_list results list as generated by \code{\link{.interpolation_point}}
#' @param spatial_data spatial data provided
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#'
#' @return an object with the same class and structure as the provided spatial
#'   data with the results of the interpolation joined. In the case of spatial
#'   data being an sf, the results are added as a list-type column that can be
#'   unnested with \code{\link[tidyr]{unnest}}. In the case of a stars raster
#'   object, interpolation results are added as attributes (variables)
#'
#' @noRd
.binding_interpolation_results <- function(res_list, spatial_data, verbose) {
  # debug
  # browser()

  ## points
  if (inherits(spatial_data, 'sf')) {
    binded_result <-
      spatial_data |>
      dplyr::as_tibble() |>
      dplyr::mutate(interpolated_data = res_list) |>
      sf::st_as_sf()

    return(binded_result)
  }

  ## rasters
  # check on dimensions
  x_name_index <-
    which(
      names(stars::st_dimensions(spatial_data)) %in% c('x', 'X', 'long', 'longitude')
    )
  y_name_index <-
    which(
      names(stars::st_dimensions(spatial_data)) %in% c('y', 'Y', 'lat', 'latitude')
    )

  assertthat::assert_that(
    length(x_name_index) > 0,
    msg = "Can't find x dimension. x dimension in spatial data must be named 'x', 'X', 'long' or 'longitude'"
  )
  assertthat::assert_that(
    length(y_name_index) > 0,
    msg = "Can't find y dimension. y dimension in spatial data must be named 'y', 'Y', 'lat' or 'latitude'"
  )

  # dimensions
  result_dimensions <- stars::st_dimensions(
    x = stars::st_get_dimension_values(spatial_data, x_name_index),
    y = stars::st_get_dimension_values(spatial_data, y_name_index),
    date = res_list[[1]][['dates']]
  )

  # info
  .verbosity_control(
    cli::cli_alert_info("Binding together interpolation results"),
    verbose
  )

  # helper
  .result_arrays_creator <- function(variable, res_list, dims) {
    variable_vecs <- purrr::map(res_list, ~ .x[[variable]])
    # array_dims <- c(y = dims[['y']], x = dims[['x']], date = dims[['date']])

    var_long_vec <- numeric()
    for (i in 1:length(variable_vecs[[1]])) {
      var_long_vec <- c(
        var_long_vec,
        variable_vecs |> purrr::map_dbl(~.x[i])
      )
    }
    var_long_vec |> array(dim = dims)
  }

  res_vars <- names(res_list[[1]]) |>
    purrr::keep(~ !.x %in% c("dates", "DOY"))
  binded_result <- res_vars |>
    purrr::map(~ .result_arrays_creator(.x, res_list, dim(result_dimensions))) |>
    purrr::set_names(res_vars) |>
    stars::st_as_stars(dimensions = result_dimensions)

  # crs
  sf::st_crs(binded_result) <- sf::st_crs(spatial_data)

  # user data, if slope and/or aspect are null, they will be null
  binded_result[["elevation"]] <- spatial_data[["elevation"]]
  binded_result[["slope"]] <- spatial_data[["slope"]]
  binded_result[["aspect"]] <- spatial_data[["aspect"]]

  .verbosity_control(
    cli::cli_alert_success("Interpolation process finished"),
    verbose
  )
  return(binded_result)

}

#' Topography raster (stars) cells to points
#'
#' Converting stars cells to a topography point object compatible with
#' .interpolation_point
#'
#' This function takes on a stars raster and check for mandatory variables,
#' returning an sf object
#'
#' @param stars topography stars object with at least elevation variable
#'
#' @return a sf object with raster cells converted to points (taking the center
#'   of the cell)
#'
#' @noRd
.stars2sf <- function(stars) {
  # assertions
  assertthat::assert_that(
    inherits(stars, "stars"), msg = "No stars class object detected"
  )
  assertthat::assert_that(
    "elevation" %in% names(stars),
    msg = "No elevation variable/attribute found in data"
  )

  stars |>
    sf::st_as_sf(as_points = TRUE, na.rm = FALSE) |>
    dplyr::select(dplyr::any_of(c("elevation", "slope", "aspect")))

}

#' Interpolation process for spatial data
#'
#' Interpolate spatial data to obtain downscaled meteorologic variables
#'
#' This function takes a spatial data object (sf or stars raster), an
#' interpolator object (\code{\link{create_meteo_interpolator}}) and a
#' vector of dates to perform the interpolation of the meteorologic variables
#' for the spatial locations present in the \code{spatial_data} object.
#'
#' @param spatial_data An sf or stars raster object to interpolate
#' @param interpolator A meteoland interpolator object, as created by
#' \code{\link{create_meteo_interpolator}}
#' @param dates vector with dates to interpolate (must be within the
#' interpolator date range). Default to NULL (all dates present in the
#' interpolator object)
#' @param variables vector with variable names to be interpolated. NULL (default),
#' will interpolate all variables. Accepted names are "Temperature", "Precipitation",
#' "RelativeHumidity", "Radiation" and "Wind"
#' @param ignore_convex_hull_check Logical indicating whether errors in convex hull checks should be ignored. 
#' Checking for points to be inside the convex hull will normally raise an error
#' if >10% of points are outside. Setting \code{ignore_convex_hull_check = TRUE} means
#' that a warning is raised but interpolation is performed, which can be useful to users interpolating 
#' on a few points close but outside of the convex hull.
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return an object with the same class and structure as the provided spatial
#' data with the results of the interpolation joined. In the case of spatial
#' data being an sf, the results are added as a list-type column that can be
#' unnested with \code{\link[tidyr]{unnest}}. In the case of a stars raster
#' object, interpolation results are added as attributes (variables)
#' @section Spatial data:
#'
#' The spatial data provided must be of two types. (I) A sf object containing
#' POINT for each location to interpolate or (II) a stars raster object for
#' which the interpolation should be done. Independently of the class of
#' \code{spatial_data} it has to have some mandatory variables, namely
#' \code{elevation}. It should also contain \code{aspect} and \code{slope} for
#' a better interpolation process, though this two variables are not mandatory.
#'
#' @author 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF
#' 
#' @examples
#'
#' \donttest{
#' # example of data to interpolate and example interpolator
#' data("points_to_interpolate_example")
#' data("meteoland_interpolator_example")
#'
#' # interpolate data
#' res <- interpolate_data(points_to_interpolate_example, meteoland_interpolator_example)
#'
#' # check result
#' # same class as input data
#' class(res)
#' # data
#' res
#' # results for the first location
#' res[["interpolated_data"]][1]
#' # unnest results
#' tidyr::unnest(res, cols = "interpolated_data")
#' }
#'
#' @export interpolate_data
interpolate_data <- function(
    spatial_data, interpolator,
    dates = NULL, variables = NULL,
    ignore_convex_hull_check = FALSE,
    verbose = getOption("meteoland_verbosity", TRUE)
) {
  # debug
  # browser()

  ## assertions
  # spatial
  assertthat::assert_that(.is_spatial_data(spatial_data))
  assertthat::assert_that(has_topo_names(spatial_data))

  if (inherits(spatial_data, 'stars')) {
    assertthat::assert_that(.is_raster(spatial_data))
    # curvilinear
    assertthat::assert_that(
      !attr(stars::st_dimensions(spatial_data), "raster")$curvilinear,
      msg = "Curvilinear grids are not yet supported. Please reproject/warp the raster to regular coordinates."
    )
  }

  # dates
  if (!is.null(dates)) {
    assertthat::assert_that(
      assertthat::is.date(dates) | assertthat::is.time(dates),
      msg = "dates object provided doesn't contain dates (POSIX)"
    )
  }

  # variables
  if (!is.null(variables)) {
    assertthat::assert_that(
      all(variables %in% c("Temperature", "Precipitation", "RelativeHumidity", "Wind", "Radiation", "PET")),
      msg = "variables argument must be NULL (all variables) or a character vector with one o more of the following: 'Temperture', 'Precipitation', 'RelativeHumidity', 'Wind', 'Radiation'"
    )
  }

  # interpolator
  assertthat::assert_that(.is_interpolator(interpolator))

  # last spatial check. Here because that way all essential checks are done
  # before
  optional_topo_names <- c("slope", "aspect")
  if (!all(optional_topo_names %in% names(spatial_data))) {
    cli::cli_warn(c(
      "{.var aspect} and {.var slope} variables are not mandatory, but the interpolation will be less accurate without them"
    ))
  }

  # Interpolation
  res <-
    .interpolation_spatial_dispatcher(spatial_data, interpolator, dates, variables, 
                                      ignore_convex_hull_check = ignore_convex_hull_check, 
                                      verbose = verbose) |>
    # results binding
    .binding_interpolation_results(spatial_data, verbose = verbose)

  return(res)
}

#' Retrieving interpolation parameters from user
#'
#' Internal function to ensure all interpolation parameters are included in the
#' final interpolator
#'
#' This functions ensures that if no parameters are provided, the default ones
#' are used. Also, if params are partially provided, this functions ensures that
#' the rest of the parameters are taken from the defaults, allowing the user to
#' provide only those parameters they want to change from default ones. This
#' function is designed to be used internally by other functions in the package
#'
#' @param params list with the parameters provided by the user
#' @param ref reference parameters to get missing ones from params argument. Default
#'   to \code{\link{defaultInterpolationParams}}
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#'
#' @return A complete parameter list to use in the interpolator object
#' @family interpolator functions
#' @noRd
.safely_create_interpolation_params <- function(params, ref = defaultInterpolationParams(), verbose = getOption("meteoland_verbosity", TRUE)) {
  if (is.null(params)) {
    cli::cli_warn(c(
      "No interpolation parameters provided, using defaults",
      "i" = "Set the {.arg params} argument to modify parameter default values"
    ))
    return(defaultInterpolationParams())
  }

  user_params <- params[names(params) %in% names(ref)]

  if (length(user_params) < length(ref)) {
    .verbosity_control(
      cli::cli_alert_info("Some interpolation parameters are missing, using default values for those"),
      verbose
    )
  }

  if (length(params[!names(params) %in% names(ref)]) > 0) {
    offending_params <- names(
      params[!names(params) %in% names(ref)]
    )
    cli::cli_warn(c(
      "The following interpolation parameter{?s} {?was/were} provided and will not be used: {.val {offending_params}}"
    ))
  }

  for (name in names(user_params)) {
    ref[[name]] <- user_params[[name]]
  }

  return(ref)
}

#' Retrieving interpolation parameters from interpolator object
#'
#' Retrieve the parameter list from and interpolator object
#'
#'
#' @param interpolator interpolator object as returned by
#' \code{\link{create_meteo_interpolator}}
#' @return The complete parameter list from the interpolator object
#' @seealso Other interpolator functions: \code{\link{add_topo}()},
#' \code{\link{create_meteo_interpolator}()},
#' \code{\link{read_interpolator}()}, \code{\link{set_interpolation_params}()},
#' \code{\link{with_meteo}()}, \code{\link{write_interpolator}()}
#'
#' @author 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' @examples
#' # example interpolator
#' data(meteoland_interpolator_example)
#' # get the params from the interpolator
#' get_interpolation_params(meteoland_interpolator_example)
#'
#' @export get_interpolation_params
get_interpolation_params <- function(interpolator) {
  return(attr(interpolator, "params"))
}

#' Setting interpolation parameters in an interpolator object
#'
#' Changing or updating interpolation parameters in an interpolator object
#'
#' This function ensures that if no parameters are provided, the default ones
#' are used (see \code{\link{defaultInterpolationParams}}). Also, if params are
#' partially provided, this function ensures that the rest of the parameters
#' are not changed.
#'
#' @param interpolator interpolator object to update
#' @param params list with the parameters provided by the user
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return The same interpolator object provided, with the updated
#' interpolation parameters
#' @seealso Other interpolator functions: \code{\link{add_topo}()},
#' \code{\link{create_meteo_interpolator}()},
#' \code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
#' \code{\link{with_meteo}()}, \code{\link{write_interpolator}()}
#'
#' @author 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' @examples
#' # example interpolator
#' data(meteoland_interpolator_example)
#' # store the actual parameters
#' old_parameters <- get_interpolation_params(meteoland_interpolator_example)
#' # we can provide only the parameter we want to change
#' meteoland_interpolator_example <- set_interpolation_params(
#'   meteoland_interpolator_example,
#'   list(debug = TRUE)
#' )
#' # check
#' get_interpolation_params(meteoland_interpolator_example)$debug
#' # compare with old
#' old_parameters$debug
#' # the rest should be the same
#' setdiff(old_parameters, get_interpolation_params(meteoland_interpolator_example))
#'
#' @export set_interpolation_params
set_interpolation_params <- function(interpolator, params = NULL, verbose = getOption("meteoland_verbosity", TRUE)) {

  # sometimes, the interpolator cab lost the params (aggregation, subsetting...),
  # so in that case, we need to go for the default ones
  ref_params <- get_interpolation_params(interpolator)
  if (is.null(ref_params)) {
    ref_params <- defaultInterpolationParams()
  }

  # ensure the params are correct and fill any lacking with the defaults
  safe_params <-
    .safely_create_interpolation_params(params, ref_params, verbose)
  # remove any previous params
  attr(interpolator, "params") <- NULL
  # add new ones
  attr(interpolator, "params") <- safe_params

  return(interpolator)
}

#' Meteoland interpolator creation
#'
#' Function to create the meteoland interpolator
#'
#' This function takes meteorology information and a list of interpolation
#' parameters and creates the interpolator object to be ready to use.
#'
#' @param meteo_with_topo Meteo object, as returned by \code{\link{with_meteo}}
#' @param params Interpolation parameters as a list. Typically the result of
#'   \code{\link{defaultInterpolationParams}}.
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return an interpolator object (stars)
#' @seealso Other interpolator functions: \code{\link{add_topo}()},
#' \code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
#' \code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()},
#' \code{\link{write_interpolator}()}
#'
#' @author 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF
#' 
#' @examples
#'
#' # example meteo data
#' data(meteoland_meteo_example)
#'
#' # create the interpolator with default params
#' with_meteo(meteoland_meteo_example) |>
#'   create_meteo_interpolator()
#'
#' # create the interpolator with some params changed
#' with_meteo(meteoland_meteo_example) |>
#'   create_meteo_interpolator(params = list(debug = TRUE))
#'
#' @export create_meteo_interpolator
create_meteo_interpolator <- function(meteo_with_topo, params = NULL, verbose = getOption("meteoland_verbosity", TRUE)) {

  ## TODO messaging
  assertthat::assert_that(has_meteo(meteo_with_topo))
  assertthat::assert_that(has_topo(meteo_with_topo))

  assertthat::assert_that(
    is.null(params) || inherits(params, "list"),
    msg = "params must be NULL or a named list with the interpolation parameters"
  )

  .verbosity_control(
    cli::cli_alert_info("Creating interpolator..."),
    verbose
  )

  # get params
  params <- .safely_create_interpolation_params(params, verbose = verbose)

  # data preparation

  # elevation filtering
  if (any(is.na(meteo_with_topo[["elevation"]]))) {

    meteo_with_topo <- meteo_with_topo |>
      dplyr::filter(!is.na(.data$elevation))

    if (nrow(meteo_with_topo) < 1) {
      cli::cli_abort(c(
        "No elevation values for any station, stopping creation of the interpolator"
      ))
    }

    # warning here, because that way if there is no elevation whatsoever we get
    # only the error (as it should be), not the warning AND the error.
    cli::cli_warn(c(
      "Some meteo stations lack values for elevation, filtering those stations out"
    ))
  }

  # helper to avoid NAs in elevation when completing cases
  .fill_elevation <- function(arranged_data) {
    arranged_data |>
      dplyr::group_by(.data$stationID) |>
      dplyr::mutate(
        elevation = dplyr::if_else(
          is.na(.data$elevation),
          unique(purrr::keep(.data$elevation, ~!is.na(.x))),
          .data$elevation
        )
      ) |>
      dplyr::ungroup() |>
      sf::st_as_sf()
  }

  # helper to complete topo when completing cases, also, convert any aspect or slope
  # that is NA in 0 as it's better for the interpolation.
  .fill_topo <- function(arranged_data) {

    # create the aspect and slope vars if they dont exist
    if (is.null(arranged_data[["slope"]])) {
      arranged_data[["slope"]] <- 0
    }
    if (is.null(arranged_data[["aspect"]])) {
      arranged_data[["aspect"]] <- 0
    }
    # fill the variables
    arranged_data |>
      dplyr::group_by(.data$stationID) |>
      dplyr::mutate(
        # fill elevation
        elevation = dplyr::if_else(
          is.na(.data$elevation),
          unique(purrr::keep(.data$elevation, ~!is.na(.x))),
          .data$elevation
        ),
        # fill slope and aspect
        slope = dplyr::if_else(
          is.na(.data$slope),
          0,
          .data$slope
        ),
        aspect = dplyr::if_else(
          is.na(.data$aspect),
          0,
          .data$aspect
        )
      ) |>
      dplyr::ungroup() #|>
      # sf::st_as_sf()
  }

  # data arranging
  meteo_arranged <- meteo_with_topo |>
    dplyr::as_tibble() |>
    # very important step, as we need all combinations to fill the arrays
    tidyr::complete(.data$dates, .data$stationID, explicit = FALSE) |>
    dplyr::arrange(.data$stationID, .data$dates) |>
    # .fill_elevation()
    .fill_topo()
  
  geom_var <- attr(meteo_with_topo, "sf_column")

  # stations and dates
  stations <- meteo_arranged |>
    dplyr::filter(
      # !sf::st_is_empty(!!dplyr::sym(attr(meteo_with_topo, "sf_column")))
      !sf::st_is_empty(.data[[geom_var]])
    ) |>
    dplyr::select(dplyr::all_of(c("stationID", geom_var))) |>
    dplyr::distinct() |>
    dplyr::pull(geom_var) |>
    sf::st_geometry()
  dates <- unique(meteo_arranged$dates)

  # dimensions
  interpolator_dims <- stars::st_dimensions(
    date = dates, station = stations
  )

  # helper
  .interpolator_arrays_creator <- function(variable, arranged_data, dims) {
    col_names <- arranged_data |>
      dplyr::pull("stationID") |>
      unique()

    if (! variable %in% names(arranged_data)) {
      array_data <- array(NA_real_, dim = dims)
    } else {
      variable_data <- arranged_data |>
        dplyr::pull(!!variable)
      # transform logical NAs to numeric NAs
      if (is.logical(variable_data)) {
        variable_data <- as.numeric(variable_data)
      }
      array_data <- variable_data |>
        array(dim = dims)
    }

    colnames(array_data) <- col_names
    rownames(array_data) <- unique(as.character(arranged_data$dates))

    return(array_data)
  }

  stars_interpolator <-
    list(
      Temperature = "MeanTemperature",
      MinTemperature = "MinTemperature",
      MaxTemperature = "MaxTemperature",
      RelativeHumidity = "MeanRelativeHumidity",
      Precipitation = "Precipitation",
      Radiation = "Radiation",
      WindDirection = "WindDirection",
      WindSpeed = "WindSpeed",
      elevation = "elevation",
      aspect = "aspect",
      slope = "slope"
      # stationID = "stationID"
    ) |>
    purrr::map(
      .interpolator_arrays_creator,
      arranged_data = meteo_arranged,
      dims = dim(interpolator_dims)
    ) |>
    purrr::discard(is.null) |>
    stars::st_as_stars(dimensions = interpolator_dims)


  # add smoothed vars
  .verbosity_control(
    cli::cli_ul("Calculating smoothed variables..."),
    verbose
  )
  stars_interpolator[["SmoothedPrecipitation"]] <-
    .temporalSmoothing(
      t(stars_interpolator[["Precipitation"]]),
      params$St_Precipitation,
      TRUE
    ) |> t()
  colnames(stars_interpolator[["SmoothedPrecipitation"]]) <-
    colnames(stars_interpolator[["Precipitation"]])
  attributes(stars_interpolator[["SmoothedPrecipitation"]]) <-
    attributes(stars_interpolator[["Precipitation"]])

  stars_interpolator[["SmoothedTemperatureRange"]] <-
    .temporalSmoothing(
      t(stars_interpolator[["MaxTemperature"]] - stars_interpolator[["MinTemperature"]]),
      params$St_TemperatureRange,
      FALSE
    ) |> t()
  colnames(stars_interpolator[["SmoothedTemperatureRange"]]) <-
    colnames(stars_interpolator[["MinTemperature"]])
  attributes(stars_interpolator[["SmoothedTemperatureRange"]]) <-
    attributes(stars_interpolator[["MinTemperature"]])

  # update initial Rp in params
  .verbosity_control(
    cli::cli_ul("Updating intial_Rp parameter with the actual stations mean distance..."),
    verbose
  )

  # We need to pass the Rp in the same units the coordinates are,
  # because the C++ methods calculate distances with the points as
  # planar/cartesian distances. i.e. If Rp are in meters (which happens
  # if we have a CRS setted), but coordinates are lat/long, C++ methods will
  # calculate distance with points wrongly (as coordinates are in degrees)
  # For this, we extract the coordinates and pass unique. This ensures the
  # CRS is lost, so distances are calculated in the arbitrary units (the
  # same as coordinates).
  params$initial_Rp <- stations |>
    sf::st_distance() |>
    as.numeric() |>
    mean(na.rm = TRUE)

  ### We cannot give a warning now, because we don't know the units a priori
  ### (Unless we recalculate the distance again, this time with units)
  # give a warning if mean distance is bigger than 100 km
  # if (params$initial_Rp > 100000) {
  #   cli::cli_alert_danger(
  #     "initial_Rp value (mean distance between meteorological stations) is {.strong {round(params$initial_Rp / 1000, 2)}km}"
  #   )
  #   cli::cli_alert_info(c(
  #     "Please check the meteorological data. ",
  #     "Interpolation results can be affected when stations are scattered and with great distances between them."
  #   ))
  # }

  # set the params as an attribute of stars_interpolator
  attr(stars_interpolator, "params") <- params

  # return the interpolator
  .verbosity_control(
    cli::cli_alert_success("Interpolator created."),
    verbose
  )
  return(stars_interpolator)
}

# write the interpolator in the NetCDF-CF standard
# https://cfconventions.org/cf-conventions/cf-conventions.html
#' Write the interpolator object
#'
#' Write the interpolator object to a file
#'
#' This function writes the interpolator object created with
#' \code{\link{create_meteo_interpolator}} in a NetCDF-CF standard
#' compliant format, as specified in
#' https://cfconventions.org/cf-conventions/cf-conventions.html
#'
#' @param interpolator meteoland interpolator object, as created by
#' \code{\link{create_meteo_interpolator}}
#' @param filename file name for the interpolator nc file
#' @param .overwrite logical indicating if the file should be overwritten if it
#' already exists
#' @param .verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return invisible interpolator object, to allow using this function as a
#' step in a pipe
#' @seealso Other interpolator functions: \code{\link{add_topo}()},
#' \code{\link{create_meteo_interpolator}()},
#' \code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
#' \code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()}
#'
#' @examples
#'
#' \donttest{
#' # example interpolator
#' data(meteoland_interpolator_example)
#'
#' # temporal folder
#' tmp_dir <- tempdir()
#'
#' # write interpolator
#' write_interpolator(
#'   meteoland_interpolator_example,
#'   file.path(tmp_dir, "meteoland_interpolator_example.nc"),
#'   .overwrite = TRUE
#' )
#'
#' # check file exists
#' file.exists(file.path(tmp_dir, "meteoland_interpolator_example.nc"))
#'
#' # read it again
#' read_interpolator(file.path(tmp_dir, "meteoland_interpolator_example.nc"))
#' }
#' 
#' @author 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#'
#' @export write_interpolator
write_interpolator <- function(
  interpolator, filename,
  .overwrite = FALSE,
  .verbose = getOption("meteoland_verbosity", TRUE)
) {
  # debug
  # browser()

  # assertions
  assertthat::assert_that(assertthat::is.string(filename))
  assertthat::assert_that(is.logical(.overwrite))


  ## overwrite
  if (.overwrite) {
    if (file.exists(filename)) {
      file.remove(filename)
    }
  }

  assertthat::assert_that(
    !file.exists(filename),
    msg = paste0(
      filename,
      " file already exists. Please provide another filename or set .overwrite to TRUE"
    )
  )

  ## prepared data
  prepared_data_list <- names(interpolator) |>
    purrr::map(~ dplyr::as_tibble(interpolator[[.x]])) |>
    # purrr::map(~ purrr::set_names(.x, interpolator[["stationID"]][1,])) |>
    purrr::set_names(names(interpolator))
  # remove stationID because we dont need it in the nc
  # prepared_data_list$stationID <- NULL

  # and now the units, is important
  prepared_data_units <- names(prepared_data_list) |>
    purrr::map(~ switch(
      .x,
      Temperature = 'celsius', MinTemperature = 'celsius', MaxTemperature = 'celsius',
      RelativeHumidity = 'percent', Precipitation = 'mm',
      Radiation = 'MJ/m2', WindDirection = 'degree', WindSpeed = 'm/s',
      elevation = 'm', aspect = 'degree', slope = 'degree',
      SmoothedPrecipitation = 'mm', SmoothedTemperatureRange = 'celsius'
    )) |>
    purrr::set_names(names(prepared_data_list))



  interpolator_attributes <- get_interpolation_params(interpolator)
  interpolator_attributes[["debug"]] <-
    dplyr::if_else(interpolator_attributes[["debug"]], 1, 0)

  ## write logic
  cf_conventions_web <- "https://cfconventions.org/cf-conventions/cf-conventions.html"
  .verbosity_control(
    cli::cli_alert_info("Creating nc file following the NetCDF-CF conventions {.url {cf_conventions_web}}"),
    .verbose
  )

  ### TODO add correct units
  list(prepared_data_list, names(prepared_data_list), prepared_data_units) |>
    purrr::pwalk(
      ~ ncdfgeom::write_timeseries_dsg(
        nc_file = filename,
        instance_names = names(prepared_data_list[[1]]),
        lats = sf::st_coordinates(
          stars::st_get_dimension_values(interpolator, "station")
        )[,2],
        lons = sf::st_coordinates(
          stars::st_get_dimension_values(interpolator, "station")
        )[,1],
        times = stars::st_get_dimension_values(interpolator, 'date'),
        data = as.data.frame(..1),
        alts = as.numeric(prepared_data_list$elevation[1,]),
        data_unit = ..3,
        data_metadata = list(name = ..2, long_name = ..2),
        time_units = "days since 1970-01-01 00:00:00",
        coordvar_long_names = list(
          instance = "station", time = "date",
          lat = "latitude", lon = "longitude", alt = "elevation"
        ),
        attributes = c(
          interpolator_attributes,
          list(
            title = "meteoland interpolator",
            provider_name = "meteoland R package",
            crs = sf::st_crs(interpolator)$wkt
          )
        ),
        add_to_existing = TRUE
      )
    )

  .verbosity_control(
    cli::cli_alert_info("Adding spatial info to nc file"),
    .verbose
  )
  filename <- ncdfgeom::write_geometry(
    nc_file = filename,
    geom_data = sf::st_as_sf(stars::st_get_dimension_values(interpolator, "station")),
    variables = c("instance_name", "time", "lat", "lon", names(prepared_data_list))
  )

  .verbosity_control(
    cli::cli_alert_success("Done"),
    .verbose
  )
  return(invisible(interpolator))
}

#' Read interpolator files
#'
#' Read interpolator files created with \code{\link{write_interpolator}}
#'
#' This function takes the file name of the nc file storing an interpolator
#' object and load it into the work environment
#'
#' @param filename interpolator file name
#' @return an interpolator (stars) object
#' @seealso Other interpolator functions: \code{\link{add_topo}()},
#' \code{\link{create_meteo_interpolator}()},
#' \code{\link{get_interpolation_params}()},
#' \code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()},
#' \code{\link{write_interpolator}()}
#'
#' @examples
#'
#' \donttest{
#' # example interpolator
#' data(meteoland_interpolator_example)
#'
#' # temporal folder
#' tmp_dir <- tempdir()
#'
#' # write interpolator
#' write_interpolator(
#'   meteoland_interpolator_example,
#'   file.path(tmp_dir, "meteoland_interpolator_example.nc")
#' )
#'
#' # check file exists
#' file.exists(file.path(tmp_dir, "meteoland_interpolator_example.nc"))
#'
#' # read it again
#' read_interpolator(file.path(tmp_dir, "meteoland_interpolator_example.nc"))
#' }
#'
#' @author 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' @export read_interpolator
read_interpolator <- function(filename) {
  # debug
  # browser()

  # get the attributes
  interpolation_attributes <- ncmeta::nc_meta(filename)$attribute |>
    dplyr::filter(
      .data$variable == "NC_GLOBAL",
      .data$name %in% names(defaultInterpolationParams())
    ) |>
    dplyr::pull(.data$value)

  # Check if attributes exist, if not is not an interpolator
  if (length(interpolation_attributes) < 1) {
    cli::cli_abort(c(
      "{.file {filename}} is not a meteoland interpolator file.",
      "x" = "No interpolation params stored in the file"
    ))
  }

  # remember to convert debug value to logical
  interpolation_attributes$debug <- as.logical(interpolation_attributes$debug)

  # get the data from the nc file
  ts_data <- ncdfgeom::read_timeseries_dsg(filename)

  # get the geometries
  wkt_crs <- ncmeta::nc_meta(filename)$attribute |>
    dplyr::filter(.data$variable == "NC_GLOBAL", .data$name == 'crs') |>
    dplyr::pull(.data$value) |>
    as.character()
  geom_data <- ncdfgeom::read_geometry(filename) |>
    sf::st_transform(crs = sf::st_crs(wkt_crs))

  # get the dates
  dates <- ts_data$time

  # build the interpolator object
  interpolator_dims <- stars::st_dimensions(
    date = dates,
    station = sf::st_geometry(geom_data)
  )

  .array_creation_helper <- function(dataframe, interpolator_dims, dates) {
    array_data <- dataframe |>
      as.matrix() |>
      array(dim = dim(interpolator_dims))

    colnames(array_data) <- names(dataframe)
    rownames(array_data) <- as.character(dates)
    return(array_data)
  }

  stars_interpolator <-
    ts_data$data_frames |>
    purrr::map(
      .array_creation_helper,
      interpolator_dims = interpolator_dims, dates = dates
    ) |>
    stars::st_as_stars(dimensions = interpolator_dims)

  attr(stars_interpolator, 'params') <- interpolation_attributes

  return(stars_interpolator)

}

#' Calibration and validation of interpolation procedures
#'
#' Calibration and validation of interpolation procedures
#'
#' Function \code{interpolator_calibration} determines optimal interpolation
#' parameters \code{"N"} and \code{"alpha"} for a given meteorological
#' variable. Optimization is done by minimizing mean absolute error ("MAE")
#' (Thornton \emph{et al.} 1997). Function
#' \code{interpolation_cross_validation} calculates average mean absolute
#' errors ("MAE") for the prediction period of the interpolator object. In both
#' calibration and cross validation procedures, predictions for each
#' meteorological station are made using a \emph{leave-one-out} procedure (i.e.
#' after excluding the station from the predictive set).
#'
#' @param interpolator A meteoland interpolator object, as created by
#' \code{\link{create_meteo_interpolator}}
#' @param stations A vector with the stations (numeric for station indexes or
#' character for stations id) to be used to calculate \code{"MAE"}. All
#' stations with data are included in the training set but predictive
#' \code{"MAE"} are calculated for the stations subset indicated in
#' \code{stations} param only. If \code{NULL} all stations are used in the
#' predictive \code{"MAE"} calculation.
#' @param variable A string indicating the meteorological variable for which
#' interpolation parameters \code{"N"} and \code{"alpha"} will be calibrated.
#' Accepted values are:
#'   \itemize{
#'     \item{\code{MinTemperature} (kernel for minimum temperature)}
#'     \item{\code{MaxTemperature} (kernel for maximum temperature)}
#'     \item{\code{DewTemperature} (kernel for dew-temperature (i.e. relative humidity))}
#'     \item{\code{Precipitation} (to calibrate the same
#'           kernel for both precipitation events and regression of precipitation amounts;
#'           not recommended)}
#'     \item{\code{PrecipitationAmount} (kernel for regression of precipitation amounts)}
#'     \item{\code{PrecipitationEvent} (kernel for precipitation events)}
#'   }
#' @param N_seq Numeric vector with \code{"N"} values to be tested
#' @param alpha_seq Numeric vector with \code{"alpha"}
#' @param update_interpolation_params Logical indicating if the interpolator
#' object must be updated with the calculated parameters. Default to FALSE
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return If \code{update_interpolation_params} is FALSE (default),
#' \code{interpolator_calibration} returns a list with the following items
#' \itemize{ \item{MAE: A numeric matrix with the mean absolute error values,
#' averaged across stations, for each combination of parameters \code{"N"} and
#' \code{"alpha"}} \item{minMAE: Minimum MAE value} \item{N: Value of parameter
#' \code{"N"} corresponding to the minimum MAE} \item{alpha: Value of parameter
#' \code{"alpha"} corresponding the the minimum MAE} \item{observed: matrix with
#' observed values (meteorological measured values)} \item{predicted: matrix with
#' interpolated values for the optimum parameter combination} } If
#' \code{update_interpolation_params} is FALSE, \code{interpolator_calibration}
#' returns the interpolator provided with the parameters updated
#'
#' @references Thornton, P.E., Running, S.W., 1999. An improved algorithm for
#' estimating incident daily solar radiation from measurements of temperature,
#' humidity, and precipitation. Agric. For. Meteorol. 93, 211–228.
#' doi:10.1016/S0168-1923(98)00126-9.
#'
#' De Caceres M, Martin-StPaul N, Turco M, Cabon A, Granda V (2018) Estimating
#' daily meteorological data and downscaling climate models over landscapes.
#' Environmental Modelling and Software 108: 186-196.
#'
#' @author 
#' Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF
#' 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' @examples
#'
#' \donttest{
#' # example interpolator
#' data("meteoland_interpolator_example")
#'
#' # As the calibration for all stations can be time consuming, we are gonna
#' # interpolate only for the first 5 stations of the 198 and only a handful
#' # of parameter combinations
#' calibration <- interpolator_calibration(
#'   meteoland_interpolator_example,
#'   stations = 1:5,
#'   variable = "MaxTemperature",
#'   N_seq = seq(10, 20, by = 5),
#'   alpha_seq = seq(8, 9, by = 0.25)
#' )
#'
#' # we can update the interpolator params directly:
#' updated_interpolator <- interpolator_calibration(
#'   meteoland_interpolator_example,
#'   stations = 1:5,
#'   update_interpolation_params = TRUE,
#'   variable = "MaxTemperature",
#'   N_seq = seq(10, 20, by = 5),
#'   alpha_seq = seq(8, 9, by = 0.25)
#' )
#'
#'
#' # check the new interpolator have the parameters updated
#' get_interpolation_params(updated_interpolator)$N_MaxTemperature
#' get_interpolation_params(updated_interpolator)$alpha_MaxTemperature
#' }
#'
#' @export interpolator_calibration
interpolator_calibration <- function(
    interpolator,
    stations = NULL,
    update_interpolation_params = FALSE,
    variable = "MinTemperature",
    N_seq = seq(10, 30, by = 5),
    alpha_seq = seq(0.25, 10, by = 0.25),
    verbose = getOption("meteoland_verbosity", TRUE)
) {

  ### assertions
  # interpolator
  assertthat::assert_that(.is_interpolator(interpolator))
  # stations
  assertthat::assert_that(
    is.null(stations) || is.numeric(stations) || is.character(stations),
    msg = "stations must be NULL or a numeric vector with the stations indexes"
  )
  # variables
  assertthat::assert_that(
    is.character(variable), msg = "variable argument must be a character"
  )
  variable <- match.arg(variable, choices = c(
    "MinTemperature", "MaxTemperature", "DewTemperature",
    "Precipitation", "PrecipitationAmount", "PrecipitationEvent"
  ))
  # seqs
  assertthat::assert_that(
    is.numeric(N_seq),
    msg = "N_seq must be a numeric vector"
  )
  assertthat::assert_that(
    is.numeric(alpha_seq),
    msg = "N_seq must be a numeric vector"
  )
  if (variable %in% c("MinTemperature", "MaxTemperature")) {
    assertthat::assert_that(
      all(N_seq >= 10),
      msg = "N_seq must start at 10 or bigger when calibrating temperature (Min or Max)"
    )
  }

  # get selected stations
  stations_sfc <- stars::st_get_dimension_values(interpolator, "station")
  stations_coords <- sf::st_coordinates(stations_sfc)
  stations_elevation <- interpolator[["elevation"]][1,]
  stations_length <- length(stations_sfc)

  # station vector filling. If null, all the indexes, if character, convert to indexes
  stations <- .station_indexes_converter(stations, interpolator)

  selected_stations <- rep(FALSE, stations_length)
  selected_stations[stations] <- TRUE

  # get parameters
  interpolator_params <- get_interpolation_params(interpolator)

  # MAE matrix
  mae_matrix <- matrix(
    0, nrow = length(N_seq), ncol = length(alpha_seq),
    dimnames = list(N_seq = as.character(N_seq), alpha_seq = as.character(alpha_seq))
  )

  # variable data
  variable_matrix <- switch(
    variable,
    "PrecipitationAmount" = t(interpolator[["Precipitation"]]),
    "PrecipitationEvent" = t(interpolator[["Precipitation"]] > 0),
    "DewTemperature" = t(.dewpointTemperatureFromRH(
      0.606 * interpolator[["MaxTemperature"]] +
        0.394 * interpolator[["MinTemperature"]],
      interpolator[["RelativeHumidity"]]
    )),
    t(interpolator[[variable]])
  )
  smoothed_matrix <- t(interpolator[["SmoothedPrecipitation"]])

  .verbosity_control(
    cli::cli_alert_info("Total number of stations: {.val {nrow(variable_matrix)}}"),
    verbose
  )

  stations_no_data <- rowSums(is.na(variable_matrix)) == ncol(variable_matrix)
  variable_matrix <- variable_matrix[!stations_no_data,]
  stations_coords <- stations_coords[!stations_no_data,]
  stations_elevation <- stations_elevation[!stations_no_data]
  selected_stations <- selected_stations[!stations_no_data]
  stations <- which(selected_stations)

  .verbosity_control(
    cli::cli_alert_info("Number of stations with available data: {.val {nrow(variable_matrix)}}"),
    verbose
  )

  # MAE assesment
  selected_variable_matrix <- variable_matrix[stations,]
  selected_stations_coords <- stations_coords[stations,]
  selected_stations_elevation <- stations_elevation[stations]

  .verbosity_control(
    cli::cli_alert_info("Number of stations used for MAE calculation: {.val {length(stations)}}"),
    verbose
  )

  .verbosity_control(
    cli::cli_alert_info("Number of parameters combinations to test: {.val {ncol(mae_matrix)*nrow(mae_matrix)}}"),
    verbose
  )

  min_mae <- 9999999.0
  min_i <- NA
  min_j <- NA

  .verbosity_control(
    cli::cli_alert_info(
      "Starting evaluation of parameter combinations for {.val {variable}}..."
    ),
    verbose
  )

  # super complicated triple loop. This could be improved but I don't know how??
  for (i in N_seq) {
    for (j in alpha_seq) {
      # create the results matrix (and add the dim names)
      predicted_variable_matrix <-
        matrix(0, nrow(selected_variable_matrix), ncol(selected_variable_matrix))
      dimnames(predicted_variable_matrix) <- dimnames(selected_variable_matrix)

      .verbosity_control(
        cli::cli_ul("Evaluating N: {.val {i}}, alpha: {.val {j}}..."),
        verbose
      )

      # and now loop for stations
      # We use 1:length(stations) instead of the stations numeric indexes, because
      # the indexes relates to the interpolator, with all the stations, but in this
      # case, the matrixes already have only the selected stations rows
      # BUT!!!, for some of the values in the functions we need the original station
      # index
      for (station in 1:length(stations)) {
        original_station_index <- stations[station]
        predicted_variable_matrix[station, ] <- switch(
          variable,
          "DewTemperature" = .interpolateTdewSeriesPoints(
            Xp = selected_stations_coords[station, 1],
            Yp = selected_stations_coords[station, 2],
            Zp = selected_stations_elevation[station],
            X = stations_coords[-original_station_index, 1],
            Y = stations_coords[-original_station_index, 2],
            Z = stations_elevation[-original_station_index],
            T = variable_matrix[-original_station_index, ],
            iniRp = interpolator_params$initial_Rp,
            alpha = j, N = i,
            iterations = interpolator_params$iterations
          ),
          "Precipitation" = .interpolatePrecipitationSeriesPoints(
            Xp = selected_stations_coords[station, 1],
            Yp = selected_stations_coords[station, 2],
            Zp = selected_stations_elevation[station],
            X = stations_coords[-original_station_index, 1],
            Y = stations_coords[-original_station_index, 2],
            Z = stations_elevation[-original_station_index],
            P = variable_matrix[-original_station_index, ],
            Psmooth = smoothed_matrix[-original_station_index, ],
            iniRp = interpolator_params$initial_Rp,
            alpha_event = j, alpha_amount = j,
            N_event = i, N_amount = i,
            iterations = interpolator_params$iterations,
            popcrit = interpolator_params$pop_crit,
            fmax = interpolator_params$f_max
          ),
          "PrecipitationAmount" = .interpolatePrecipitationSeriesPoints(
            Xp = selected_stations_coords[station, 1],
            Yp = selected_stations_coords[station, 2],
            Zp = selected_stations_elevation[station],
            X = stations_coords[-original_station_index, 1],
            Y = stations_coords[-original_station_index, 2],
            Z = stations_elevation[-original_station_index],
            P = variable_matrix[-original_station_index, ],
            Psmooth = smoothed_matrix[-original_station_index, ],
            iniRp = interpolator_params$initial_Rp,
            alpha_event = interpolator_params$alpha_PrecipitationEvent,
            alpha_amount = j,
            N_event = interpolator_params$N_PrecipitationEvent,
            N_amount = i,
            iterations = interpolator_params$iterations,
            popcrit = interpolator_params$pop_crit,
            fmax = interpolator_params$f_max
          ),
          "PrecipitationEvent" = .interpolatePrecipitationEventSeriesPoints(
            Xp = selected_stations_coords[station, 1],
            Yp = selected_stations_coords[station, 2],
            Zp = selected_stations_elevation[station],
            X = stations_coords[-original_station_index, 1],
            Y = stations_coords[-original_station_index, 2],
            Z = stations_elevation[-original_station_index],
            Pevent = variable_matrix[-original_station_index, ],
            iniRp = interpolator_params$initial_Rp,
            alpha = j,
            N = i,
            iterations = interpolator_params$iterations,
            popcrit = interpolator_params$pop_crit
          ),
          .interpolateTemperatureSeriesPoints(
            Xp = selected_stations_coords[station, 1],
            Yp = selected_stations_coords[station, 2],
            Zp = selected_stations_elevation[station],
            X = stations_coords[-original_station_index, 1],
            Y = stations_coords[-original_station_index, 2],
            Z = stations_elevation[-original_station_index],
            T = variable_matrix[-original_station_index, ],
            iniRp = interpolator_params$initial_Rp,
            alpha = j, N = i,
            iterations = interpolator_params$iterations
          )
        )
      }

      # Now we have the predicted matrix for each estation for a given combination
      # of N and alpha. Now is time to calculate MAE
      if (variable %in% c("MinTemperature", "MaxTemperature", "DewTemperature")) {
        mae_matrix[as.character(i), as.character(j)] <-
          mean(abs(predicted_variable_matrix - selected_variable_matrix), na.rm = TRUE)

        # check if NAs are generated (weights are zero for the parameter combination) and increase
        # MAE to discard this parameter combination. Remember, increased MAE has to be bigger than
        # the min_mae initial value (min_mae <- 9999999.0)
        if (any(is.na(predicted_variable_matrix))) {
          .verbosity_control(
            cli::cli_ul("N = {.val {i}} & alpha = {.val {j}} removed from calibration as they generate zero weights in regression"),
            verbose
          )
          mae_matrix[as.character(i), as.character(j)] <- 9999999.9
        }
      } else {
        denominator_1 <- 0
        denominator_2 <- 0
        numerator_1 <- 0
        numerator_2 <- 0
        for (col in 1:ncol(selected_variable_matrix)) {
          observed_prec <- selected_variable_matrix[, col]
          predicted_prec <- predicted_variable_matrix[!is.na(observed_prec), col]
          weight <- sum(!is.na(observed_prec))
          sum_observed <- sum(observed_prec, na.rm = TRUE)
          sum_predicted <- sum(predicted_prec, na.rm = TRUE)
          if (weight > 0) {
            numerator_1 = numerator_1 + abs(sum_predicted/weight - sum_observed/weight)
            denominator_1 = denominator_1 + 1
          }
        }
        for (row in 1:nrow(selected_variable_matrix)) {
          observed_prec <- selected_variable_matrix[row, ]
          predicted_prec <- predicted_variable_matrix[row, !is.na(observed_prec)]
          weight <- sum(!is.na(observed_prec))
          sum_observed <- sum(observed_prec, na.rm = TRUE)
          sum_predicted <- sum(predicted_prec, na.rm = TRUE)
          if (weight > 0) {
            numerator_2 = numerator_2 + abs(sum_predicted/weight - sum_observed/weight)
            denominator_2 = denominator_2 + 1
          }
        }

        mae_matrix[as.character(i), as.character(j)] <-
          sqrt((numerator_1/denominator_1) * (numerator_2/denominator_2))
      }

      if (mae_matrix[as.character(i), as.character(j)] < min_mae) {
        min_mae <- mae_matrix[as.character(i), as.character(j)]
        min_i <- i
        min_j <- j
        optimal_observed <- selected_variable_matrix
        optimal_predicted <- predicted_variable_matrix
      }
    }
  }

  .verbosity_control(
    cli::cli_alert_success("Calibration done: Minimum MAE: {.val {min_mae}}; N: {.val {min_i}}; alpha: {.val {min_j}}"),
    verbose
  )

  res <- list(
    MAE = mae_matrix, minMAE = min_mae,
    N = min_i, alpha = min_j,
    observed = optimal_observed,
    predicted = optimal_predicted
  )

  if (isTRUE(update_interpolation_params)) {
    params_names <- c("alpha_", "N_") |>
      paste0(variable)
    interpolator_params <- get_interpolation_params(interpolator)
    interpolator_params[[params_names[1]]] <- res[["alpha"]]
    interpolator_params[[params_names[2]]] <- res[["N"]]

    interpolator_updated <-
      set_interpolation_params(interpolator, interpolator_params)

    return(interpolator_updated)
  }

  return(res)
}

meteo_test_correct <- readRDS("meteo_and_topo_test.rds")
meteo_test_no_topo <- meteo_test_correct |>
  dplyr::select(-elevation, -aspect, -slope)

test_that("create_meteo_interpolator works as expected", {
  expect_error(create_meteo_interpolator(meteo_test_no_topo), "elevation")
  expect_error(
    create_meteo_interpolator(meteo_test_correct, params = "tururu"),
    "named list"
  )
  expect_warning(
    create_meteo_interpolator(
      meteo_test_correct, params = list(debig = FALSE)
    ),
    "will not be used"
  )
  expect_message(
    create_meteo_interpolator(
      meteo_test_correct, params = list(debug = FALSE)
    ),
    "are missing"
  )
  expect_identical(
    suppressWarnings(.safely_create_interpolation_params(NULL)),
    defaultInterpolationParams()
  )
  expect_identical(
    .safely_create_interpolation_params(defaultInterpolationParams()), defaultInterpolationParams()
  )
  expect_identical(
    suppressWarnings(.safely_create_interpolation_params(list(debug = FALSE))),
    defaultInterpolationParams()
  )
  expect_identical(
    .safely_create_interpolation_params(list(debug = TRUE))$wind_height, defaultInterpolationParams()$wind_height
  )
  expect_true(
    .safely_create_interpolation_params(list(debug = TRUE))$debug
  )

  params_names <- c(
    "initial_Rp", "iterations", "alpha_MinTemperature", "alpha_MaxTemperature",
    "alpha_DewTemperature", "alpha_PrecipitationEvent", "alpha_PrecipitationAmount",
    "alpha_Wind", "N_MinTemperature", "N_MaxTemperature", "N_DewTemperature",
    "N_PrecipitationEvent", "N_PrecipitationAmount", "N_Wind", "St_Precipitation",
    "St_TemperatureRange", "pop_crit", "f_max", "wind_height", 
    "wind_roughness_height", "penman_albedo", "penman_windfun","debug"
  )
  expect_named(suppressWarnings(.safely_create_interpolation_params(NULL)), params_names)
  expect_named(suppressWarnings(.safely_create_interpolation_params(list(tururu = 'tururu'))), params_names)
  expect_named(suppressWarnings(.safely_create_interpolation_params(list(iterations = 25))), params_names)

  meteo_test_correct_elevation_nas <- meteo_test_correct
  meteo_test_correct_elevation_nas[1:5, "elevation"] <- NA
  meteo_test_correct_elevation_all_nas <- meteo_test_correct |>
    dplyr::mutate(elevation = NA_real_)
  expect_warning(
    create_meteo_interpolator(
      meteo_test_correct_elevation_nas, params = defaultInterpolationParams()
    ),
    "lack values for elevation"
  )
  expect_error(
    suppressWarnings(create_meteo_interpolator(
      meteo_test_correct_elevation_all_nas,
      params = defaultInterpolationParams()
    )),
    "No elevation values"
  )
  expect_warning(
    interpolator_test <- create_meteo_interpolator(meteo_test_correct),
    "No interpolation parameters"
  )

  expect_false(is.null(dimnames(interpolator_test[[1]])[[1]]))
  expect_false(is.null(dimnames(interpolator_test[[1]])[[2]]))

  # check that aspect and slope are zeroes
  expect_true(all(interpolator_test[["aspect"]] == 0))
  expect_true(all(interpolator_test[["slope"]] == 0))

  # everything should be ok, no error (hence the NA)
  expect_error(.is_interpolator(interpolator_test), NA)

  # test data
  interpolator_test_no_dimensions <- interpolator_test |>
    stars::st_set_dimensions('date', names = 'tururu')
  interpolator_test_no_attr <- interpolator_test
  attr(interpolator_test_no_attr, "params") <- NULL
  interpolator_test_no_meteo_names <- interpolator_test
  temp_index <-
    which(names(interpolator_test_no_meteo_names) == 'MinTemperature')
  names(interpolator_test_no_meteo_names)[temp_index] <- "tururu"
  interpolator_test_no_topo_names <- interpolator_test
  elev_index <-
    which(names(interpolator_test_no_meteo_names) == 'elevation')
  names(interpolator_test_no_topo_names)[elev_index] <- "tururu"

  expect_error(
    .is_interpolator(interpolator_test_no_dimensions),
    "missing the correct dimensions"
  )
  expect_error(
    .is_interpolator(interpolator_test_no_attr),
    "missing the interpolation parameters"
  )
  expect_error(
    .is_interpolator(interpolator_test_no_meteo_names),
    "comply"
  )
  expect_error(
    .is_interpolator(interpolator_test_no_topo_names),
    "comply"
  )

  ## TODO
  # tests that interpolator has all names when variables are missing from
  # meteo (interpolator variables are created filled with NAs)
  meteo_test_no_relative_humidity <- meteo_test_correct |>
    dplyr::select(!dplyr::contains("RelativeHumidity"))
  meteo_test_no_precipitation <- meteo_test_correct |>
    dplyr::select(!dplyr::contains("Precipitation"))

  expect_warning(
    interpolator_test_no_relative_humidity <-
      create_meteo_interpolator(meteo_test_no_relative_humidity),
    "No interpolation parameters"
  )
  expect_named(
    interpolator_test_no_relative_humidity, names(interpolator_test)
  )
  expect_true(
    all(is.na(interpolator_test_no_relative_humidity[["RelativeHumidity"]]))
  )
  expect_identical(
    interpolator_test_no_relative_humidity[["MinTemperature"]],
    interpolator_test[["MinTemperature"]]
  )

  expect_warning(
    interpolator_test_no_precipitation <-
      create_meteo_interpolator(meteo_test_no_precipitation),
    "No interpolation parameters"
  )
  expect_named(
    interpolator_test_no_precipitation, names(interpolator_test)
  )
  expect_true(
    all(is.na(interpolator_test_no_precipitation[["Precipitation"]]))
  )
  expect_true(
    all(is.na(interpolator_test_no_precipitation[["SmoothedPrecipitation"]]))
  )
  expect_identical(
    interpolator_test_no_precipitation[["MinTemperature"]],
    interpolator_test[["MinTemperature"]]
  )

})

suppressWarnings(
  interpolator_test <- create_meteo_interpolator(meteo_test_correct)
)

test_that("defaultInterpolationParams work as expected", {
  interpolation_param_names <- c(
    "initial_Rp", "iterations", "alpha_MinTemperature", "alpha_MaxTemperature",
    "alpha_DewTemperature", "alpha_PrecipitationEvent", "alpha_PrecipitationAmount",
    "alpha_Wind", "N_MinTemperature", "N_MaxTemperature", "N_DewTemperature",
    "N_PrecipitationEvent", "N_PrecipitationAmount", "N_Wind", "St_Precipitation",
    "St_TemperatureRange", "pop_crit", "f_max", "wind_height", "wind_roughness_height", 
    "penman_albedo", "penman_windfun","debug"
  )
  expect_type(defaultInterpolationParams(), "list")
  expect_length(defaultInterpolationParams(), length(interpolation_param_names))
  expect_named(defaultInterpolationParams(), interpolation_param_names)
})

test_that("get and set params methods work as expected", {

  # get
  expect_identical(get_interpolation_params(interpolator_test), attr(interpolator_test, 'params'))

  # set
  expect_warning(set_interpolation_params(interpolator_test), "using defaults")
  expect_identical(
    get_interpolation_params(suppressWarnings(set_interpolation_params(interpolator_test))),
    defaultInterpolationParams()
  )
  expect_warning(set_interpolation_params(interpolator_test, list(debig = FALSE)), "will not be used")
  expect_message(set_interpolation_params(interpolator_test, list(debug = TRUE)), "using default")
  expect_s3_class(set_interpolation_params(interpolator_test, list(debug = TRUE)), "stars")
  expect_true(
    get_interpolation_params(set_interpolation_params(interpolator_test, list(debug = TRUE)))$debug
  )
  interpolator_updated <- set_interpolation_params(interpolator_test, list(N_MinTemperature = 99))
  interpolator_updated_2 <- set_interpolation_params(interpolator_updated, list(N_MaxTemperature = 125))
  expect_true(get_interpolation_params(interpolator_updated)$N_MinTemperature == 99)
  expect_true(
    get_interpolation_params(interpolator_updated_2)$N_MinTemperature == 99 &&
      get_interpolation_params(interpolator_updated_2)$N_MaxTemperature == 125
  )
  expect_false(get_interpolation_params(interpolator_updated)$N_MaxTemperature == 125)
})

test_that("write and read interpolators works as expected", {
  tmp_dir <- tempdir()
  tmp_file <- paste0(tmp_dir, "/test_interpolator.nc")

  expect_equal(
    write_interpolator(interpolator_test, tmp_file),
    interpolator_test
  )
  expect_equal(
    interpolator_write <-
      write_interpolator(interpolator_test, tmp_file, .overwrite = TRUE),
    interpolator_test
  )
  expect_error(write_interpolator(interpolator_test, tmp_file), NULL)
  expect_equal(
    interpolator_read <- read_interpolator(tmp_file),
    interpolator_test
  )
  expect_identical(
    interpolator_read,
    interpolator_write
  )
  expect_error(
    read_interpolator(system.file("nc/reduced.nc", package = "stars")),
    "not a meteoland interpolator"
  )

  expect_error(.is_interpolator(interpolator_write), NA)
  expect_error(.is_interpolator(interpolator_read), NA)
})

#' Default interpolation parameters
#' 
#' Returns a list with the default parameterization for interpolation. Most
#' parameter values are set according to Thornton et al. (1997).
#' 
#' 
#' @return A list with the following items (default values in brackets):
#' \itemize{ \item\code{initial_Rp [= 140000]}: Initial truncation radius.
#' \item\code{iterations [= 3]}: Number of station density iterations.
#' \item\code{alpha_MinTemperature [= 3.0]}: Gaussian shape parameter for
#' minimum temperature. \item\code{alpha_MaxTemperature [= 3.0]}: Gaussian
#' shape parameter for maximum temperature. \item\code{alpha_DewTemperature [=
#' 3.0]}: Gaussian shape parameter for dew-point temperature.
#' \item\code{alpha_PrecipitationEvent [= 5.0]}: Gaussian shape parameter for
#' precipitation events. \item\code{alpha_PrecipitationAmount [= 5.0]}:
#' Gaussian shape parameter for the regression of precipitation amounts.
#' \item\code{alpha_Wind [= 3.0]}: Gaussian shape parameter for wind.
#' \item\code{N_MinTemperature [= 30]}: Average number of stations with
#' non-zero weights for minimum temperature. \item\code{N_MaxTemperature [=
#' 30]}: Average number of stations with non-zero weights for maximum
#' temperature. \item\code{N_DewTemperature [= 30]}: Average number of stations
#' with non-zero weights for dew-point temperature.
#' \item\code{N_PrecipitationEvent [= 5]}: Average number of stations with
#' non-zero weights for precipitation events. \item\code{N_PrecipitationAmount
#' [= 20]}: Average number of stations with non-zero weights for the regression
#' of precipitation amounts. \item\code{N_Wind [= 2]}: Average number of
#' stations with non-zero weights for wind. \item\code{St_Precipitation [= 5]}:
#' Number of days for the temporal smoothing of precipitation.
#' \item\code{St_TemperatureRange [= 15]}: Number of days for the temporal
#' smoothing of temperature range. \item\code{pop_crit [= 0.50]}: Critical
#' precipitation occurrence parameter. \item\code{f_max [= 0.6]}: Maximum value
#' for precipitation regression extrapolations (0.6 equals to a maximum of 4
#' times extrapolation). \item\code{wind_height [= 10]}: Wind measurement
#' height (in m). \item\code{wind_roughness_height [= 0.001]}: Wind roughness
#' height (in m), for PET calculations. \item\code{penman_albedo [= 0.25]}: Albedo
#' for PET calculations. \item\code{penman_windfun [= "1956"]}: Wind speed function version, 
#' either "1948" or "1956", for PET calculation. \item\code{debug [= FALSE]}: Boolean flag to show extra
#' console output. }
#' @author Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
#' @seealso \code{\link{interpolate_data}}
#' @references Thornton, P.E., Running, S.W., White, M. A., 1997. Generating
#' surfaces of daily meteorological variables over large regions of complex
#' terrain. J. Hydrol. 190, 214–251. doi:10.1016/S0022-1694(96)03128-9.
#' 
#' De Caceres M, Martin-StPaul N, Turco M, Cabon A, Granda V (2018) Estimating
#' daily meteorological data and downscaling climate models over landscapes.
#' Environmental Modelling and Software 108: 186-196.
#' @export
defaultInterpolationParams<-function() {
  return(list(
    initial_Rp = 140000, #Initial Rp value (in meters)
    iterations = 3, #Number of iterations in reestimation of Rp
    alpha_MinTemperature = 3.0, #Parameter of the Gaussian filter for temperature
    alpha_MaxTemperature = 3.0, #Parameter of the Gaussian filter for temperature
    alpha_DewTemperature = 3.0, #Parameter of the Gaussian filter for temperature
    alpha_PrecipitationEvent = 5.0, #Parameter of the Gaussian filter for precipitation
    alpha_PrecipitationAmount = 5.0, #Parameter of the Gaussian filter for precipitation
    alpha_Wind = 3.0, #Parameter of the Gaussian filter for precipitation
    N_MinTemperature = 30, #Number of required values for temperature
    N_MaxTemperature = 30, #Number of required values for temperature
    N_DewTemperature = 30, #Number of required values for temperature
    N_PrecipitationEvent = 5, #Number of required values for precipitation event
    N_PrecipitationAmount = 20, #Number of required values for precipitation amount
    N_Wind = 2, #Number of required values for precipitation
    St_Precipitation = 5, #Integer with the number of days for precipitation smoothing
    St_TemperatureRange = 15, #Integer with the number of days for smoothing of temperature range
    pop_crit = 0.50, #Critical value for probability of precipitation
    f_max = 0.60, #Parameter for corrections of precipitation with elevation (f_max<1)
    wind_height = 10, #Wind height (in m)
    wind_roughness_height = 0.001, # Wind roughness height (in m) for PET calculation.
    penman_albedo = 0.25, # Albedo for PET calculation.
    penman_windfun = "1956", # Wind speed function version, either "1948" or "1956", for PET calculation.
    debug = FALSE
    ))
}

#' Check if meteo data has the mandatory variables
#'
#' Check if meteo data has the mandatory variables
#'
#' This function ensures that meteo object have the mandatory variables, and is
#' used in combination with \code{\link[assertthat]{on_failure}} to produce a
#' meaningful and human readable error when no mandatory variables are present.
#'
#' @param meteo meteo object
#'
#' @return TRUE if mandatory variables are present, FALSE otherwise
#'
#' @family Argument checks
#' @noRd
has_meteo_names <- function(meteo) {
  # meteo names
  mandatory_meteo_names <- c("MinTemperature", "MaxTemperature")

  all(mandatory_meteo_names %in% names(meteo))
}

assertthat::on_failure(has_meteo_names) <- function(call, env) {
  paste0(
    "Names found in ", deparse(call$meteo), " don't comply with the required names:\n",
    deparse(call$meteo),
    " should have the following meteorology variables:\n",
    "  - MinTemperature ***\n", "  - MaxTemperature ***\n",
    "  - Precipitation\n", "  - RelativeHumidity\n",
    "  - Radiation\n", "  - WindSpeed\n", "  - WindDirection\n",
    "\n ***: mandatory variables"
  )
}

#' Check if meteo data has the mandatory topography variables
#'
#' Check if meteo data has the mandatory topography variables
#'
#' This function ensures that meteo object have the mandatory topography names,
#' and is used in combination with \code{\link[assertthat]{on_failure}} to produce
#' a meaningful and human readable error when no mandatory variables are present.
#'
#' @param meteo meteo object
#'
#' @return TRUE if mandatory variables are present, FALSE otherwise
#'
#' @family Argument checks
#' @noRd
has_topo_names <- function(meteo) {
  # topography names
  mandatory_topo_names <- c("elevation")
  # check
  all(mandatory_topo_names %in% names(meteo))
}

assertthat::on_failure(has_topo_names) <- function(call, env) {
  paste0(
    "Names found in ", deparse(call$meteo), " don't comply with the required names:\n",
    deparse(call$meteo),
    " should have the following topography variables:\n",
    "  - elevation ***\n",
    "  - aspect\n", "  - slope\n",
    "\n ***: mandatory variables\n"
  )
}

#' Check if meteo data has unique stations ID
#'
#' Check if meteo data has unique stations ID
#'
#' This function ensures that meteo object have unique stations ID with only
#' one geometry per station code
#'
#' @param meteo meteo object
#'
#' @return TRUE if station IDs are unique, FALSE otherwise
#'
#' @family Argument checks
#' @noRd
has_unique_ids <- function(meteo) {
  distinct_rows <- meteo |>
    dplyr::as_tibble() |>
    dplyr::select("stationID", attr(meteo, "sf_column")) |>
    dplyr::distinct()

  nrow(distinct_rows) == length(unique(distinct_rows[["stationID"]]))

}

assertthat::on_failure(has_unique_ids) <- function(call, env) {

  duplicated_stations <- env$meteo |>
    dplyr::select("stationID") |>
    dplyr::distinct() |>
    dplyr::filter(duplicated(.data$stationID)) |>
    dplyr::pull("stationID")

  paste0(
    "There are more geometries in the data than unique station IDs. ",
    "Duplicated stations IDs are:\n",
    paste(duplicated_stations, collapse = '\n')
  )
}


#' Checks for meteo
#'
#' Checks for meteo
#'
#' This function checks that meteo object has everything is needed to create the
#' interpolator. Checks done include being a sf POINT object, correct variables,
#' numeric variables, dates and meteo stations names.
#'
#' @param meteo meteo object
#'
#' @family Argument checks
#'
#' @return Informative error when any check fails, invisible TRUE otherwise
#' @noRd
has_meteo <- function(meteo) {
  # sf object
  assertthat::assert_that(inherits(meteo, 'sf'), msg = "meteo must be an sf object")
  assertthat::assert_that(all(sf::st_is(meteo, 'POINT')), msg = "meteo station geometries must be POINT")

  # names
  assertthat::assert_that(has_meteo_names(meteo))

  # number when it matters
  assertthat::assert_that(
    is.numeric(meteo[["MinTemperature"]]),
    is.numeric(meteo[["MaxTemperature"]])
  )

  # dates
  assertthat::assert_that(
    "dates" %in% names(meteo),
    msg = "meteo must have a variable called dates with the interpolation time span"
  )
  assertthat::assert_that(
    assertthat::is.date(meteo$dates) || assertthat::is.time(meteo$dates),
    msg = "dates variable must be a date or time class"
  )

  # stations
  assertthat::assert_that(
    "stationID" %in% names(meteo),
    msg = "meteo must have a stationID variable identifying the meteorological stations"
  )

  assertthat::assert_that(
    has_unique_ids(meteo)
  )
}

#' Checks for topo
#'
#' Checks for topo
#'
#' This function checks that topo object has everything is needed to create the
#' interpolator. Checks done include being a dataframe, correct variables,
#' numeric variables, and stations names.
#'
#' @param topo topo object
#'
#' @family Argument checks
#'
#' @return Informative error when any check fails, invisible TRUE otherwise
#' @noRd
has_topo <- function(topo) {

  # dataframe or similar with station_id or geometry
  assertthat::assert_that(
    inherits(topo, 'data.frame'), msg = "topo must be a data.frame/tibble"
  )

  # var names
  assertthat::assert_that(has_topo_names(topo))

  # numeric what it has to be numeric
  assertthat::assert_that(is.numeric(topo[["elevation"]]))

  assertthat::assert_that(
    "stationID" %in% names(topo),
    msg = "topo must have a stationID variable identifying the meteorological stations"
  )
}

#' Check class of provided spatial data
#'
#' Check class of provided spatial data
#'
#' This function checks the class of the provided spatial data is in the allowed
#' classes (stars and sf)
#'
#' @param spatial_data spatial data provided
#'
#' @return TRUE if spatial data is a stars or sf object, FALSE otherwise
#'
#' @noRd
.is_spatial_data <- function(spatial_data) {
  inherits(spatial_data, c('sf', 'stars'))
}


assertthat::on_failure(.is_spatial_data) <- function(call, env) {
  paste0(
    "Spatial data provided (", deparse(call$spatial_data), ")\n",
    "must be an sf object (for points) or an stars object (for raster and grids).\n",
    "Other spatial classes (from sp, terra... packages) must be converted first."
  )
}

#' Check if spatial data stars is a raster
#'
#' Check if spatial data stars is a raster
#'
#' This function checks if the provided stars object is a raster, not a vector
#' data cube (which is not allowed)
#'
#' @param spatial_data stars spatial data provided
#'
#' @return TRUE when spatial data is a raster, FALSE when is a vector data cube
#'
#' @noRd
.is_raster <- function(spatial_data) {
  spatial_dimension <-
    purrr::map_lgl(sf::st_coordinates(spatial_data), inherits, what = 'sfc') |>
    any()
  !spatial_dimension
}

assertthat::on_failure(.is_raster) <- function(call, env) {
  paste0(
    "stars object provided (", deparse(call$spatial_data), ")\n",
    "is a vector data cube. Please convert to sf points data or raster data\n",
    "before the interpolatation."
  )
}

#' Assertions to check if the object provided is an interpolator
#'
#' Assertions to check if the object provided is an interpolator
#'
#' This function checks all the mandatory characteristics of an interpolator
#' object (dimensions, parameters, variables).
#'
#' @param interpolator meteoland interpolator object, as created by
#'   \code{\link{create_meteo_interpolator}}
#'
#' @return invisible TRUE if the object is a meteoland complying interpolator,
#'   an informative error otherwise
#'
#' @noRd
.is_interpolator <- function(interpolator) {
  has_params <- !is.null(get_interpolation_params(interpolator))
  has_correct_dimensions <-
    all(c('date', 'station') %in% names(stars::st_dimensions(interpolator)))

  assertthat::assert_that(
    has_params,
    msg = "interpolator object is missing the interpolation parameters attribute."
  )
  assertthat::assert_that(
    has_correct_dimensions,
    msg = "interpolator object is missing the correct dimensions (date and station)"
  )
  assertthat::assert_that(has_meteo_names(interpolator))
  assertthat::assert_that(has_topo_names(interpolator))
}

test_that(".stars2sf works as intended", {
  expect_error(.stars2sf("wrong_object"), "stars class")
  expect_error(.stars2sf(raster_to_interpolate_example[c("slope", "aspect")]), "No elevation")
  expect_s3_class((res <- .stars2sf(raster_to_interpolate_example)), "sf")
  expect_named(res, c("elevation", "slope", "aspect", "geometry"))
  expect_identical(
    as.integer(nrow(raster_to_interpolate_example) * ncol(raster_to_interpolate_example)),
    nrow(res)
  )
  # geometries in both, stars and sf, must be the same
  expect_identical(
    as.numeric(sf::st_coordinates(res)[,1]),
    as.numeric(sf::st_coordinates(raster_to_interpolate_example)[,1])
  )
  expect_identical(
    as.numeric(sf::st_coordinates(res)[,2]),
    as.numeric(sf::st_coordinates(raster_to_interpolate_example)[,2])
  )
})

test_that(".is_spatial_data, .is_raster works as intended", {
  # .is_spatial_data
  expect_false(.is_spatial_data(25))
  expect_false(.is_spatial_data("25"))
  # expect_false(.is_spatial_data(terra::rast(raster_to_interpolate_example)))
  # expect_false(.is_spatial_data(sf::as_Spatial(points_to_interpolate_example)))
  expect_true(.is_spatial_data(points_to_interpolate_example))
  expect_true(.is_spatial_data(raster_to_interpolate_example))
  # .is_raster
  expect_false(.is_raster(meteoland_interpolator_example))
  expect_true(.is_raster(raster_to_interpolate_example))
})

test_that("interpolation process works as intended", {
  # interpolation varaibles names
  interpolation_var_names <- c(
    "MeanTemperature", "MinTemperature", "MaxTemperature", "Precipitation",
    "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
    "Radiation", "WindSpeed", "WindDirection", "PET"
  )

  # argument errors
  expect_error(
    interpolate_data(25, meteoland_interpolator_example), "must be"
  )
  expect_error(
    interpolate_data(meteoland_interpolator_example, meteoland_interpolator_example), "vector data cube"
  )
  expect_error(
    interpolate_data(sf::st_transform(raster_to_interpolate_example, crs = 4326), meteoland_interpolator_example),
    "Curvilinear grids"
  )
  expect_error(
    interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example, dates = 'tururu'),
    "dates object provided"
  )
  expect_error(
    interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example, variables = 'tururu'),
    "variables argument must be"
  )
  expect_error(interpolate_data(raster_to_interpolate_example, raster_to_interpolate_example), NULL)
  expect_warning(
    interpolate_data(dplyr::select(points_to_interpolate_example, -slope), meteoland_interpolator_example),
    "not mandatory"
  )

  # points
  expect_s3_class(
    (points_res <- interpolate_data(points_to_interpolate_example, meteoland_interpolator_example)),
    'sf'
  )
  expect_identical(nrow(points_res), nrow(points_to_interpolate_example))
  expect_true(all(names(points_to_interpolate_example) %in% names(points_res)))
  expect_true("interpolated_data" %in% names(points_res))
  expect_true(all(
    c(interpolation_var_names, "dates", "DOY") %in%
      names(points_res |> tidyr::unnest(cols = c(interpolated_data)))
  ))
  expect_identical(sf::st_geometry(points_res), sf::st_geometry(points_to_interpolate_example))
  expect_identical(sf::st_crs(points_res), sf::st_crs(points_to_interpolate_example))
  # we expect all vars with data, as we have everything we need to interpolate/calculate any
  # variable
  expect_true(
    (points_res$interpolated_data[[1]] |> dplyr::select(where(\(x) {all(is.na(x))})) |> ncol()) < 1
  )

  # raster
  expect_s3_class(
    (raster_res <- interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example)),
    'stars'
  )
  expect_identical(dim(raster_res)[1:2], dim(raster_to_interpolate_example)[1:2])
  expect_true(all(names(raster_to_interpolate_example) %in% names(raster_res)))
  expect_true(all(interpolation_var_names %in% names(raster_res)))
  expect_identical(sf::st_crs(raster_res), sf::st_crs(raster_to_interpolate_example))
  # we expect all vars with data, as we have everything we need to interpolate/calculate any
  # variable
  purrr::map(
    purrr::set_names(names(raster_res)),
    .f = \(x) {expect_false(all(is.na(raster_res[[x]])))}
  )

  # date checks
  dates_all_ok <- as.Date(c("2022-04-24", "2022-04-25", "2022-04-26"))
  dates_some_ok <- c(dates_all_ok, "2022-05-13")
  dates_none_ok <- as.Date(c("2022-05-01", "2022-05-02"))
  expect_s3_class(
    (points_res_dates <- interpolate_data(
      points_to_interpolate_example, meteoland_interpolator_example, dates_all_ok
    )),
    'sf'
  )
  expect_warning(
    interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example, dates_some_ok),
    "Some `dates` are outside the `interpolator` date range, only dates inside will be used"
  )
  expect_error(
    interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example, dates_none_ok),
    "`dates` supplied are outside the `interpolator` date range. No possible interpolation."
  )
  expect_identical(nrow(points_res_dates), nrow(points_to_interpolate_example))
  expect_true(all(names(points_to_interpolate_example) %in% names(points_res_dates)))
  expect_true("interpolated_data" %in% names(points_res_dates))
  expect_true(all(
    c(interpolation_var_names, "dates", "DOY") %in%
      names(points_res_dates |> tidyr::unnest(cols = c(interpolated_data)))
  ))
  expect_identical(sf::st_geometry(points_res_dates), sf::st_geometry(points_to_interpolate_example))
  expect_identical(sf::st_crs(points_res_dates), sf::st_crs(points_to_interpolate_example))
  expect_identical(
    nrow(points_res_dates[["interpolated_data"]][[1]]), 3L
  )
  expect_identical(
    points_res_dates[["interpolated_data"]][[1]]$dates, dates_all_ok
  )

  points_all_ok <- points_to_interpolate_example
  points_one_bad <- points_all_ok |>
    dplyr::mutate(
      geometry = points_all_ok$geometry + c(5, rep(0, length(points_all_ok$geometry) - 1))
    ) |>
    sf::st_set_crs(4326)
  points_more_bad <- points_all_ok |>
    dplyr::mutate(
      geometry = points_all_ok$geometry + c(5, 5, 5, rep(0, length(points_all_ok$geometry) - 3))
    ) |>
    sf::st_set_crs(4326)

  expect_s3_class(interpolate_data(points_all_ok, meteoland_interpolator_example),'sf')
  expect_warning(
    interpolate_data(points_one_bad, meteoland_interpolator_example),
    'Some points are outside the convex hull'
  )
  expect_error(
    interpolate_data(points_more_bad, meteoland_interpolator_example),
    'More than 10% of the points'
  )

  # variable argument points
  expect_s3_class(
    (points_radiation_res <-
       interpolate_data(points_to_interpolate_example, meteoland_interpolator_example, variables = "Radiation")),
    'sf'
  )
  expect_identical(
    points_res$interpolated_data[[1]]$Radiation,
    points_radiation_res$interpolated_data[[1]]$Radiation
  )
  expect_identical(
    points_res$interpolated_data[[1]]$MinTemperature,
    points_radiation_res$interpolated_data[[1]]$MinTemperature
  )
  expect_identical(
    points_res$interpolated_data[[1]]$MeanRelativeHumidity,
    points_radiation_res$interpolated_data[[1]]$MeanRelativeHumidity
  )
  expect_identical(
    points_res$interpolated_data[[1]]$Precipitation,
    points_radiation_res$interpolated_data[[1]]$Precipitation
  )
  expect_true(
    all(is.na(points_radiation_res$interpolated_data[[1]]$WindDirection))
  )
  expect_true(
    all(is.na(points_radiation_res$interpolated_data[[1]]$PET))
  )

  expect_s3_class(
    (points_twovars_res <-
       interpolate_data(points_to_interpolate_example, meteoland_interpolator_example, variables = c("Precipitation", "Wind"))),
    'sf'
  )
  expect_identical(
    points_res$interpolated_data[[1]]$Precipitation,
    points_twovars_res$interpolated_data[[1]]$Precipitation
  )
  expect_identical(
    points_res$interpolated_data[[1]]$WindDirection,
    points_twovars_res$interpolated_data[[1]]$WindDirection
  )
  expect_true(
    all(is.na(points_twovars_res$interpolated_data[[1]]$MinTemperature))
  )
  expect_true(
    all(is.na(points_twovars_res$interpolated_data[[1]]$Radiation))
  )
  expect_true(
    all(is.na(points_twovars_res$interpolated_data[[1]]$MeanRelativeHumidity))
  )
  expect_true(
    all(is.na(points_twovars_res$interpolated_data[[1]]$PET))
  )

  # variables argument raster
  expect_s3_class(
    (raster_radiation_res <-
       interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example, variables = "Radiation")),
    'stars'
  )
  expect_identical(
    raster_res[["Radiation"]],
    raster_radiation_res[["Radiation"]]
  )
  expect_identical(
    raster_res[["MinTemperature"]],
    raster_radiation_res[["MinTemperature"]]
  )
  expect_identical(
    raster_res[["MeanRelativeHumidity"]],
    raster_radiation_res[["MeanRelativeHumidity"]]
  )
  expect_identical(
    raster_res[["Precipitation"]],
    raster_radiation_res[["Precipitation"]]
  )
  expect_true(
    all(is.na(raster_radiation_res[["WindDirection"]]))
  )
  expect_true(
    all(is.na(raster_radiation_res[["PET"]]))
  )

  expect_s3_class(
    (raster_twovars_res <-
       interpolate_data(raster_to_interpolate_example, meteoland_interpolator_example, variables = c("Precipitation", "Wind"))),
    'stars'
  )
  expect_identical(
    raster_res[["WindDirection"]],
    raster_twovars_res[["WindDirection"]]
  )
  expect_identical(
    raster_res[["Precipitation"]],
    raster_twovars_res[["Precipitation"]]
  )
  expect_true(
    all(is.na(raster_twovars_res[["MinTemperature"]]))
  )
  expect_true(
    all(is.na(raster_twovars_res[["MeanRelativeHumidity"]]))
  )
  expect_true(
    all(is.na(raster_twovars_res[["Radiation"]]))
  )
  expect_true(
    all(is.na(raster_twovars_res[["PET"]]))
  )

  # testing that missing variables in the interpolator behaves as they should
  #   - no relative humidity, then it is interpolated with temperatures, so the
  #     variable has data in the res
  #   - no precipitation, then precipitation is NA in the results, but radiation
  #     is interpolated as clear days, with a warning to the user
  interpolator_test_no_relative_humidity <- meteoland_interpolator_example
  interpolator_test_no_relative_humidity[["RelativeHumidity"]] <- NA_real_
  interpolator_test_no_precipitation <- meteoland_interpolator_example
  interpolator_test_no_precipitation[["Precipitation"]] <- NA_real_
  interpolator_test_no_precipitation[["SmoothedPrecipitation"]] <- NA_real_
  interpolator_test_no_precipitation[["Radiation"]] <- NA_real_

  expect_s3_class(
    points_res_no_relative_humidity <-
      interpolate_data(points_to_interpolate_example, interpolator_test_no_relative_humidity),
    "sf"
  )
  1:length(unique(points_res_no_relative_humidity$plot_id)) |>
    purrr::walk(
      .f = \(point) {
        expect_false(
          any(is.na(points_res_no_relative_humidity$interpolated_data[[point]]$MeanRelativeHumidity))
        )
        expect_false(
          any(is.na(points_res_no_relative_humidity$interpolated_data[[point]]$MinRelativeHumidity))
        )
        expect_false(
          any(is.na(points_res_no_relative_humidity$interpolated_data[[point]]$MaxRelativeHumidity))
        )
      }
    )

  expect_warning(
    points_res_no_precipitation <-
      interpolate_data(points_to_interpolate_example, interpolator_test_no_precipitation),
    "assuming clear days"
  )

  1:length(unique(points_res_no_precipitation$plot_id)) |>
    purrr::walk(
      .f = \(point) {
        expect_true(
          all(is.na(points_res_no_precipitation$interpolated_data[[point]]$Precipitation))
        )
        expect_false(
          any(is.na(points_res_no_precipitation$interpolated_data[[point]]$Radiation))
        )
      }
    )
  
  # test interpolation when using N_*Temperature < 10 and NAs are generated when
  # weights become zero.

  # Browse[1]> selected_stations_coords[station, 1]
  #       X 
  # 0.66789 

  # Browse[1]> selected_stations_coords[station, 2]
  #        Y 
  # 41.35991 
  # Browse[1]> selected_stations_elevation[station]
  #  UM 
  # 505 
  # Browse[1]> stations_elevation[-original_station_index] |> names()
  # Rp 1.13
  req_stations <- c(
    "C6", "C7", "C8", "C9", "CC", "CD", "CE", "CG", "CI", "CJ", "CL", "CP",
    "CQ", "CR", "CT", "CW", "CY", "D1", "D2", "D3", "D4", "D5", "D6", "D7",
    "D8", "D9", "DB", "DF", "DG", "DI", "DJ", "DK", "DL",
    "DN", "DO", "DP", "DQ", "H1", "J5", "KP", "MQ", "MR", "MS", "MV", "U1",
    "U2", "U3", "U4", "U6", "U7", "U9", "UA", "UB", "UE", "UF", "UG", "UH",
    "UI", "UJ", "UK", "UN", "UO", "UP", "UQ", "US", "UU",
    "UW", "UX", "UY", "V1", "V3", "V4", "V5", "V8", "VA", "VB", "VC", "VD",
    "VE", "VH", "VK", "VM", "VN", "VO", "VP", "VQ", "VS", "VU", "VV", "VX",
    "VY", "VZ", "W1", "W4", "W5", "W8", "W9", "WA", "WB",
    "WC", "WD", "WE", "WG", "WI", "WJ", "WK", "WL", "WM", "WN", "WO", "WP",
    "WQ", "WR", "WS", "WT", "WU", "WV", "WW", "WX", "WZ", "X1", "X2", "X3",
    "X4", "X5", "X6", "X7", "X8", "X9", "XA", "XB", "XC",
    "XD", "XE", "XF", "XG", "XH", "XI", "XJ", "XK", "XL", "XM", "XN", "XO",
    "XP", "XQ", "XR", "XS", "XT", "XU", "XV", "XX", "XY", "XZ", "Y4", "Y5",
    "Y6", "YA", "YB", "YC", "YD", "YE", "YF", "YG", "YH",
    "YJ", "YK", "YL", "YM", "YN", "YO", "YP", "YQ", "Z1", "Z2", "Z3", "Z5",
    "Z6", "Z7", "Z8", "Z9", "ZB", "ZC", "ZD"
  )
  interpolator_stations <- meteoland_interpolator_example[["elevation"]] |> colnames()

  low_n_interpolator <- set_interpolation_params(
    meteoland_interpolator_example[,,which(interpolator_stations %in% req_stations)],
    list(N_MinTemperature = 5, alpha_MinTemperature = 9.5, initial_Rp = 1.13644)
  )
  low_n_trigger_points <- dplyr::bind_cols(
    stars::st_get_dimension_values(meteoland_interpolator_example, "station")[65],
    meteoland_interpolator_example[["elevation"]][1,65],
    meteoland_interpolator_example[["aspect"]][1,65],
    meteoland_interpolator_example[["slope"]][1,65]
  ) |>
    purrr::set_names("geometry", "elevation", "aspect", "slope") |>
    sf::st_as_sf()

  expect_warning(
    interpolate_data(low_n_trigger_points, low_n_interpolator),
    "Could not perform temperature interpolation"
  )
})

test_that("interpolator calibration works as expected", {
  expect_error(
    interpolator_calibration("tururu"),
    "missing the interpolation parameters"
  )
  interpolator_no_params <- meteoland_interpolator_example
  attr(interpolator_no_params, "params") <- NULL
  expect_error(
    interpolator_calibration(interpolator_no_params),
    "missing the interpolation parameters"
  )
  interpolator_wrong_dims <- meteoland_interpolator_example
  dimnames(interpolator_wrong_dims) <- c('tururu', 'larara')
  expect_error(
    interpolator_calibration(interpolator_wrong_dims),
    "missing the correct dimensions"
  )
  interpolator_no_meteo_names <- meteoland_interpolator_example
  names(interpolator_no_meteo_names) <- c('tururu', names(interpolator_no_meteo_names)[-2])
  expect_error(
    interpolator_calibration(interpolator_no_meteo_names),
    "Names found in interpolator don't comply with the required names"
  )
  # N under 10 only for temps
  expect_error(
    interpolator_calibration(meteoland_interpolator_example, N_seq = c(5, 10), alpha_seq = c(9, 9.5)),
    "start at 10 or bigger"
  )
  expect_type(
    suppressWarnings(
      interpolator_calibration(
        meteoland_interpolator_example, variable = "DewTemperature",
        N_seq = c(5, 10), alpha_seq = c(9, 9.5)
      )
    ),
    'list'
  )

  expect_type(
    suppressWarnings(test_calibration <-
       interpolator_calibration(meteoland_interpolator_example, N_seq = c(10, 20), alpha_seq = c(9, 9.5))),
    'list'
  )
  expect_named(test_calibration, c("MAE", "minMAE", "N", "alpha", "observed", "predicted"))
  expect_true(is.matrix(test_calibration$MAE))
  expect_identical(dim(test_calibration$MAE), c(2L,2L))
  expect_named(dimnames(test_calibration$MAE), c("N_seq", "alpha_seq"))
  expect_true(is.numeric(test_calibration$minMAE))
  expect_length(test_calibration$minMAE, 1)
  expect_identical(min(test_calibration$MAE, na.rm = TRUE), test_calibration$minMAE)
  expect_true(is.numeric(test_calibration$N))
  expect_length(test_calibration$N, 1)
  expect_true(is.numeric(test_calibration$alpha))
  expect_length(test_calibration$alpha, 1)
  expect_true(is.matrix(test_calibration$observed))
  expect_true(is.matrix(test_calibration$predicted))
  expect_false(is.null(dimnames(test_calibration$observed)[[1]]))
  expect_false(is.null(dimnames(test_calibration$observed)[[2]]))
  expect_false(is.null(dimnames(test_calibration$predicted)[[1]]))
  expect_false(is.null(dimnames(test_calibration$predicted)[[2]]))
  expect_true("2022-04-25" %in% dimnames(test_calibration$predicted)[[2]])

  # selecting stations
  expect_type(
    (test_calibration_three_stations <- interpolator_calibration(
      meteoland_interpolator_example,
      stations = c(76, 83, 187),
      N_seq = c(10, 20), alpha_seq = c(9, 9.5))),
    'list'
  )

  expect_named(test_calibration_three_stations, c("MAE", "minMAE", "N", "alpha", "observed", "predicted"))
  expect_true(is.matrix(test_calibration_three_stations$MAE))
  expect_identical(dim(test_calibration_three_stations$MAE), c(2L,2L))
  expect_named(dimnames(test_calibration_three_stations$MAE), c("N_seq", "alpha_seq"))
  expect_true(is.numeric(test_calibration_three_stations$minMAE))
  expect_length(test_calibration_three_stations$minMAE, 1)
  expect_identical(min(test_calibration_three_stations$MAE, na.rm = TRUE), test_calibration_three_stations$minMAE)
  expect_true(is.numeric(test_calibration_three_stations$N))
  expect_length(test_calibration_three_stations$N, 1)
  expect_true(is.numeric(test_calibration_three_stations$alpha))
  expect_length(test_calibration_three_stations$alpha, 1)
  expect_true(is.matrix(test_calibration_three_stations$observed))
  expect_length(dimnames(test_calibration_three_stations$observed)[[1]], 3)
  expect_identical(dimnames(test_calibration_three_stations$observed)[[1]], c("V3", "VD", "ZB"))
  expect_true(is.matrix(test_calibration_three_stations$predicted))
  expect_length(dimnames(test_calibration_three_stations$predicted)[[1]], 3)
  expect_identical(dimnames(test_calibration_three_stations$predicted)[[1]], c("V3", "VD", "ZB"))

  # stations as names
  expect_type(
    (test_calibration_three_stations_names <- interpolator_calibration(
      meteoland_interpolator_example,
      stations = c("V3", "VD", "ZB"),
      N_seq = c(10, 20), alpha_seq = c(9, 9.5))),
    'list'
  )
  expect_identical(test_calibration_three_stations_names, test_calibration_three_stations)

  # returning an interpolator
  expect_s3_class(
    (test_calibrated_interpolator <- interpolator_calibration(
      meteoland_interpolator_example,
      stations = c(76, 83, 187),
      update_interpolation_params = TRUE,
      N_seq = c(10, 20), alpha_seq = c(9, 9.5))),
    'stars'
  )
  expect_error(.is_interpolator(test_calibrated_interpolator), NA)
  expect_false(
    attr(meteoland_interpolator_example, 'params')$alpha_MinTemperature ==
      attr(test_calibrated_interpolator, 'params')$alpha_MinTemperature
  )
  expect_false(
    attr(meteoland_interpolator_example, 'params')$N_MinTemperature ==
      attr(test_calibrated_interpolator, 'params')$N_MinTemperature
  )
  expect_identical(
    meteoland_interpolator_example$MinTemperature, test_calibrated_interpolator$MinTemperature
  )
  expect_identical(names(meteoland_interpolator_example), names(test_calibrated_interpolator))
  expect_identical(
    names(attr(meteoland_interpolator_example, 'params')),
    names(attr(test_calibrated_interpolator, 'params'))
  )

  expect_identical(
    test_calibration_three_stations$N,
    attr(test_calibrated_interpolator, 'params')$N_MinTemperature
  )
  expect_identical(
    test_calibration_three_stations$alpha,
    attr(test_calibrated_interpolator, 'params')$alpha_MinTemperature
  )

})

test_that("interpolation cross validation works as expected", {

  expect_type(
    (crossvalidation_test <- interpolation_cross_validation(meteoland_interpolator_example)),
    "list"
  )

  expect_named(
    crossvalidation_test, c("errors", "station_stats", "dates_stats", "r2")
  )
  expect_true(all(
    is.data.frame(crossvalidation_test[["errors"]]),
    is.data.frame(crossvalidation_test[["station_stats"]]),
    is.data.frame(crossvalidation_test[["dates_stats"]])
  ))
  expect_type(crossvalidation_test[["r2"]], "list")
  expect_named(
    crossvalidation_test[["errors"]],
    c(
      "dates", "station", "stationID",
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation", "Precipitation"),
        "_error"
      ),
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation", "Precipitation"),
        "_predicted"
      ),
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation", "Precipitation"),
        "_observed"
      )
    )
  )
  expect_named(
    crossvalidation_test[["station_stats"]],
    c(
      "station", "stationID",
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation", "TotalPrecipitation", "DaysPrecipitation"),
        "_station_bias"
      ),
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation"),
        "_station_mae"
      ),
      paste0(
        c("TotalPrecipitation", "DaysPrecipitation"),
        "_station_relative_bias"
      ),
      paste0("FreqPrecipitation", c("_station_observed", "_station_predicted")),
      paste0("TotalPrecipitation", c("_station_observed", "_station_predicted"))
    ),
    ignore.order = TRUE
  )
  expect_named(
    crossvalidation_test[["dates_stats"]],
    c(
      "dates",
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation", "TotalPrecipitation", "StationsPrecipitation"),
        "_date_bias"
      ),
      paste0(
        c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation"),
        "_date_mae"
      ),
      paste0(
        c("TotalPrecipitation", "StationsPrecipitation"),
        "_date_relative_bias"
      ),
      paste0("FreqPrecipitation", c("_date_observed", "_date_predicted")),
      paste0("TotalPrecipitation", c("_date_observed", "_date_predicted"))
    ),
    ignore.order = TRUE
  )
  expect_named(
    crossvalidation_test[["r2"]],
    c("MinTemperature", "MaxTemperature", "RangeTemperature", "RelativeHumidity", "Radiation")
  )
  expect_equal(
    nrow(crossvalidation_test[["errors"]]),
    as.numeric(nrow(meteoland_interpolator_example[["MinTemperature"]]) * ncol(meteoland_interpolator_example[["MinTemperature"]]))
  )
  expect_equal(
    nrow(crossvalidation_test[["station_stats"]]),
    as.numeric(ncol(meteoland_interpolator_example[["MinTemperature"]]))
  )
  expect_equal(
    nrow(crossvalidation_test[["dates_stats"]]),
    as.numeric(nrow(meteoland_interpolator_example[["MinTemperature"]]))
  )

  # some stations
  expect_type(
    (crossvalidation_three_stations_test <-
       interpolation_cross_validation(meteoland_interpolator_example, stations = c(76L, 83L, 187L))),
    "list"
  )
  expect_named(
    crossvalidation_three_stations_test, c("errors", "station_stats", "dates_stats", "r2")
  )
  expect_true(all(
    is.data.frame(crossvalidation_three_stations_test[["errors"]]),
    is.data.frame(crossvalidation_three_stations_test[["station_stats"]]),
    is.data.frame(crossvalidation_three_stations_test[["dates_stats"]])
  ))
  expect_type(crossvalidation_three_stations_test[["r2"]], "list")

  expect_length(
    unique(crossvalidation_three_stations_test[["errors"]][["station"]]), 3
  )
  expect_true(
    all(unique(crossvalidation_three_stations_test[["errors"]][["stationID"]]) %in% c("V3", "VD", "ZB"))
  )
  expect_length(
    crossvalidation_three_stations_test[["station_stats"]][["station"]], 3
  )
  expect_true(
    all(crossvalidation_three_stations_test[["station_stats"]][["stationID"]] %in% c("V3", "VD", "ZB"))
  )

  # some stations by name
  expect_type(
    (crossvalidation_three_stations_name_test <-
       interpolation_cross_validation(meteoland_interpolator_example, stations = c("V3", "VD", "ZB"))),
    "list"
  )
  expect_identical(
    crossvalidation_three_stations_name_test, crossvalidation_three_stations_test
  )

})

library(sf)
### TODO
### This tests should be done with more than meteospain meteo, as we accept also worldmet and
### issues from that data can be hidden in these tests. The idea is to create a mixture of
### meteospain and worldmet data and use that as meteo_test object
meteo_test_correct <- readRDS("meteo_and_topo_test.rds")
meteo_test_names <- readRDS("meteospain_daily_test.rds")
meteo_test_no_sf <- dplyr::as_tibble(meteo_test_correct)
meteo_test_no_dates <- meteo_test_correct |> dplyr::select(-dates)
meteo_test_wrong_date_format <- meteo_test_correct |>
  dplyr::mutate(dates = as.character(dates))
meteo_test_no_id <- meteo_test_correct |> dplyr::select(-stationID)
meteo_test_no_numeric <- meteo_test_correct |>
  dplyr::mutate(MinTemperature = as.character(MinTemperature))
meteo_test_no_points <- meteo_test_correct |>
  dplyr::slice(1:10) |>
  sf::st_union() |>
  sf::st_as_sf()
meteo_test_correct_non_unique_ids <- meteo_test_correct
meteo_test_correct_non_unique_ids$geometry[1] <- meteo_test_correct_non_unique_ids$geometry[5000]

# # meteo test data, 1 month
# meteo_test <- readRDS('meteo_test.rds') |>
#   dplyr::filter(timestamp < (as.Date("2021-02-01") |> as.POSIXct()))
# meteo_test_correct <- meteospain2meteoland(meteo_test)
# # test_data
# meteo_test_no_sf <- dplyr::as_tibble(meteo_test_correct)
# meteo_test_no_dates <- meteo_test_correct |> dplyr::select(-dates)
# meteo_test_wrong_date_format <- meteo_test_correct |>
#   dplyr::mutate(dates = as.character(dates))
# meteo_test_no_id <- meteo_test_correct |> dplyr::select(-stationID)
# meteo_test_no_numeric <- meteo_test_correct |>
#   dplyr::mutate(MinTemperature = as.character(MinTemperature))
# meteo_test_no_points <- meteo_test_correct |>
#   dplyr::slice(1:10) |>
#   sf::st_union() |>
#   sf::st_as_sf()
# meteo_test_correct_non_unique_ids <- meteo_test_correct
# meteo_test_correct_non_unique_ids$geometry[1] <- meteo_test_correct_non_unique_ids$geometry[5000]

test_that("has_meteo checks work", {
  expect_true(has_meteo(meteo_test_correct))
  expect_error(has_meteo(meteo_test_names), "Names found in")
  expect_error(has_meteo(meteo_test_no_sf), "meteo must be an sf")
  expect_error(has_meteo(meteo_test_no_dates), "variable called dates")
  expect_error(has_meteo(meteo_test_wrong_date_format), "must be a date")
  expect_error(has_meteo(meteo_test_no_id), "stationID variable")
  expect_error(has_meteo(meteo_test_no_numeric), "MinTemperature")
  expect_error(has_meteo(meteo_test_no_points), "geometries must be")
  expect_error(has_meteo(meteo_test_correct_non_unique_ids), "There are more geometries")
})

topo_test_correct <- meteo_test_correct |>
  dplyr::mutate(aspect = NA_real_, slope = NA_real_) |>
  dplyr::as_tibble() |>
  dplyr::select(stationID, elevation, aspect, slope)
# test data
topo_test_sf <- meteo_test_correct |>
  dplyr::select(stationID, elevation) |>
  dplyr::mutate(aspect = NA_real_, slope = NA_real_)
topo_test_no_names <- topo_test_correct |>
  dplyr::rename(Elevation = "elevation")
topo_test_no_id <- topo_test_correct |>
  dplyr::select(-stationID)
topo_test_no_numeric <- topo_test_correct |>
  dplyr::mutate(elevation = as.character(elevation))

test_that("has_topo checks works", {
  expect_true(has_topo(topo_test_correct))
  expect_true(has_topo(topo_test_sf))
  expect_error(has_topo(topo_test_no_names), "elevation")
  expect_error(has_topo(topo_test_no_id), "stationID")
  expect_error(has_topo(topo_test_no_numeric), "elevation")
})

# test_data
meteo_test_no_topo <- meteo_test_correct |>
  dplyr::select(-elevation, -aspect, -slope)

test_that("with_meteo add_topo combo works", {
  expect_equal(meteo_test_no_topo, with_meteo(meteo_test_no_topo))
  expect_error(with_meteo(meteo_test_names), "Names found in")
  expect_error(with_meteo(meteo_test_no_sf), "meteo must be an sf")
  expect_error(with_meteo(meteo_test_no_dates), "variable called dates")
  expect_error(with_meteo(meteo_test_wrong_date_format), "must be a date")
  expect_error(with_meteo(meteo_test_no_id), "stationID variable")
  expect_error(with_meteo(meteo_test_no_numeric), "MinTemperature")
  expect_error(with_meteo(meteo_test_no_points), "geometries must be")

  expect_error(
    with_meteo(meteo_test_no_topo) |> add_topo(topo_test_no_names), "elevation"
  )
  expect_error(
    with_meteo(meteo_test_no_topo) |> add_topo(topo_test_no_id), "stationID"
  )
  expect_error(
    with_meteo(meteo_test_no_topo) |> add_topo(topo_test_no_numeric), "elevation"
  )
  expect_s3_class(
    meteo_test_with_topo <- with_meteo(meteo_test_no_topo) |>
      add_topo(topo_test_correct),
    "sf"
  )
  expect_identical(
    nrow(meteo_test_with_topo), nrow(meteo_test_no_topo)
  )
  expect_true(
    all(c("elevation", "MinTemperature", "MaxTemperature", "Precipitation")
        %in% names(meteo_test_with_topo))
  )

  # special cases
  # both spatial. Topo is converted internally to tibble, so the process
  # should work, but also return identical results as with no sf object.
  # Also, no duplicated geometries should be found now
  expect_s3_class(
    meteo_test_with_topo_sf <-
      with_meteo(meteo_test_no_topo) |> add_topo(topo_test_sf),
    "sf"
  )
  expect_equal(meteo_test_with_topo, meteo_test_with_topo_sf)
  # meteo has already topo but we joined equally
  expect_warning(
    with_meteo(meteo_test_correct) |> add_topo(topo_test_correct),
    "Topography variables found"
  )
})

#' From meteospain to meteoland meteo objects
#'
#' Adapting meteospain meteo objects to meteoland meteo objects
#'
#' This function converts \code{meteospain} R package meteo objects to
#' compatible meteoland meteo objects by selecting the needed variables and
#' adapting the names to comply with meteoland requirements.
#'
#' @param meteo meteospain meteo object.
#' @param complete logical indicating if the meteo data missing variables
#' should be calculated (if possible). Default to FALSE.
#' @param params A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.
#' @return a compatible meteo object to use with meteoland.
#'
#' @examples
#'
#' if (interactive()) {
#'   # meteospain data
#'   library(meteospain)
#'   mg_april_2022_data <- get_meteo_from(
#'     "meteogalicia",
#'     meteogalicia_options("daily", as.Date("2022-04-01"), as.Date("2022-04-30"))
#'   )
#'
#'   # just convert
#'   meteospain2meteoland(mg_april_2022_data)
#'   # convert and complete
#'   meteospain2meteoland(mg_april_2022_data, complete = TRUE)
#'
#' }
#'
#' @export meteospain2meteoland
meteospain2meteoland <- function(meteo, complete = FALSE, params = defaultInterpolationParams()) {

  # assertions
  assertthat::assert_that(
    "timestamp" %in% names(meteo),
    msg = "Provided data has no timestamp variable. Is this a meteospain dataset?"
  )
  assertthat::assert_that(
    is.logical(complete) && !is.na(complete),
    msg = "'complete' argument must be logical (FALSE or TRUE)"
  )

  # check dictionary
  dictionary <- .meteospain_variables_dictionary(TRUE)
  if ("mean_temperature" %in% names(meteo)) {
    dictionary <- .meteospain_variables_dictionary(FALSE)
  }

  .reshape_meteo(meteo, dictionary, complete, params)

}

#' From worldmet to meteoland meteo objects
#'
#' Adapting \code{\link[worldmet]{importNOAA}} meteo objects to meteoland meteo objects
#'
#' This function converts \code{\link[worldmet]{importNOAA}} meteo objects to
#' compatible meteoland meteo objects by selecting the needed variables and
#' adapting the names to comply with meteoland requirements. Also it aggregates
#' subdaily data as well as complete missing variables if possible (setting
#' \code{complete = TRUE})
#'
#' @param meteo worldmet meteo object.
#' @param complete logical indicating if the meteo data missing variables
#' should be calculated (if possible). Default to FALSE.
#' @param params A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.
#' @return a compatible meteo object to use with meteoland.
#'
#' @examples
#'
#' if (interactive()) {
#'   # worldmet data
#'   library(worldmet)
#'   worldmet_stations <- worldmet::getMeta(lat = 42, lon = 0, n = 2, plot = FALSE)
#'   worldmet_subdaily_2022 <-
#'     worldmet::importNOAA(worldmet_stations$code, year = 2022, hourly = TRUE)
#'
#'   # just convert
#'   worldmet2meteoland(worldmet_subdaily_2022)
#'   # convert and complete
#'   worldmet2meteoland(worldmet_subdaily_2022, complete = TRUE)
#'
#' }
#'
#' @export
worldmet2meteoland <- function(meteo, complete = FALSE, params = defaultInterpolationParams()) {

  # assertions
  assertthat::assert_that(
    all(c("date", "code") %in% names(meteo)),
    msg = "Provided data has no date or code variables. Is this a worldmet dataset?"
  )
  assertthat::assert_that(
    is.logical(complete) && !is.na(complete),
    msg = "'complete' argument must be logical (FALSE or TRUE)"
  )

  meteo |>
    # Sometimes, on worldmet, some points lack data (inclided spatial), so we remove those
    dplyr::filter(!is.na(.data$latitude)) |>
    # We convert to sf
    sf::st_as_sf(
      coords = c("longitude", "latitude"), crs = sf::st_crs(4326)
    ) |>
    # And reshape
    .reshape_meteo(
      dictionary = .worldmet_variables_dictionary(),
      complete = complete,
      params = params
    )

}

#' Reshape meteo API
#'
#' Convert meteo to meteoland complying sf
#'
#' The logic of the API is simple. If we have a dictionary with the corresponding names
#' we can just drop units (just in case, looking at you meteospain) and rename variables with the
#' dictionary. Once we have renamed the variables, the rest of the logic is the same independently
#' of the origin of the data:
#'
#' (I) Aggregate meteo if we detect subdaily, (II) fix possible mismatches between stations
#' and coordinates (sometimes, coordinates get updated and you end with a station with two sets
#' of coordinates, looking at you AEMET), and (III), if complete is TRUE, then complete the meteo
#' vars if possible.
#'
#' @section dictionary:
#' A dictionary is a named list, in which each element is the meteo object variable names. The list
#' is named with the corresponding official meteoland meteo names. For example, for worldmet
#' datasets, the \code{link{worldmet2meteoland}} function uses as dictionary the follwing list:
#'
#' \code{
#'    list(
#'      dates = "date",
#'      stationID = "code",
#'      elevation = "elev",
#'      WindDirection = "wd",
#'      WindSpeed = "ws",
#'      Temperature = "air_temp",
#'      RelativeHumidity = "RH",
#'      Precipitation = "precip_6"
#'    )
#' }
#'
#' @param meteo Meteo sf object
#' @param dictionary A dictionary (named list) with the names of the meteo object variables.
#'   See details.
#' @param params A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.
#'
#' @return A meteo sf object complying with meteoland \code{\link{with_meteo}} standard
#'
#' @noRd
.reshape_meteo <- function(meteo, dictionary, complete, params) {

  # browser()

  processed_meteo <- meteo |>
    units::drop_units() |>
    # rename vars
    .rename_meteo_vars(dictionary) |>
    # before anything else, we need to convert stationID to character (worldmet returns a factor)
    dplyr::mutate(stationID = as.character(.data$stationID)) |>
    # aggregate subdaily if needed
    .aggregate_subdaily_meteo() |>
    # fix station geometries if needed (meteospain)
    .fix_station_geometries()

  if (complete) {
    processed_meteo <- processed_meteo |>
      complete_meteo(params = params)
  }

  return(processed_meteo)

}

#' Rename meteo object variables
#'
#' Rename meteo object variables to \code{\link{with_meteo}} standard
#'
#' @param meteo meteo sf object, unitless
#' @param dictionary A dictionary (named list) with the names of the meteo object variables.
#'
#' @return A meteo sf object compatible with \code{\link{with_meteo}}
#'
#' @noRd
.rename_meteo_vars <- function(meteo, dictionary) {

  # copy the meteo
  meteo_res <- meteo

  # walk through the dictionary and update meteo copy
  purrr::iwalk(
    dictionary,
    .f = function(var, name) {
      if (var %in% names(meteo)) {
        meteo_res <<- meteo_res |>
          # seems like a recurrent error in sf that renaming variables leads to
          # bad sf objects. With this we ensure a correct sf each loop.
          sf::st_sf() |>
          dplyr::rename(!!name := !!var)
      }
    }
  )

  return(meteo_res)
}

#' Worldmet subdaily dictionary
#'
#' Helper for creating the variable dictionary for \code{\link{worldmet2meteoland}}
#'
#' @return A dictionary (named list) with the worldmet and meteoland variables correspondences
#' @noRd
.worldmet_variables_dictionary <- function() {
  res <- list(
    dates = "date",
    stationID = "code",
    elevation = "elev",
    WindDirection = "wd",
    WindSpeed = "ws",
    Temperature = "air_temp",
    RelativeHumidity = "RH",
    Precipitation = "precip_6"
  )

  return(res)
}

#' meteospain subdaily dictionary
#'
#' Helper for creating the variable dictionary for \code{\link{meteospain2meteoland}}
#'
#' @param subdaily Logical. If meteospain data is subdaily, should be set to TRUE. This is done
#'   automatically if called from \code{\link{meteospain2meteoland}}
#'
#' @return A dictionary (named list) with the meteospain and meteoland variables correspondences
#' @noRd
.meteospain_variables_dictionary <- function(subdaily = TRUE) {
  # daily
  res <- list(
    dates = "timestamp",
    stationID = "station_id",
    elevation = "altitude",
    MeanTemperature = "mean_temperature",
    MinTemperature = "min_temperature",
    MaxTemperature = "max_temperature",
    MeanRelativeHumidity = "mean_relative_humidity",
    MinRelativeHumidity = "min_relative_humidity",
    MaxRelativeHumidity = "max_relative_humidity",
    Precipitation = "precipitation",
    WindDirection = "mean_wind_direction",
    WindSpeed = "mean_wind_speed",
    Radiation = "global_solar_radiation"
  )

  if (subdaily) {
    # subdaily
    res <- list(
      dates = "timestamp",
      stationID = "station_id",
      elevation = "altitude",
      Temperature = "temperature",
      MinTemperature = "min_temperature",
      MaxTemperature = "max_temperature",
      RelativeHumidity = "relative_humidity",
      Precipitation = "precipitation",
      WindDirection = "wind_direction",
      WindSpeed = "wind_speed",
      Radiation = "global_solar_radiation"
    )
  }

  return(res)

}

#' Aggregating subdaily meteo
#'
#' Aggregate subdaily meteo
#'
#' Function first check if data is subdaily by counting the ocurrences of station code in the same
#' day. If more than one, aggregating is enabled. Each variable is aggregated with the corresponding
#' function (\code{sum} for precipitation, \code{min} for minimums...)
#'
#' @param meteo meteo sf object with variables names already complying with \code{\link{with_meteo}}
#'
#' @return A daily aggregated meteo object ready to \code{\link{with_meteo}}
#' @noRd
.aggregate_subdaily_meteo <- function(meteo) {

  # browser()
  # check if data is subdaily
  geometry_name <- .get_geometry_name(meteo)

  grouped_meteo <- meteo |>
    dplyr::as_tibble() |>
    dplyr::select(-!!geometry_name) |>
    dplyr::mutate(
      dates = lubridate::floor_date(.data$dates, unit = "day")
    ) |>
    dplyr::group_by(.data$dates, .data$stationID)

  station_date_values <- grouped_meteo |>
    dplyr::count() |>
    dplyr::pull("n")

  # if only one value per station and date exists for all combinations,
  # no need of summarising
  if (!any(station_date_values > 1)) {
    return(meteo)
  }

  cli::cli_warn(c(
    "Provided meteo data seems to be in subdaily time steps, aggregating to daily scale"
  ))

  grouped_meteo |>
    .create_missing_vars() |>
    # ensure solar radiation is never negative
    dplyr::mutate(
      Radiation = dplyr::if_else(.data$Radiation < 0, 0, .data$Radiation)
    ) |>
    # summarise
    dplyr::summarise(
      stationID = .is_na_or_fun(.data$stationID, dplyr::first),
      elevation = .is_na_or_fun(.data$elevation, dplyr::first),
      MeanTemperature = .is_na_or_fun(.data$Temperature, mean, na.rm = TRUE),
      MinTemperature = .is_na_or_fun(.data$Temperature, min, .data$MinTemperature, na.rm = TRUE),
      MaxTemperature = .is_na_or_fun(.data$Temperature, max, .data$MaxTemperature, na.rm = TRUE),
      MeanRelativeHumidity = .is_na_or_fun(.data$RelativeHumidity, mean, na.rm = TRUE),
      MinRelativeHumidity = .is_na_or_fun(.data$RelativeHumidity, min, na.rm = TRUE),
      MaxRelativeHumidity = .is_na_or_fun(.data$RelativeHumidity, max, na.rm = TRUE),
      Precipitation = .is_na_or_fun(.data$Precipitation, sum, na.rm = TRUE),
      WindSpeed = .is_na_or_fun(.data$WindSpeed, mean, na.rm = TRUE),
      WindDirection = .is_na_or_fun(.data$WindDirection, .summarise_wind_direction),
      Radiation = .is_na_or_fun(.data$Radiation, sum, na.rm = TRUE)
    ) |>
    dplyr::ungroup() |>
    dplyr::left_join(
      meteo |>
        dplyr::ungroup() |>
        dplyr::select("stationID", !!geometry_name) |>
        dplyr::distinct(),
      by = c('stationID')
    ) |>
    sf::st_as_sf()

}

#' Helper to fix geometries problems
#'
#' Get the latest geometry for the meteo station
#'
#' Sometimes (AEMET), coordinates get updated, and data ends having differents coordinates in
#' different dates for the same station. This helper ensures to get the latest coordinates values
#' for a station when more than one set of coordinates is found
#'
#' @param meteo A daily meteo object complying with \code{\link{with_meteo}}
#'
#' @return The same meteo object but with only one set of coordinates per station
#' @noRd
.fix_station_geometries <- function(meteo) {

  geometry_name <- .get_geometry_name(meteo)

  distinct_rows <- meteo |>
    dplyr::ungroup() |>
    dplyr::arrange(.data$dates) |>
    dplyr::select("stationID", !!geometry_name) |>
    dplyr::distinct()

  # If more geometries than station IDs, issue a warning and filter by the last
  # geometry to remove duplicates
  if (nrow(distinct_rows) != length(unique(distinct_rows[["stationID"]]))) {
    cli::cli_warn(c(
      "Some stations have different metadata (elevation, coordinates...) for different dates.",
      "i" = "Choosing the most recent metadata"
    ))

    distinct_rows <- distinct_rows |>
      dplyr::group_by(.data$stationID) |>
      dplyr::filter(as.character(.data[[geometry_name]]) == dplyr::last(as.character(.data[[geometry_name]])))

    meteo <- meteo |>
      dplyr::as_tibble() |>
      dplyr::select(-!!geometry_name) |>
      dplyr::left_join(distinct_rows, by = 'stationID') |>
      # duplicated rows has been generated in the aggregation step due to different geometries for
      # the same stationID. We remove those duplicates
      dplyr::distinct() |>
      sf::st_as_sf()
  }

  return(meteo)

}

#' Helper for summarizing wind direction
#'
#' Helper for summarizing wind direction
#'
#' @param wind_direction Vector with subdaily values of wind direction
#'
#' @returns The "average" wind direction for the vector provided
#' @noRd
.summarise_wind_direction <- function(wind_direction) {
  wind_direction <- as.numeric(wind_direction)
  y <- sum(cos(wind_direction * pi / 180), na.rm = TRUE) / length(wind_direction)
  x <- sum(sin(wind_direction * pi / 180), na.rm = TRUE) / length(wind_direction)
  dv <- (180 / pi) * atan2(x, y)

  if (dv < 0) {
    dv <- dv + 360
  }
  return(dv)
}

# na or fun, if all vector is NAs, return NAs, if not apply the function
#' Helper for creating NA variables
#'
#' Helper for creating NA variables with double class instead of logical
#'
#' When applying a function to an all NA vector, things go bad. This helper ensures that the
#' summarising funcion can be applied, and if all the vector is NA, then returns a NA_real_ to
#' maintaing the vector class in the tibble.
#'
#' @param var Variable vector
#' @param fun Function to apply
#' @param ... Arguments for fun
#'
#' @return The result of fun(var) if possible, NA_real_ if not
#' @noRd
.is_na_or_fun <- function(var, fun, ...) {
  if (all(is.na(var))) {
    return(NA_real_)
  }
  fun(var, ...)
}

# create missing vars

#' Helper to create missing vars in meteo
#'
#' Helper to create missing vars in meteo
#'
#' If any variable needed for summarising is missing, errors ensue. So we made sure any needed
#' variable is there
#'
#' @param meteo meteo sf object complying with \code{\link{with_meteo}}
#'
#' @return meteo sf object with missing empty variables (NA_real_)
#' @noRd
.create_missing_vars <- function(meteo) {
  meteo_names <- names(meteo)
  if (!"MinTemperature" %in% meteo_names) {
    meteo$MinTemperature <- NA_real_
  }
  if (!"MaxTemperature" %in% meteo_names) {
    meteo$MaxTemperature <- NA_real_
  }
  if (!"Radiation" %in% meteo_names) {
    meteo$Radiation <- NA_real_
  }
  if (!"Precipitation" %in% meteo_names) {
    meteo$Precipitation <- NA_real_
  }
  if (!"RelativeHumidity" %in% meteo_names) {
    meteo$RelativeHumidity <- NA_real_
  }
  return(meteo)
}


#' Complete missing meteo variables
#'
#' Calculates missing values of relative humidity, radiation and potential
#' evapotranspiration from a data frame with daily values of
#' minimum/maximum/mean temperature and precipitation. The function 
#' takes a meteo object (with meteoland names) and complete any
#' missing variable if it is possible
#'
#' @param meteo meteoland weather data
#' @param params A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' 
#' @return the same \code{meteo} data provided with the the variables completed
#'
#' @details
#' #' The function fills values for humidity, radiation and PET only if they are
#' missing in the input data frame. If a column 'SpecificHumidity' is present
#' in the input data, relative humidity is calculated from it. Otherwise,
#' relative humidity is calculated assuming that dew point temperature equals
#' the minimum temperature. Potential solar radiation is calculated from
#' latitude, slope and aspect. Incoming solar radiation is then corrected
#' following Thornton & Running (1999) and potential evapotranspiration
#' following Penman (1948).
#'
#' @references 
#' Thornton, P.E., Running, S.W., 1999. An improved algorithm for
#' estimating incident daily solar radiation from measurements of temperature,
#' humidity, and precipitation. Agric. For. Meteorol. 93, 211-228.
#'
#' Penman, H. L. 1948. Natural evaporation from open water, bare soil and
#' grass. Proceedings of the Royal Society of London. Series A. Mathematical
#' and Physical Sciences, 193, 120-145. 
#' 
#' @author 
#' Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF
#' 
#' Victor Granda \enc{García}{Garcia}, EMF-CREAF
#' 
#' @examples
#'
#' \donttest{
#' # example data
#' data("meteoland_meteo_example")
#'
#' # remove MinRelativeHumidity
#' meteoland_meteo_example$MinRelativeHumidity <- NULL
#' # complete vars
#' completed_meteo <- complete_meteo(meteoland_meteo_example)
#' # check MinRelativeHumidity
#' completed_meteo$MinRelativeHumidity
#' }
#'
#' @export complete_meteo
complete_meteo <- function(meteo,
                           params = defaultInterpolationParams(),
                           verbose = getOption("meteoland_verbosity", TRUE)) {

  # assertions
  assertthat::assert_that(has_meteo(meteo))
  assertthat::assert_that(has_topo(meteo))

  .verbosity_control(
    cli::cli_alert_info("Completing missing variables if possible:"),
    verbose
  )

  # This no!!! We need a function for each row
  .complete_relative_humidity <- function(RelativeHumidity, MinTemperature, MeanTemperature, verbose) {

    .verbosity_control(
      cli::cli_ul("RelativeHumidity"),
      verbose
    )

    purrr::pmap_dbl(
      list(RelativeHumidity, MinTemperature, MeanTemperature),
      ~ dplyr::if_else(
        is.na(..1),
        max(min(100 * utils_saturationVP(..2) / utils_saturationVP(..3), 100), 0),
        ..1
      )
    )
  }

  .complete_min_relative_humidity <- function(MinRelativeHumidity, MinTemperature, MaxTemperature, verbose) {

    .verbosity_control(
      cli::cli_ul("MinRelativeHumidity"),
      verbose
    )

    purrr::pmap_dbl(
      list(MinRelativeHumidity, MinTemperature, MaxTemperature),
      ~ dplyr::if_else(
        is.na(..1),
        max(min(100 * utils_saturationVP(..2) / utils_saturationVP(..3), 100), 0),
        ..1
      )
    )
  }

  .complete_max_relative_humidity <- function(MaxRelativeHumidity, verbose) {
    .verbosity_control(
      cli::cli_ul("MaxRelativeHumidity"),
      verbose
    )

    dplyr::if_else(
      is.na(MaxRelativeHumidity), 100, MaxRelativeHumidity
    )
  }

  .complete_radiation <- function(
    Radiation, dates, MinTemperature, MaxTemperature,
    MinRelativeHumidity, MaxRelativeHumidity,
    Precipitation,
    elevation, slope, aspect,
    geometry, verbose
  ) {
    .verbosity_control(
      cli::cli_ul("Radiation"),
      verbose
    )

    julian_day <- purrr::map_int(
      dates,
      ~ radiation_julianDay(
        as.numeric(format(.x, "%Y")),
        as.numeric(format(.x, "%m")),
        as.numeric(format(.x, "%d"))
      )
    )
    solar_constant <- purrr::map_dbl(julian_day, radiation_solarConstant)
    declination <- purrr::map_dbl(julian_day, radiation_solarDeclination)
    average_vapour_pressure <- purrr::pmap_dbl(
      list(MinTemperature, MaxTemperature, MinRelativeHumidity, MaxRelativeHumidity),
      utils_averageDailyVP
    )
    slope_rad <- dplyr::if_else(
      is.na(slope), 0, slope*(pi/180)
    )
    aspect_rad <- dplyr::if_else(
      is.na(aspect), 0, aspect*(pi/180)
    )
    latitude_rad <-
      sf::st_coordinates(sf::st_transform(geometry, 4326))[,2] * (pi/180)

    diff_temp <- MaxTemperature - MinTemperature

    purrr::pmap_dbl(
      list(
        solar_constant, latitude_rad, elevation, slope_rad,
        aspect_rad, declination, diff_temp, diff_temp,
        average_vapour_pressure, Precipitation
      ),
      radiation_solarRadiation
    )
  }
  
  .complete_pet<-function(PET, dates, MinTemperature, MaxTemperature,
                          MinRelativeHumidity, MaxRelativeHumidity,
                          Radiation, WindSpeed,
                          elevation, slope, aspect,
                          geometry, params, 
                          verbose) {
    
    .verbosity_control(
      cli::cli_ul("PET"),
      verbose
    )
    if(!("wind_height" %in% names(params))) params$wind_height = 10
    if(!("wind_roughness_height" %in% names(params))) params$wind_roughness_height = 0.001
    if(!("penman_albedo" %in% names(params))) params$penman_albedo = 0.25
    if(!("penman_windfun" %in% names(params))) params$penman_windfun = "1956"
    
    julian_day <- purrr::map_int(
      dates,
      ~ radiation_julianDay(
        as.numeric(format(.x, "%Y")),
        as.numeric(format(.x, "%m")),
        as.numeric(format(.x, "%d"))
      )
    )
    slope_rad <- dplyr::if_else(
      is.na(slope), 0, slope*(pi/180)
    )
    aspect_rad <- dplyr::if_else(
      is.na(aspect), 0, aspect*(pi/180)
    )
    latitude_rad <-
      sf::st_coordinates(sf::st_transform(geometry, 4326))[,2] * (pi/180)
    
    purrr::pmap_dbl(
      list(
        latitude_rad, elevation, slope_rad, aspect_rad, julian_day,
        MinTemperature, MaxTemperature, MinRelativeHumidity, MaxRelativeHumidity, 
        Radiation, WindSpeed,
        z = params$wind_height,
        z0 = params$wind_roughness_height,
        alpha = params$penman_albedo,
        windfun = params$penman_windfun
      ),
      penman
    )
  }

  # fill missong vars in meteo with NA
  c(
    "MeanTemperature", "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
    "Precipitation", "Radiation","PET", "aspect", "slope"
  ) |>
    purrr::walk(
      \(variable) {
        if (is.null(meteo[[variable]])) {
          meteo[[variable]] <<- NA_real_
        }
      }
    )

  meteo_completed <- meteo |>
    dplyr::mutate(
      MeanRelativeHumidity = .complete_relative_humidity(
        .data$MeanRelativeHumidity, .data$MinTemperature,
        .data$MeanTemperature, verbose
      ),
      MinRelativeHumidity = .complete_min_relative_humidity(
        .data$MinRelativeHumidity, .data$MinTemperature,
        .data$MaxTemperature, verbose
      ),
      MaxRelativeHumidity = .complete_max_relative_humidity(.data$MaxRelativeHumidity, verbose),
      Radiation = .complete_radiation(
        .data$Radiation, .data$dates, .data$MinTemperature, .data$MaxTemperature,
        .data$MinRelativeHumidity, .data$MaxRelativeHumidity,
        .data$Precipitation,
        .data$elevation, .data$slope, .data$aspect,
        sf::st_geometry(meteo), verbose
      ),
      PET = .complete_pet(.data$PET, .data$dates, .data$MinTemperature, .data$MaxTemperature,
                          .data$MinRelativeHumidity, .data$MaxRelativeHumidity,
                          .data$Radiation, .data$WindSpeed,
                          .data$elevation, .data$slope, .data$aspect,
                          sf::st_geometry(meteo), params, 
                          verbose
      )
    )

  .verbosity_control(
    cli::cli_alert_success("Done"),
    verbose
  )

  return(meteo_completed)
}

# complete meteo
test_that("complete_meteo works", {
  meteo_test <- readRDS('meteospain_daily_test.rds') |> meteospain2meteoland(complete = FALSE)
  meteo_test_no_precip <- meteo_test |> dplyr::select(!"Precipitation")
  meteo_test_no_rad <- meteo_test |> dplyr::select(!"Radiation")

  expect_s3_class(completed_ok <- complete_meteo(meteo_test), "sf")
  expect_s3_class(completed_no_precip <- complete_meteo(meteo_test_no_precip), "sf")
  expect_s3_class(completed_no_rad <- complete_meteo(meteo_test_no_rad), "sf")

  expect_true(sum(is.na(completed_ok$MeanRelativeHumidity)) < sum(is.na(meteo_test$MeanRelativeHumidity)))
  expect_true(sum(is.na(completed_ok$Radiation)) < sum(is.na(meteo_test$Radiation)))

  expect_true(sum(is.na(completed_no_precip$MeanRelativeHumidity)) < sum(is.na(meteo_test_no_precip$MeanRelativeHumidity)))
  expect_true(sum(is.na(completed_no_precip$Radiation)) < sum(is.na(meteo_test_no_precip$Radiation)))

  expect_true(sum(is.na(completed_no_rad$MeanRelativeHumidity)) < sum(is.na(meteo_test_no_rad$MeanRelativeHumidity)))
  expect_true(any(!is.na(completed_no_rad$Radiation)))
})


# worldmet2meteoland
test_that("worldmet2meteoland works", {

  # skip_on_cran()

  # test data
  worldmet_test <- readRDS("worldmet_test.rds")

  expect_s3_class(test_res <- suppressWarnings(worldmet2meteoland(worldmet_test)), 'sf')
  expect_true(
    all(c(
      "MeanTemperature", "MinTemperature", "MaxTemperature",
      "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "stationID", "dates", "elevation"
    ) %in% names(test_res))
  )
  expect_true(sum(duplicated(test_res)) < 1)

  meteo_test_no_dates <- worldmet_test |> dplyr::select(-date)
  meteo_test_no_relative_humidity <- worldmet_test |>
    dplyr::select(-RH)

  expect_error(
    suppressWarnings(worldmet2meteoland(meteo_test_no_dates)),
    "Provided data has no date or code variables"
  )

  expect_s3_class(
    test_res_norh <- suppressWarnings(worldmet2meteoland(meteo_test_no_relative_humidity)),
    "sf"
  )
  expect_true(
    all(c(
      "MinTemperature", "MaxTemperature", "Precipitation",
      "stationID", "dates"
    ) %in% names(test_res_norh))
  )
  expect_true(sum(duplicated(test_res_norh)) < 1)

  expect_s3_class(
    test_complete_no_changes <- suppressWarnings(worldmet2meteoland(worldmet_test, complete = TRUE)),
    "sf"
  )
  expect_identical(nrow(test_complete_no_changes), nrow(test_res))
  expect_identical(names(test_complete_no_changes), c(names(test_res), 'PET','aspect', 'slope'))
  expect_identical(test_complete_no_changes$MinTemperature, test_res$MinTemperature)
  expect_false(
    identical(test_res$Radiation, test_complete_no_changes$Radiation)
  )
  expect_true(sum(duplicated(test_complete_no_changes)) < 1)


})

test_that("meteospain2meteoland works", {
  # test data
  meteo_test <- readRDS('meteospain_daily_test.rds')
  meteo_test_non_unique_ids <- meteo_test
  meteo_test_non_unique_ids$geometry[1] <- meteo_test_non_unique_ids$geometry[5000]
  meteo_test_subdaily_with_errors <- readRDS('meteospain_subdaily_test.rds')
  meteo_test_subdaily_with_errors$geometry[1] <- meteo_test_subdaily_with_errors$geometry[5000]

  # daily
  expect_s3_class(test_res <- meteospain2meteoland(meteo_test), 'sf')
  expect_true(
    all(c(
      "MeanTemperature", "MinTemperature", "MaxTemperature",
      "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "stationID", "dates", "elevation"
    ) %in% names(test_res))
  )
  expect_true(sum(duplicated(test_res)) < 1)

  meteo_test_no_dates <- meteo_test |> dplyr::select(-timestamp)
  meteo_test_no_relative_humidity <- meteo_test |>
    dplyr::select(-mean_relative_humidity)

  expect_error(
    meteospain2meteoland(meteo_test_no_dates),
    "Provided data has no timestamp variable"
  )
  expect_s3_class(
    test_res_norh <- meteospain2meteoland(meteo_test_no_relative_humidity),
    "sf"
  )
  expect_true(
    all(c(
      "MinTemperature", "MaxTemperature", "Precipitation",
      "stationID", "dates"
    ) %in% names(test_res_norh))
  )
  expect_true(sum(duplicated(test_res_norh)) < 1)

  expect_warning(
    meteo_test_fixed <- meteospain2meteoland(meteo_test_non_unique_ids),
    "Choosing the most recent metadata"
  )
  expect_identical(meteo_test_fixed, test_res)
  expect_true(sum(duplicated(meteo_test_fixed)) < 1)

  # complete checks

  expect_s3_class(
    test_complete_no_changes <- meteospain2meteoland(meteo_test, complete = TRUE),
    "sf"
  )
  expect_identical(nrow(test_complete_no_changes), nrow(test_res))
  expect_identical(names(test_complete_no_changes), c(names(test_res),'PET', 'aspect', 'slope'))
  expect_identical(test_complete_no_changes$MinTemperature, test_res$MinTemperature)
  expect_false(
    identical(test_res$MeanRelativeHumidity, test_complete_no_changes$MeanRelativeHumidity)
  )
  expect_true(sum(duplicated(test_complete_no_changes)) < 1)

  meteo_test_no_vars <- dplyr::mutate(
    meteo_test,
    mean_relative_humidity = NA_real_,
    min_relative_humidity = NA_real_,
    max_relative_humidity = NA_real_,
    global_solar_radiation = NA_real_
  )

  expect_s3_class(
    test_complete_all <- meteospain2meteoland(meteo_test_no_vars, complete = TRUE),
    "sf"
  )
  expect_identical(nrow(test_complete_all), nrow(test_res))
  expect_identical(names(test_complete_all), c(names(test_res), 'PET','aspect', 'slope'))
  expect_identical(test_complete_all$MinTemperature, test_res$MinTemperature)
  expect_true(any(!is.na(test_complete_all$MeanRelativeHumidity)))
  expect_true(any(!is.na(test_complete_all$MinRelativeHumidity)))
  expect_true(all(!is.na(test_complete_all$MaxRelativeHumidity))) # is all because we put 100 in all
  expect_true(any(!is.na(test_complete_all$Radiation)))

  expect_true(sum(duplicated(test_complete_all)) < 1)

  # subdaily checks

  # we have two warnings here, warnings are tricky to catch when there are more than one. One is
  # catched, the others trigger and the test fails :(
  # expect_warning(
  #   .reshape_meteo(meteo_test_subdaily_with_errors, .meteospain_variables_dictionary(TRUE), FALSE),
  #   "Choosing the most recent metadata"
  # )
  # expect_warning(
  #   .reshape_meteo(meteo_test_subdaily_with_errors, .meteospain_variables_dictionary(TRUE), FALSE),
  #   "Provided meteo data seems to be in subdaily time steps"
  # )
  # expect_warning(
  #   (subdaily_fixed <- .fix_station_geometries(meteo_test_subdaily_with_errors)),
  #   "Choosing the most recent metadata"
  # )

  # we expect no error
  expect_no_error(test_subdaily <- suppressWarnings(meteospain2meteoland(meteo_test_subdaily_with_errors)))
  expect_s3_class(test_subdaily, 'sf')
  expect_true(
    all(c(
      "MeanTemperature", "MinTemperature", "MaxTemperature",
      "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "stationID", "dates", "elevation"
    ) %in% names(test_subdaily))
  )
  expect_true(sum(duplicated(test_subdaily)) < 1)

  expect_s3_class(
    suppressWarnings(test_subdaily_complete <- meteospain2meteoland(meteo_test_subdaily_with_errors, complete = TRUE)),
    "sf"
  )
  expect_identical(nrow(test_subdaily_complete), nrow(test_subdaily))
  expect_identical(names(test_subdaily_complete), c(names(test_subdaily), 'PET', 'aspect', 'slope'))
  expect_identical(test_subdaily_complete$MinTemperature, test_subdaily$MinTemperature)
  expect_false(
    identical(test_subdaily$MeanRelativeHumidity, test_subdaily_complete$MeanRelativeHumidity)
  )
  expect_true(sum(duplicated(test_subdaily_complete)) < 1)


})

# TODO to test fringe cases when metadata of station changes, test this with
# subdaily_meteo_generating_duplicated_rows_in_meteospain2meteoland.rds which
# generates duplicated rows in the process.
test_that("meteospain2meteoland works with subdaily fringe cases", {

  meteo_test_subdaily_with_errors <-
    readRDS("subdaily_duplicated_rows_generator.rds")
  expect_no_error(test_subdaily <- suppressWarnings(meteospain2meteoland(meteo_test_subdaily_with_errors)))
  expect_s3_class(test_subdaily, 'sf')
  expect_true(
    all(c(
      "MeanTemperature", "MinTemperature", "MaxTemperature",
      "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "stationID", "dates", "elevation"
    ) %in% names(test_subdaily))
  )
  expect_true(sum(duplicated(test_subdaily)) < 1)

  expect_s3_class(
    suppressWarnings(test_subdaily_complete <- meteospain2meteoland(meteo_test_subdaily_with_errors, complete = TRUE)),
    "sf"
  )
  expect_identical(nrow(test_subdaily_complete), nrow(test_subdaily))
  expect_identical(names(test_subdaily_complete), c(names(test_subdaily),'PET', 'aspect', 'slope'))
  expect_identical(test_subdaily_complete$MinTemperature, test_subdaily$MinTemperature)
  expect_false(
    identical(test_subdaily$MeanRelativeHumidity, test_subdaily_complete$MeanRelativeHumidity)
  )
  expect_true(sum(duplicated(test_subdaily_complete)) < 1)
})

#' Humidity conversion tools
#'
#' @description
#' Functions to transform relative humidity to specific humidity or dew point
#' temperature and viceversa.
#'
#' @param Tc A numeric vector of temperature in degrees Celsius.
#' @param HR A numeric vector of relative humidity (in %).
#' @param Td A numeric vector of dew temperature in degrees Celsius.
#' @param HS A numeric vector of specific humidity (unitless).
#' @param allowSaturated Logical flag to allow values over 100%
#' @return A numeric vector with specific or relative humidity.
#' @author Nicholas Martin-StPaul, INRA
#'
#' Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
#' @seealso \code{\link{complete_meteo}}
#' @rdname humidity-conversion-tools
#' @export
humidity_relative2dewtemperature <- function(Tc, HR) {

  # assertions
  assertthat::assert_that(
    is.numeric(Tc), is.numeric(HR),
    msg = "Tc and HR must be numeric vectors"
  )
  assertthat::assert_that(
    identical(length(Tc), length(HR)), msg = "Tc and HR must be the same length"
  )

  if(is.data.frame(Tc)) Tc = as.matrix(Tc)
  if(is.data.frame(HR)) HR = as.matrix(HR)
  if(is.matrix(Tc) && is.matrix(HR)) {
    Td = .dewpointTemperatureFromRH(Tc,HR)
    dimnames(Td) = dimnames(Tc)
  } else {
    Td = as.vector(.dewpointTemperatureFromRH(as.matrix(Tc),as.matrix(HR)))
  }
  return(Td)
}
#' @rdname humidity-conversion-tools
#' @export
humidity_dewtemperature2relative<-function(Tc, Td, allowSaturated = FALSE) {

  # assertions
  assertthat::assert_that(
    is.numeric(Tc), is.numeric(Td),
    msg = "Tc and Td must be numeric vectors"
  )
  assertthat::assert_that(
    assertthat::is.flag(allowSaturated), msg = "allowSaturated must be TRUE or FALSE"
  )
  assertthat::assert_that(
    identical(length(Tc), length(Td)), msg = "Tc and Td must be the same length"
  )

  HR= as.vector(.relativeHumidityFromDewpointTemp(as.numeric(Tc),as.numeric(Td)))
  if(!allowSaturated) HR[HR>100]=100 # On ne passe pas audessus de 100
  return(HR)
}

#Functions to switch from relative humidity to specific humidity
#' @rdname humidity-conversion-tools
#' @export
humidity_specific2relative <- function(Tc, HS, allowSaturated = FALSE) {

  # assertions
  assertthat::assert_that(
    is.numeric(Tc), is.numeric(HS),
    msg = "Tc and HS must be numeric vectors"
  )
  assertthat::assert_that(
    assertthat::is.flag(allowSaturated), msg = "allowSaturated must be TRUE or FALSE"
  )
  assertthat::assert_that(
    identical(length(Tc), length(HS)), msg = "Tc and HS must be the same length"
  )

  #-------------------------------------------------------------
  #D?claration des constantes pour le calcul de l'HR et de la T en ?C
  Mas=28.966 # masse molaire air sec (g/mol)
  Mh2o=18 # masse molaire H2O(g/mol)
  Rgz=8.314472 # %J/mol/K cste gaz parfait
  p_air=101325 #  %en Pa
  #-------------------------------------------------------------

  #Calcul de l'Humidit? relative bas? sur Hs et Tk
  Tk=Tc+273.15
  Dair=((p_air)/(Rgz*(Tk)))*Mas # Calcul de la masse volumique de l'air sec en g.m-3
  masshum=HS*Dair # Masse d'eau dans l'air g.m-3
  nhum=masshum/Mh2o # en mol.m-3
  ea=nhum*(Rgz*(Tk)) # Pression de vapeur r?elle (en Pa)
  es=6.108*exp(17.27*Tc/(237.2+Tc))*100 # Pression de vapeur saturante ? la Temp?rature T (en Pa)
  HR=ea/es*100 #Calcul de l'HR
  if(!allowSaturated) HR[HR>100]=100 # On ne passe pas audessus de 100
  return(HR)
}
#' @rdname humidity-conversion-tools
#' @export
humidity_relative2specific<-function(Tc, HR){

  # assertions
  assertthat::assert_that(
    is.numeric(Tc), is.numeric(HR),
    msg = "Tc and HR must be numeric vectors"
  )
  assertthat::assert_that(
    identical(length(Tc), length(HR)), msg = "Tc and HR must be the same length"
  )

  #-------------------------------------------------------------
  #D?claration des constantes pour le calcul de l'HR et de la T en ?C
  Mas=28.966 # masse molaire air sec (g/mol)
  Mh2o=18 # masse molaire H2O(g/mol)
  Rgz=8.314472 # %J/mol/K cste gaz parfait
  p_air=101325 #  %en Pa
  #-------------------------------------------------------------

  #Calcul de l'Humidit? relative bas? sur Hs et Tk
  Tk=Tc+273.15

  Dair=((p_air)/(Rgz*(Tk)))*Mas # Calcul de la masse volumique de l'air sec en g.m-3
  es=6.108*exp(17.27*Tc/(237.2+Tc))*100 # Pression de vapeur saturante ? la Temp?rature T (en Pa)
  ea=HR*es/100 #Calcul de l'ea
  nhum=ea/(Rgz*(Tk)) # Pression de vapeur r?elle (en Pa)
  masshum=nhum*Mh2o # en mol.m-3
  HS=masshum/Dair
  return(HS)
}

temperature <- seq(5, 30, 5)
relative_humidity <- seq(50, 100, 10)
dew_temperature <- seq(6, 1, -1)
specific_humidity <- seq(0.00266, 0.0266, length.out = 6)

test_that("humidity_relative2dewtemperature works as expected", {

  # vector
  expect_type(
    (test_res <- humidity_relative2dewtemperature(temperature, relative_humidity)),
    "double"
  )
  expect_length(test_res, length(temperature))

  # matrix
  temperature_matrix <- matrix(temperature, nrow = 2, ncol = 3)
  relative_humidity_matrix <- matrix(relative_humidity, nrow = 2, ncol = 3)
  expect_no_error(
    (test_res_matrix <- humidity_relative2dewtemperature(temperature_matrix, relative_humidity_matrix))
  )
  expect_true(inherits(test_res_matrix, "matrix"))
  expect_equal(dim(test_res_matrix), c(2,3))

  # error when no numbers
  expect_error(
    humidity_relative2dewtemperature(letters, LETTERS),
    "must be numeric vectors"
  )
  expect_error(
    humidity_relative2dewtemperature(letters[1:6], relative_humidity),
    "must be numeric vectors"
  )
  expect_error(
    humidity_relative2dewtemperature(temperature, LETTERS[1:6]),
    "must be numeric vectors"
  )
  expect_error(
    humidity_relative2dewtemperature(temperature, c(relative_humidity, 100, 30)),
    "must be the same length"
  )
})

test_that("humidity_dewtemperature2relative works as expected", {
  # vector
  expect_type(
    (test_res <- humidity_dewtemperature2relative(temperature, dew_temperature)),
    "double"
  )
  expect_length(test_res, length(temperature))

  # matrix
  temperature_matrix <- matrix(temperature, nrow = 2, ncol = 3)
  dew_temperature_matrix <- matrix(dew_temperature, nrow = 2, ncol = 3)
  expect_no_error(
    (test_res_matrix <- humidity_dewtemperature2relative(temperature_matrix, dew_temperature_matrix))
  )
  expect_identical(test_res, test_res_matrix)

  # it should be no value above 100
  expect_false(any(test_res > 100))

  # error when no numbers
  expect_error(
    humidity_dewtemperature2relative(letters, LETTERS),
    "must be numeric vectors"
  )
  expect_error(
    humidity_dewtemperature2relative(letters[1:6], dew_temperature),
    "must be numeric vectors"
  )
  expect_error(
    humidity_dewtemperature2relative(temperature, LETTERS[1:6]),
    "must be numeric vectors"
  )
  expect_error(
    humidity_dewtemperature2relative(temperature, dew_temperature, "TRUE"),
    "TRUE or FALSE"
  )
  expect_error(
    humidity_dewtemperature2relative(temperature, c(dew_temperature, 3, 1)),
    "must be the same length"
  )

})

test_that("humidity_specific2relative works as expected", {
  # vector
  expect_type(
    (test_res <- humidity_specific2relative(temperature, specific_humidity)),
    "double"
  )
  expect_length(test_res, length(temperature))

  # matrix
  temperature_matrix <- matrix(temperature, nrow = 2, ncol = 3)
  specific_humidity_matrix <- matrix(specific_humidity, nrow = 2, ncol = 3)
  expect_no_error(
    (test_res_matrix <- humidity_specific2relative(temperature_matrix, specific_humidity_matrix))
  )
  expect_true(inherits(test_res_matrix, "matrix"))
  expect_equal(dim(test_res_matrix), c(2,3))
  expect_identical(test_res, as.vector(test_res_matrix))

  # it should be no value above 100
  expect_false(any(test_res > 100))
  expect_true(any(humidity_specific2relative(temperature, specific_humidity, TRUE) > 100))

  # error when no numbers
  expect_error(
    humidity_specific2relative(letters, LETTERS),
    "must be numeric vectors"
  )
  expect_error(
    humidity_specific2relative(letters[1:6], specific_humidity),
    "must be numeric vectors"
  )
  expect_error(
    humidity_specific2relative(temperature, LETTERS[1:6]),
    "must be numeric vectors"
  )
  expect_error(
    humidity_specific2relative(temperature, specific_humidity, "TRUE"),
    "TRUE or FALSE"
  )
  expect_error(
    humidity_specific2relative(temperature, c(specific_humidity, 0.003)),
    "must be the same length"
  )
})

test_that("humidity_relative2specific works as expected", {
  # vector
  expect_type(
    (test_res <- humidity_relative2specific(temperature, relative_humidity)),
    "double"
  )
  expect_length(test_res, length(temperature))

  # matrix
  temperature_matrix <- matrix(temperature, nrow = 2, ncol = 3)
  relative_humidity_matrix <- matrix(relative_humidity, nrow = 2, ncol = 3)
  expect_no_error(
    (test_res_matrix <- humidity_relative2specific(temperature_matrix, relative_humidity_matrix))
  )
  expect_true(inherits(test_res_matrix, "matrix"))
  expect_equal(dim(test_res_matrix), c(2,3))
  expect_identical(test_res, as.vector(test_res_matrix))

  # error when no numbers
  expect_error(
    humidity_relative2specific(letters, LETTERS),
    "must be numeric vectors"
  )
  expect_error(
    humidity_relative2specific(letters[1:6], relative_humidity),
    "must be numeric vectors"
  )
  expect_error(
    humidity_relative2specific(temperature, LETTERS[1:6]),
    "must be numeric vectors"
  )
  expect_error(
    humidity_specific2relative(temperature, c(relative_humidity, 30, 100)),
    "must be the same length"
  )
})

test_that("precipitation_rainfall_erosivity works as expected", {
  meteo_data_test <- meteoland_meteo_example |>
    as.data.frame() |>
    dplyr::filter(stationID == "VC") |>
    # magrittr::set_rownames(as.Date(unique(meteoland_meteo_example$dates))) |>
    dplyr::select(-!!attr(meteoland_meteo_example, "sf_column"), -stationID, -elevation, -aspect, -slope)
  long_test <- 0.43
  scale_test <- "month"
  average_test <- TRUE

  expect_type(
    (rainfallErosivity_monthly_test <- precipitation_rainfall_erosivity(
      meteo_data_test, long_test, scale_test, average_test
    )),
    'double'
  )
  expect_length(rainfallErosivity_monthly_test, 1)
  expect_named(rainfallErosivity_monthly_test, '4')

  expect_type(
    (rainfallErosivity_yearly_test <- precipitation_rainfall_erosivity(
      meteo_data_test, long_test, "year", average_test
    )),
    'double'
  )
  expect_length(rainfallErosivity_yearly_test, 1)
  expect_named(rainfallErosivity_yearly_test, NULL)
  expect_equal(rainfallErosivity_monthly_test[[1]], rainfallErosivity_yearly_test[[1]])

  # errors
  expect_error(precipitation_rainfall_erosivity(
    "meteo_data_test", long_test, scale_test, average_test
  ))

  expect_error(precipitation_rainfall_erosivity(
    meteo_data_test, "long_test", scale_test, average_test
  ))

  expect_error(precipitation_rainfall_erosivity(
    meteo_data_test, long_test, "scale_test", average_test
  ))

  expect_error(precipitation_rainfall_erosivity(
    meteo_data_test, long_test, scale_test, "average_test"
  ))

  # more than one month
  meteo_data_months_test <- meteoland_meteo_example |>
    as.data.frame() |>
    dplyr::filter(stationID %in% c("VC", "VY")) |>
    # simulate another month
    dplyr::mutate(dates = dplyr::if_else(
      stationID == "VY",
      dates + lubridate::dmonths(1),
      dates
    )) |>
    dplyr::select(-!!attr(meteoland_meteo_example, "sf_column"), -stationID, -elevation, -aspect, -slope)

  expect_type(
    (rainfallErosivity_monthly_months_test <- precipitation_rainfall_erosivity(
      meteo_data_months_test, long_test, scale_test, average_test
    )),
    'double'
  )
  expect_length(rainfallErosivity_monthly_months_test, 2)
  expect_named(rainfallErosivity_monthly_months_test, c('4', '5'))

  expect_type(
    (rainfallErosivity_yearly_months_test <- precipitation_rainfall_erosivity(
      meteo_data_months_test, long_test, "year", average_test
    )),
    'double'
  )
  expect_length(rainfallErosivity_yearly_months_test, 1)
  expect_named(rainfallErosivity_yearly_months_test, NULL)

  # more than one year
  meteo_data_years_test <- meteoland_meteo_example |>
    as.data.frame() |>
    dplyr::filter(stationID %in% c("VC", "VY")) |>
    # simulate another month
    dplyr::mutate(dates = dplyr::if_else(
      stationID == "VY",
      dates + lubridate::dyears(1),
      dates
    )) |>
    dplyr::select(-!!attr(meteoland_meteo_example, "sf_column"), -stationID, -elevation, -aspect, -slope)

  expect_type(
    (rainfallErosivity_monthly_years_test <- precipitation_rainfall_erosivity(
      meteo_data_years_test, long_test, scale_test, average_test
    )),
    'double'
  )
  expect_length(rainfallErosivity_monthly_years_test, 1)
  expect_named(rainfallErosivity_monthly_years_test, c('4'))

  expect_type(
    (rainfallErosivity_monthly_years_noaverage_test <- precipitation_rainfall_erosivity(
      meteo_data_years_test, long_test, scale_test, FALSE
    )),
    'double'
  )
  expect_length(rainfallErosivity_monthly_years_noaverage_test, 2)
  expect_named(rainfallErosivity_monthly_years_noaverage_test, c('4', '4'))

  expect_type(
    (rainfallErosivity_yearly_years_test <- precipitation_rainfall_erosivity(
      meteo_data_years_test, long_test, 'year', average_test
    )),
    'double'
  )
  expect_length(rainfallErosivity_yearly_years_test, 1)
  expect_named(rainfallErosivity_yearly_years_test, NULL)

  expect_type(
    (rainfallErosivity_yearly_years_noaverage_test <- precipitation_rainfall_erosivity(
      meteo_data_years_test, long_test, 'year', FALSE
    )),
    'double'
  )
  expect_length(rainfallErosivity_yearly_years_noaverage_test, 2)
  expect_named(rainfallErosivity_yearly_years_noaverage_test, c('2022', '2023'))

  # mutate (recursion) tests
  expect_s3_class(
    (recursion_test <- points_to_interpolate_example |>
      interpolate_data(meteoland_interpolator_example, verbose = FALSE) |>
      dplyr::mutate(erosivity = precipitation_rainfall_erosivity(
        interpolated_data, sf::st_coordinates(geometry)[,1]
      ))),
    "tbl"
  )
  expect_true("erosivity" %in% names(recursion_test))
})

#' Ensure meteo object is ready to create an interpolator object
#'
#' Check integrity of meteo objects
#'
#' This function is the first step in the creation of a meteoland interpolator,
#' ensuring the meteo provided contains all the required elements
#'
#' @param meteo meteo object
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return invisible meteo object ready to pipe in the interpolator creation
#' @seealso Other interpolator functions: \code{\link{add_topo}()},
#' \code{\link{create_meteo_interpolator}()},
#' \code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
#' \code{\link{set_interpolation_params}()}, \code{\link{write_interpolator}()}
#'
#' @examples
#' # example meteo
#' data(meteoland_meteo_example)
#' with_meteo(meteoland_meteo_example)
#'
#' @export with_meteo
with_meteo <- function(meteo, verbose = getOption("meteoland_verbosity", TRUE)) {
  .verbosity_control(
    cli::cli_alert_info("Checking meteorology object..."),
    verbose
  )
  assertthat::assert_that(has_meteo(meteo))
  .verbosity_control(
    cli::cli_alert_success("meteorology object ok"),
    verbose
  )
  return(invisible(meteo))
}

#' Add topography data to meteo object
#'
#' Add topography data to meteo object
#'
#' When using meteo data without topography info to create an interpolator,
#' topography must be added
#'
#' @param meteo meteo object
#' @param topo topo object
#' @param verbose Logical indicating if the function must show messages and info.
#' Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#' to TRUE. It can be turned off for the function with FALSE, or session wide with
#' \code{options(meteoland_verbosity = FALSE)}
#' @return meteo with the topography info added
#' @seealso Other interpolator functions:
#' \code{\link{create_meteo_interpolator}()},
#' \code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
#' \code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()},
#' \code{\link{write_interpolator}()}
#'
#' @examples
#' # example meteo
#' data(meteoland_meteo_no_topo_example)
#' # example topo
#' data(meteoland_topo_example)
#' # add topo
#' with_meteo(meteoland_meteo_no_topo_example) |>
#'   add_topo(meteoland_topo_example)
#'
#' @export add_topo
add_topo <- function(meteo, topo, verbose = getOption("meteoland_verbosity", TRUE)) {

  assertthat::assert_that(has_meteo(meteo))
  .verbosity_control(
    cli::cli_alert_info("Checking topography object..."),
    verbose
  )
  assertthat::assert_that(has_topo(topo))
  .verbosity_control(
    cli::cli_alert_success("topography object ok"),
    verbose
  )

  .verbosity_control(
    cli::cli_alert_info("Adding topography to meteo (by station ID)..."),
    verbose
  )

  # check if meteo has topo already
  if (any(c("elevation", "aspect", "slope") %in% names(meteo))) {
    cli::cli_warn(c(
      "Topography variables found in the meteo object.",
      "They will be ignored as a new topography is provided."
    ))
    meteo <- meteo |>
      dplyr::select(!dplyr::any_of(c("elevation", "aspect", "slope")))
  }

  res <- dplyr::left_join(
    meteo,
    # ensure topo is a tibble, with unique rows
    topo |>
      dplyr::as_tibble() |>
      dplyr::select(
        dplyr::any_of(c("stationID", "elevation", "aspect", "slope"))
      ) |>
      dplyr::distinct(),
    by = 'stationID'
  ) |>
    sf::st_as_sf()

  .verbosity_control(
    cli::cli_alert_success("Topography added"),
    verbose
  )

  return(res)
}

.onAttach <- function(lib, pkg)  {
  packageStartupMessage("Package 'meteoland' [ver. ",
                        utils::packageDescription("meteoland",
                                                  fields="Version"),"]",
                        appendLF = TRUE)
}

#' meteoland: Landscape Meteorology Tools
#'
#' Functions to estimate weather variables at any position of a landscape
#'
#' @name meteoland-package
#' @aliases meteoland meteoland-package
#' @docType package
#' @author \strong{Maintainer}: Miquel De Cáceres
#' \email{miquelcaceres@@gmail.com}
#' [ORCID](https://orcid.org/0000-0001-7132-2080)
#'
#' Authors: \itemize{
#' \item{ Víctor Granda
#' [ORCID](https://orcid.org/0000-0002-0469-1991)}
#' \item{ Nicolas Martin
#' [ORCID](https://orcid.org/0000-0001-7574-0108)}
#' \item{ Antoine Cabon
#' [ORCID](https://orcid.org/0000-0001-6426-1726)}
#' }
#' @seealso Useful links: \itemize{ \item{
#' \url{https://emf-creaf.github.io/meteoland/index.html}} }
#'
#' @references De Caceres et al., 2018 (\doi{10.1016/j.envsoft.2018.08.003}))
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
# #' @importFrom utils head tail
# #' @importFrom methods as new is slot .valueClassTest coerce
# #' @importFrom grDevices topo.colors colorRampPalette
# #' @importFrom graphics lines par hist abline
# #' @import stars sf
#' @importFrom lifecycle deprecated
#' @importFrom Rcpp evalCpp
#' @importFrom Rcpp sourceCpp
#' @importFrom cubelyr tbl_cube
#' @importFrom stats aggregate complete.cases cor
#' @importFrom dplyr .data
#' @importFrom rlang :=
#' @importFrom methods .valueClassTest
#' @useDynLib meteoland, .registration = TRUE
## usethis namespace: end
NULL

#' Precipitation daily concentration
#'
#' Function \code{precipitation_concentration()} calculates daily precipitation
#' concentration (Martin-Vide et al. 2004).
#'
#' @aliases precipitation_concentration
#' @param p A numeric vector with daily precipitation values.
#' @return Function \code{precipitation_concentration()} returns a value
#' between 0 (equal distribution of rainfall) and 1 (one day concentrates all
#' rainfall).
#' @author Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF.
#' @references Martin-Vide J (2004) Spatial distribution of a daily precipitation
#' concentration index in peninsular Spain. International Journal of
#' Climatology 24, 959–971. doi:10.1002/joc.1030.
#' @export
precipitation_concentration <- function(p) {
  
  p_max = ceiling(max(p, na.rm=TRUE))
  f = cut(p, breaks=c(0.1,1:p_max), include.lowest =TRUE, right=FALSE)
  #Frequency distribution per class
  ni = table(f)
  mid = seq(0.5, p_max-0.5, by=1)
  Pi =mid*ni
  #Totals
  P_tot = sum(Pi)
  n_tot = sum(ni)
  #Cumulative values
  ni_cum = ni
  Pi_cum = Pi
  nclass = length(ni)
  if(nclass>1) {
    for(i in 2:nclass) {
      ni_cum[i] = ni_cum[i]+ni_cum[i-1]
      Pi_cum[i] = Pi_cum[i]+Pi_cum[i-1]
    }
  }
  #Relative cumulative frequencies
  X = as.vector(ni_cum/n_tot)
  Y = as.vector(Pi_cum/P_tot)
  X = X[ni>0]
  Y = Y[ni>0]
  # print(cbind(mid,ni,ni_cum,Pi, Pi_cum,X,Y))
  # plot(X,Y, type="l")
  X2 = X^2
  Y2 = Y^2
  SX = sum(X)
  SY = sum(Y)
  LX = log(X, base=exp(1))
  LY = log(Y, base=exp(1))
  SLX = sum(LX)
  SLY = sum(LY)
  SX2 = sum(X2)
  SY2 = sum(Y2)
  N = length(X)
  den = (N*SX2)-(SX^2)
  a_num = (SX2*SLY)+(SX*sum(X*LX))-(SX2*SLX)-(SX*sum(X*LY))
  b_num = (N*sum(X*LY))+(SX*SLX)- (N*sum(X*LX))-(SX*SLY)
  b =  b_num/den/100
  if(den==0) b = 0
  a = exp(a_num/den)
  if(den==0) a = 1
  A  = (a/b)*(exp(1)^(b*100))*(100-(1/b))
  if(b==0) A = 0
  S <- 5000 - A
  ci =2*S/10000
  return(ci)
}

#' Example data set for meteo data from weather stations
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Example data set of spatial location, topography and daily meteorological
#' records from 189 weather stations in Catalonia (NE Spain) corresponding to
#' April 2022.
#'
#'
#' @name meteoland_meteo_example
#' @docType data
#' @format sf object
#' @source 'Servei Meteorològic de Catalunya' (SMC)
#' @keywords datasets
#' @examples
#'
#' data(meteoland_meteo_example)
#'
NULL

#' Example data set for meteo data from weather stations, without topography
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Example data set of spatial location and daily meteorological
#' records from 189 weather stations in Catalonia (NE Spain) corresponding to
#' April 2022.
#'
#'
#' @name meteoland_meteo_no_topo_example
#' @docType data
#' @format sf object
#' @source 'Servei Meteorològic de Catalunya' (SMC)
#' @keywords datasets
#' @examples
#'
#' data(meteoland_meteo_no_topo_example)
#'
NULL

#' Example data set for topography data from weather stations, without meteo
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Example data set of spatial location and topography
#' records from 189 weather stations in Catalonia (NE Spain).
#'
#'
#' @name meteoland_topo_example
#' @docType data
#' @format sf object
#' @source 'Servei Meteorològic de Catalunya' (SMC)
#' @keywords datasets
#' @examples
#'
#' data(meteoland_topo_example)
#'
NULL

#' Example data set of points for interpolation of weather variables
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Example data set of spatial location and topography
#' records from 15 experimental plots in Catalonia (NE Spain).
#'
#'
#' @name points_to_interpolate_example
#' @docType data
#' @format sf object
#' @source Spanish National Forest Inventory
#' @keywords datasets
#' @examples
#'
#' data(points_to_interpolate_example)
#'
NULL

#' Example raster data set for interpolation of weather variables
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Example raster data set of spatial location and topography
#' records from Catalonia (NE Spain). Cell size is 1km x 1km and raster size is 10x10 cells.
#'
#'
#' @name raster_to_interpolate_example
#' @docType data
#' @format stars object
#' @source ICGC
#' @keywords datasets
#' @examples
#'
#' data(raster_to_interpolate_example)
#'
NULL

#' Example interpolator object
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Example interpolator with daily meteorological
#' records from 189 weather stations in Catalonia (NE Spain) corresponding to
#' April 2022.
#'
#'
#' @name meteoland_interpolator_example
#' @docType data
#' @format stars data cube object
#' @source Spanish National Forest Inventory
#' @keywords datasets
#' @examples
#'
#' data(meteoland_interpolator_example)
#'
NULL

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.dailyEquilibriumPET <- function(Temp, Rn) {
    .Call(`_meteoland_dailyEquilibriumPET`, Temp, Rn)
}

#' Potential evapotranspiration
#'
#' Functions to calculate potential evapotranspiration using Penman or
#' Penman-Monteith.
#'
#' The code was adapted from package `Evapotranspiration', which follows
#' McMahon et al. (2013). If wind speed is not available, an alternative
#' formulation for potential evapotranspiration is used as an approximation
#' (Valiantzas 2006)
#'
#' @aliases penman penmanmonteith
#' @param latrad Latitude in radians.
#' @param elevation Elevation (in m).
#' @param slorad Slope (in radians).
#' @param asprad Aspect (in radians from North).
#' @param J Julian day, number of days since January 1, 4713 BCE at noon UTC.
#' @param Tmax Maximum temperature (degrees Celsius).
#' @param Tmin Minimum temperature (degrees Celsius).
#' @param RHmin Minimum relative humidity (percent).
#' @param RHmax Maximum relative humidity (percent).
#' @param R_s Solar radiation (MJ/m2).
#' @param u With wind speed (m/s).
#' @param z Wind measuring height (m).
#' @param z0 Roughness height (m).
#' @param alpha Albedo.
#' @param windfun Wind speed function version, either "1948" or "1956".
#' @param rc Canopy vapour flux (stomatal) resistance (s·m-1).
#' @param Rn Daily net radiation (MJ·m-2·day-1).
#' @return Potential evapotranspiration (in mm of water).
#' @author Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
#' @seealso \code{\link{interpolate_data}}
#' @references Penman, H. L. 1948. Natural evaporation from open water, bare
#' soil and grass. Proceedings of the Royal Society of London. Series A.
#' Mathematical and Physical Sciences, 193, 120-145.
#'
#' Penman, H. L. 1956. Evaporation: An introductory survey. Netherlands Journal
#' of Agricultural Science, 4, 9-29.
#'
#' McMahon, T.A., Peel, M.C., Lowe, L., Srikanthan, R., McVicar, T.R. 2013.
#' Estimating actual, potential, reference crop and pan evaporation using
#' standard meteorological data: a pragmatic synthesis. Hydrology & Earth
#' System Sciences 17, 1331–1363. doi:10.5194/hess-17-1331-2013.
#' @export
penman <- function(latrad, elevation, slorad, asprad, J, Tmin, Tmax, RHmin, RHmax, R_s, u, z = 10.0, z0 = 0.001, alpha = 0.25, windfun = "1956") {
    .Call(`_meteoland_PenmanPET`, latrad, elevation, slorad, asprad, J, Tmin, Tmax, RHmin, RHmax, R_s, u, z, z0, alpha, windfun)
}

#' @describeIn penman Penman Monteith method
#' @export
penmanmonteith <- function(rc, elevation, Tmin, Tmax, RHmin, RHmax, Rn, u = NA_real_) {
    .Call(`_meteoland_PenmanMonteithPET`, rc, elevation, Tmin, Tmax, RHmin, RHmax, Rn, u)
}

.penmanpoint <- function(latrad, elevation, slorad, asprad, J, Tmin, Tmax, RHmin, RHmax, R_s, u, z = 10.0, z0 = 0.001, alpha = 0.25, windfun = "1956") {
    .Call(`_meteoland_PenmanPETPointSeries`, latrad, elevation, slorad, asprad, J, Tmin, Tmax, RHmin, RHmax, R_s, u, z, z0, alpha, windfun)
}

.penmanmonteithpoint <- function(rc, elevation, Tmin, Tmax, RHmin, RHmax, Rn, u) {
    .Call(`_meteoland_PenmanMonteithPETPointSeries`, rc, elevation, Tmin, Tmax, RHmin, RHmax, Rn, u)
}

.PenmanPETPointsDay <- function(latrad, elevation, slorad, asprad, J, Tmin, Tmax, RHmin, RHmax, R_s, u, z = 10.0, z0 = 0.001, alpha = 0.25, windfun = "1956") {
    .Call(`_meteoland_PenmanPETPointsDay`, latrad, elevation, slorad, asprad, J, Tmin, Tmax, RHmin, RHmax, R_s, u, z, z0, alpha, windfun)
}

.temporalSmoothing <- function(input, numDays, prec) {
    .Call(`_meteoland_temporalSmoothing`, input, numDays, prec)
}

.slope <- function(data, nrows, ncols, cellWidth, cellHeight) {
    .Call(`_meteoland_slope`, data, nrows, ncols, cellWidth, cellHeight)
}

.aspect <- function(data, nrows, ncols, cellWidth, cellHeight) {
    .Call(`_meteoland_aspect`, data, nrows, ncols, cellWidth, cellHeight)
}

#' @describeIn interpolation_temperature Precipitation 
#' @export
interpolation_precipitation <- function(Xp, Yp, Zp, X, Y, Z, P, Psmooth, iniRp = 140000, alpha_event = 6.25, alpha_amount = 6.25, N_event = 20L, N_amount = 20L, iterations = 3L, popcrit = 0.5, fmax = 0.95, debug = FALSE) {
    .Call(`_meteoland_interpolatePrecipitationPoints`, Xp, Yp, Zp, X, Y, Z, P, Psmooth, iniRp, alpha_event, alpha_amount, N_event, N_amount, iterations, popcrit, fmax, debug)
}

.interpolatePrecipitationSeriesPoints <- function(Xp, Yp, Zp, X, Y, Z, P, Psmooth, iniRp = 140000, alpha_event = 6.25, alpha_amount = 6.25, N_event = 20L, N_amount = 20L, iterations = 3L, popcrit = 0.5, fmax = 0.95, debug = FALSE) {
    .Call(`_meteoland_interpolatePrecipitationSeriesPoints`, Xp, Yp, Zp, X, Y, Z, P, Psmooth, iniRp, alpha_event, alpha_amount, N_event, N_amount, iterations, popcrit, fmax, debug)
}

.interpolatePrecipitationEventSeriesPoints <- function(Xp, Yp, Zp, X, Y, Z, Pevent, iniRp = 140000, alpha = 6.25, N = 20L, iterations = 3L, popcrit = 0.5) {
    .Call(`_meteoland_interpolatePrecipitationEventSeriesPoints`, Xp, Yp, Zp, X, Y, Z, Pevent, iniRp, alpha, N, iterations, popcrit)
}

.pseudoRainfall <- function(RainM, daysMonthAll, shape = 2.0, scale = 4.0, firstMonth = 1L) {
    .Call(`_meteoland_pseudoRainfall`, RainM, daysMonthAll, shape, scale, firstMonth)
}

#' Solar radiation utility functions
#'
#' Set of functions used in the calculation of incoming solar radiation and net
#' radiation.
#'
#'
#' @aliases radiation_dateStringToJulianDays radiation_daylength radiation_daylengthseconds radiation_directDiffuseInstant radiation_directDiffuseDay radiation_potentialRadiation radiation_julianDay radiation_skyLongwaveRadiation radiation_outgoingLongwaveRadiation radiation_netRadiation radiation_solarRadiation radiation_solarConstant radiation_solarElevation radiation_solarDeclination radiation_sunRiseSet
#' @param dateStrings A character vector with dates in format "YYYY-MM-DD".
#' @param latrad Latitude (in radians North).
#' @param slorad Slope (in radians).
#' @param asprad Aspect (in radians from North).
#' @param delta Solar declination (in radians).
#' @param solarConstant Solar constant (in kW·m-2).
#' @param hrad Solar hour (in radians).
#' @param R_s Daily incident solar radiation (MJ·m-2).
#' @param clearday Boolean flag to indicate a clearsky day (vs. overcast).
#' @param nsteps Number of daily substeps.
#' @param J Julian day (integer), number of days since January 1, 4713 BCE at
#' noon UTC.
#' @param year,month,day Year, month and day as integers.
#' @param alpha Surface albedo (from 0 to 1).
#' @param Tair Air temperature (in degrees Celsius).
#' @param vpa Average daily vapor pressure (kPa).
#' @param c Proportion of sky covered by clouds (0-1).
#' @param tmin,tmax Minimum and maximum daily temperature (ºC).
#' @param elevation Elevation above sea level (in m).
#' @param precipitation Precipitation (in mm).
#' @param diffTemp Difference between maximum and minimum temperature (ºC).
#' @param diffTempMonth Difference between maximum and minimum temperature,
#' averaged over 30 days (ºC).
#' @return Values returned for each function are: \itemize{
#' \item\code{radiation_dateStringToJulianDays}: A vector of Julian days (i.e.
#' number of days since January 1, 4713 BCE at noon UTC).
#' \item\code{radiation_daylength}: Day length (in hours).
#' \item\code{radiation_daylengthseconds}: Day length (in seconds).
#' \item\code{radiation_directDiffuseInstant}: A vector with instantaneous
#' direct and diffusive radiation rates (for both SWR and PAR).
#' \item\code{radiation_directDiffuseDay}: A data frame with instantaneous
#' direct and diffusive radiation rates (for both SWR and PAR) for each
#' subdaily time step. \item\code{radiation_potentialRadiation}: Daily
#' (potential) solar radiation (in MJ·m-2). \item\code{radiation_julianDay}:
#' Number of days since January 1, 4713 BCE at noon UTC.
#' \item\code{radiation_skyLongwaveRadiation}: Instantaneous incoming (sky)
#' longwave radiation (W·m-2).
#' \item\code{radiation_outgoingLongwaveRadiation}: Daily outgoing longwave
#' radiation (MJ·m-2·day-1).  \item\code{radiation_netRadiation}: Daily net
#' solar radiation (MJ·m-2·day-1).  \item\code{radiation_solarConstant}: Solar
#' constant (in kW·m-2). \item\code{radiation_solarDeclination}: Solar
#' declination (in radians). \item\code{radiation_solarElevation}: Angle of
#' elevation of the sun with respect to the horizon (in radians).
#' \item\code{radiation_solarRadiation}: Daily incident solar radiation
#' (MJ·m-2·day-1). \item\code{radiation_sunRiseSet}: Sunrise and sunset hours
#' in hour angle (radians). }
#' @note Code for \code{radiation_julianDay()},
#' \code{radiation_solarConstant()} and \code{radiation_solarDeclination()} was
#' translated to C++ from R code in package 'insol' (by J. G. Corripio).
#' @author Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
#' @seealso \code{\link{interpolate_data}}
#' @references Danby, J. M. Eqn. 6.16.4 in Fundamentals of Celestial Mechanics,
#' 2nd ed. Richmond, VA: Willmann-Bell, p. 207, 1988.
#'
#' Garnier, B.J., Ohmura, A., 1968. A method of calculating the direct
#' shortwave radiation income of slopes. J. Appl. Meteorol. 7: 796-800
#'
#' McMahon, T. A., M. C. Peel, L. Lowe, R. Srikanthan, and T. R. McVicar. 2013.
#' Estimating actual, potential, reference crop and pan evaporation using
#' standard meteorological data: a pragmatic synthesis. Hydrology & Earth
#' System Sciences 17:1331–1363. See also:
#' http://www.fao.org/docrep/x0490e/x0490e06.htm.
#'
#' Reda, I. and Andreas, A. 2003. Solar Position Algorithm for Solar Radiation
#' Applications. 55 pp.; NREL Report No. TP-560-34302, Revised January 2008.
#' http://www.nrel.gov/docs/fy08osti/34302.pdf
#'
#' Spitters, C.J.T., Toussaint, H.A.J.M. and Goudriaan, J. (1986). Separating
#' the diffuse and direct components of global radiation and its implications
#' for modeling canopy photosynthesis. I. Components of incoming radiation.
#' Agricultural and Forest Meteorology, 38, 231–242.
#' @export
radiation_julianDay <- function(year, month, day) {
    .Call(`_meteoland_julianDay`, year, month, day)
}

#' @describeIn radiation_julianDay Date string to julian days
#' @export
radiation_dateStringToJulianDays <- function(dateStrings) {
    .Call(`_meteoland_dateStringToJulianDays`, dateStrings)
}

#' @describeIn radiation_julianDay solar declination
#' @export
radiation_solarDeclination <- function(J) {
    .Call(`_meteoland_solarDeclination`, J)
}

#' @describeIn radiation_julianDay solar constant
#' @export
radiation_solarConstant <- function(J) {
    .Call(`_meteoland_solarConstant`, J)
}

#' @describeIn radiation_julianDay sun rise and set
#' @export
radiation_sunRiseSet <- function(latrad, slorad, asprad, delta) {
    .Call(`_meteoland_sunRiseSet`, latrad, slorad, asprad, delta)
}

#' @describeIn radiation_julianDay solar elevation
#' @export
radiation_solarElevation <- function(latrad, delta, hrad) {
    .Call(`_meteoland_solarElevation`, latrad, delta, hrad)
}

#' @describeIn radiation_julianDay Day length
#' @export
radiation_daylength <- function(latrad, slorad, asprad, delta) {
    .Call(`_meteoland_daylength`, latrad, slorad, asprad, delta)
}

#' @describeIn radiation_julianDay Day length seconds
#' @export
radiation_daylengthseconds <- function(latrad, slorad, asprad, delta) {
    .Call(`_meteoland_daylengthseconds`, latrad, slorad, asprad, delta)
}

#' @describeIn radiation_julianDay Potential radiation
#' @export
radiation_potentialRadiation <- function(solarConstant, latrad, slorad, asprad, delta) {
    .Call(`_meteoland_RpotDay`, solarConstant, latrad, slorad, asprad, delta)
}

#' @describeIn radiation_julianDay solar Radiation
#' @export
radiation_solarRadiation <- function(solarConstant, latrad, elevation, slorad, asprad, delta, diffTemp, diffTempMonth, vpa, precipitation) {
    .Call(`_meteoland_RDay`, solarConstant, latrad, elevation, slorad, asprad, delta, diffTemp, diffTempMonth, vpa, precipitation)
}

#' @describeIn radiation_julianDay Direct diffuse instant
#' @export
radiation_directDiffuseInstant <- function(solarConstant, latrad, slorad, asprad, delta, hrad, R_s, clearday) {
    .Call(`_meteoland_directDiffuseInstant`, solarConstant, latrad, slorad, asprad, delta, hrad, R_s, clearday)
}

#' @describeIn radiation_julianDay Direct diffuse day
#' @export
radiation_directDiffuseDay <- function(solarConstant, latrad, slorad, asprad, delta, R_s, clearday, nsteps = 24L) {
    .Call(`_meteoland_directDiffuseDay`, solarConstant, latrad, slorad, asprad, delta, R_s, clearday, nsteps)
}

#' @describeIn radiation_julianDay Sky longwave radiation
#' @export
radiation_skyLongwaveRadiation <- function(Tair, vpa, c = 0) {
    .Call(`_meteoland_skyLongwaveRadiation`, Tair, vpa, c)
}

#' @describeIn radiation_julianDay Outgoing longwave radiation
#' @export
radiation_outgoingLongwaveRadiation <- function(solarConstant, latrad, elevation, slorad, asprad, delta, vpa, tmin, tmax, R_s) {
    .Call(`_meteoland_outgoingLongwaveRadiation`, solarConstant, latrad, elevation, slorad, asprad, delta, vpa, tmin, tmax, R_s)
}

#' @describeIn radiation_julianDay Net radiation
#' @export
radiation_netRadiation <- function(solarConstant, latrad, elevation, slorad, asprad, delta, vpa, tmin, tmax, R_s, alpha = 0.08) {
    .Call(`_meteoland_netRadiation`, solarConstant, latrad, elevation, slorad, asprad, delta, vpa, tmin, tmax, R_s, alpha)
}

.potentialRadiationSeries <- function(latrad, slorad, asprad, J) {
    .Call(`_meteoland_potentialRadiationSeries`, latrad, slorad, asprad, J)
}

.potentialRadiationPoints <- function(latrad, slorad, asprad, J) {
    .Call(`_meteoland_potentialRadiationPoints`, latrad, slorad, asprad, J)
}

.radiationSeries <- function(latrad, elevation, slorad, asprad, J, diffTemp, diffTempMonth, VP, P) {
    .Call(`_meteoland_radiationSeries`, latrad, elevation, slorad, asprad, J, diffTemp, diffTempMonth, VP, P)
}

.radiationPoints <- function(latrad, elevation, slorad, asprad, J, diffTemp, diffTempMonth, VP, P) {
    .Call(`_meteoland_radiationPoints`, latrad, elevation, slorad, asprad, J, diffTemp, diffTempMonth, VP, P)
}

.meteo <- function(MeteoMonth, landscapeRainfall = as.numeric( c()), ERconv = 0.05, ERsyn = 0.2, shape = 2.0, scale = 4.0, albedo = 0.17, firstMonth = 1L, cyclic = FALSE) {
    .Call(`_meteoland_meteo`, MeteoMonth, landscapeRainfall, ERconv, ERsyn, shape, scale, albedo, firstMonth, cyclic)
}

.vapourPressureFromRH <- function(T, RH) {
    .Call(`_meteoland_vapourPressureFromRH`, T, RH)
}

.dewpointTemperatureFromRH <- function(T, RH) {
    .Call(`_meteoland_dewpointTemperatureFromRH`, T, RH)
}

.temp2SVP <- function(TD) {
    .Call(`_meteoland_temp2SVP`, TD)
}

.relativeHumidityFromMinMaxTemp <- function(Tmin, Tmax) {
    .Call(`_meteoland_relativeHumidityFromMinMaxTemp`, Tmin, Tmax)
}

.relativeHumidityFromDewpointTemp <- function(T, TD) {
    .Call(`_meteoland_relativeHumidityFromDewpointTemp`, T, TD)
}

#' @describeIn interpolation_temperature Dew temperature 
#' @export
interpolation_dewtemperature <- function(Xp, Yp, Zp, X, Y, Z, T, iniRp = 140000, alpha = 3.0, N = 30L, iterations = 3L, debug = FALSE) {
    .Call(`_meteoland_interpolateTdewPoints`, Xp, Yp, Zp, X, Y, Z, T, iniRp, alpha, N, iterations, debug)
}

.interpolateTdewSeriesPoints <- function(Xp, Yp, Zp, X, Y, Z, T, iniRp = 140000, alpha = 3.0, N = 30L, iterations = 3L, debug = FALSE) {
    .Call(`_meteoland_interpolateTdewSeriesPoints`, Xp, Yp, Zp, X, Y, Z, T, iniRp, alpha, N, iterations, debug)
}

#' Low-level interpolation functions
#'
#' Low-level functions to interpolate meteorology (one day) on a set of points.
#'
#' @details
#' This functions exposes internal low-level interpolation functions written in C++
#' not intended to be used directly in any script or function. The are maintained for
#' compatibility with older versions of the package and future versions of meteoland
#' will remove this functions (they will be still accessible through the triple colon
#' notation (\code{:::}), but their use is not recommended)
#'
#'
#' @aliases interpolation_dewtemperature interpolation_temperature interpolation_precipitation interpolation_wind
#' @param Xp,Yp,Zp Spatial coordinates and elevation (Zp; in m.a.s.l) of target
#' points.
#' @param X,Y,Z Spatial coordinates and elevation (Zp; in m.a.s.l) of reference
#' locations (e.g. meteorological stations).
#' @param T Temperature (e.g., minimum, maximum or dew temperature) at the
#' reference locations (in degrees).
#' @param P Precipitation at the reference locations (in mm).
#' @param Psmooth Temporally-smoothed precipitation at the reference locations
#' (in mm).
#' @param WS,WD Wind speed (in m/s) and wind direction (in degrees from north
#' clock-wise) at the reference locations.
#' @param iniRp Initial truncation radius.
#' @param iterations Number of station density iterations.
#' @param debug Boolean flag to show extra console output.
#' @param alpha,alpha_amount,alpha_event Gaussian shape parameter.
#' @param N,N_event,N_amount Average number of stations with non-zero weights.
#' @param popcrit Critical precipitation occurrence parameter.
#' @param fmax Maximum value for precipitation regression extrapolations (0.6
#' equals to a maximum of 4 times extrapolation).
#' @param directionsAvailable A flag to indicate that wind directions are
#' available (i.e. non-missing) at the reference locations.
#' @return All functions return a vector with interpolated values for the
#' target points.
#' @author Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
#' @seealso \code{\link{defaultInterpolationParams}}
#' @references Thornton, P.E., Running, S.W., White, M. A., 1997. Generating
#' surfaces of daily meteorological variables over large regions of complex
#' terrain. J. Hydrol. 190, 214–251. doi:10.1016/S0022-1694(96)03128-9.
#'
#' De Caceres M, Martin-StPaul N, Turco M, Cabon A, Granda V (2018) Estimating
#' daily meteorological data and downscaling climate models over landscapes.
#' Environmental Modelling and Software 108: 186-196.
#' @examples
#'
#' Xp <- as.numeric(sf::st_coordinates(points_to_interpolate_example)[,1])
#' Yp <- as.numeric(sf::st_coordinates(points_to_interpolate_example)[,2])
#' Zp <- points_to_interpolate_example$elevation
#' X <- as.numeric(
#'   sf::st_coordinates(stars::st_get_dimension_values(meteoland_interpolator_example, "station"))[,1]
#' )
#' Y <- as.numeric(
#'   sf::st_coordinates(stars::st_get_dimension_values(meteoland_interpolator_example, "station"))[,2]
#' )
#' Z <- as.numeric(meteoland_interpolator_example[["elevation"]][1,])
#' Temp <- as.numeric(meteoland_interpolator_example[["MinTemperature"]][1,])
#' P <- as.numeric(meteoland_interpolator_example[["Precipitation"]][1,])
#' Psmooth <- as.numeric(meteoland_interpolator_example[["SmoothedPrecipitation"]][1,])
#' WS <- as.numeric(meteoland_interpolator_example[["WindSpeed"]][1,])
#' WD <- as.numeric(meteoland_interpolator_example[["WindDirection"]][1,])
#' iniRp <- get_interpolation_params(meteoland_interpolator_example)$initial_Rp
#' alpha <- get_interpolation_params(meteoland_interpolator_example)$alpha_MinTemperature
#' N <- get_interpolation_params(meteoland_interpolator_example)$N_MinTemperature
#' alpha_event <- get_interpolation_params(meteoland_interpolator_example)$alpha_PrecipitationEvent
#' N_event <- get_interpolation_params(meteoland_interpolator_example)$N_PrecipitationEvent
#' alpha_amount <- get_interpolation_params(meteoland_interpolator_example)$alpha_PrecipitationAmount
#' N_amount <- get_interpolation_params(meteoland_interpolator_example)$N_PrecipitationAmount
#' alpha_wind <- get_interpolation_params(meteoland_interpolator_example)$alpha_Wind
#' N_wind <- get_interpolation_params(meteoland_interpolator_example)$N_Wind
#' iterations <- get_interpolation_params(meteoland_interpolator_example)$iterations
#' popcrit <- get_interpolation_params(meteoland_interpolator_example)$pop_crit
#' fmax <- get_interpolation_params(meteoland_interpolator_example)$f_max
#' debug <- get_interpolation_params(meteoland_interpolator_example)$debug
#'
#' interpolation_temperature(
#'   Xp, Yp, Zp,
#'   X[!is.na(Temp)], Y[!is.na(Temp)], Z[!is.na(Temp)],
#'   Temp[!is.na(Temp)],
#'   iniRp, alpha, N, iterations, debug
#' )
#'
#' interpolation_wind(
#'   Xp, Yp,
#'   WS[!is.na(WD)], WD[!is.na(WD)],
#'   X[!is.na(WD)], Y[!is.na(WD)],
#'   iniRp, alpha_wind, N_wind, iterations, directionsAvailable = FALSE
#' )
#'
#' interpolation_precipitation(
#'   Xp, Yp, Zp,
#'   X[!is.na(P)], Y[!is.na(P)], Z[!is.na(P)],
#'   P[!is.na(P)], Psmooth[!is.na(P)],
#'   iniRp, alpha_event, alpha_amount, N_event, N_amount,
#'   iterations, popcrit, fmax, debug
#' )
#'
#' @export
interpolation_temperature <- function(Xp, Yp, Zp, X, Y, Z, T, iniRp = 140000, alpha = 3.0, N = 30L, iterations = 3L, debug = FALSE) {
    .Call(`_meteoland_interpolateTemperaturePoints`, Xp, Yp, Zp, X, Y, Z, T, iniRp, alpha, N, iterations, debug)
}

.interpolateTemperatureSeriesPoints <- function(Xp, Yp, Zp, X, Y, Z, T, iniRp = 140000, alpha = 3.0, N = 30L, iterations = 3L, debug = FALSE) {
    .Call(`_meteoland_interpolateTemperatureSeriesPoints`, Xp, Yp, Zp, X, Y, Z, T, iniRp, alpha, N, iterations, debug)
}

#' Physical utility functions
#'
#' Set of functions used in the calculation of physical variables.
#'
#'
#' @aliases utils_airDensity utils_atmosphericPressure utils_averageDailyVP utils_averageDaylightTemperature utils_latentHeatVaporisation utils_latentHeatVaporisationMol utils_psychrometricConstant utils_saturationVP utils_saturationVaporPressureCurveSlope
#' @param temperature Air temperature (ºC).
#' @param Tmin,Tmax Minimum and maximum daily temperature (ºC).
#' @param RHmin,RHmax Minimum and maximum relative humidity (%).
#' @param Patm Atmospheric air pressure (in kPa).
#' @param elevation Elevation above sea level (in m).
#' @return Values returned for each function are: \itemize{
#' \item{\code{utils_airDensity}: air density (in kg·m-3).}
#' \item{\code{utils_atmosphericPressure}: Air atmospheric pressure (in kPa).}
#' \item{\code{utils_averageDailyVP}: average (actual) vapour pressure (in kPa).}
#' \item{\code{utils_averageDaylightTemperature}: average daylight air
#' temperature (in ºC). \item\code{utils_latentHeatVaporisation}: Latent heat
#' of vaporisation (MJ·kg-1).  \item\code{utils_latentHeatVaporisationMol}:
#' Latent heat of vaporisation (J·mol-1).}
#' \item{\code{utils_psychrometricConstant}: Psychrometric constant (kPa·ºC-1).}
#' \item{\code{utils_saturationVP}: saturation vapour pressure (in kPa).}
#' \item{\code{utils_saturationVaporPressureCurveSlope}: Slope of the saturation
#' vapor pressure curve (kPa·ºC-1).}  }
#' @author Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
#' @references McMurtrie, R. E., D. A. Rook, and F. M. Kelliher. 1990.
#' Modelling the yield of Pinus radiata on a site limited by water and
#' nitrogen. Forest Ecology and Management 30:381–413.
#'
#' McMahon, T. A., M. C. Peel, L. Lowe, R. Srikanthan, and T. R. McVicar. 2013.
#' Estimating actual, potential, reference crop and pan evaporation using
#' standard meteorological data: a pragmatic synthesis. Hydrology & Earth
#' System Sciences 17:1331–1363. See also:
#' http://www.fao.org/docrep/x0490e/x0490e06.htm
#' @export
utils_saturationVP <- function(temperature) {
    .Call(`_meteoland_saturationVapourPressure`, temperature)
}

#' @describeIn utils_saturationVP Average daily VP
#' @export
utils_averageDailyVP <- function(Tmin, Tmax, RHmin, RHmax) {
    .Call(`_meteoland_averageDailyVapourPressure`, Tmin, Tmax, RHmin, RHmax)
}

#' @describeIn utils_saturationVP Atmospheric pressure
#' @export
utils_atmosphericPressure <- function(elevation) {
    .Call(`_meteoland_atmosphericPressure`, elevation)
}

#' @describeIn utils_saturationVP Air density
#' @export
utils_airDensity <- function(temperature, Patm) {
    .Call(`_meteoland_airDensity`, temperature, Patm)
}

#' @describeIn utils_saturationVP Daylight temperature
#' @export
utils_averageDaylightTemperature <- function(Tmin, Tmax) {
    .Call(`_meteoland_averageDaylightTemperature`, Tmin, Tmax)
}

#' @describeIn utils_saturationVP latent heat vaporisation
#' @export
utils_latentHeatVaporisation <- function(temperature) {
    .Call(`_meteoland_latentHeatVaporisation`, temperature)
}

#' @describeIn utils_saturationVP Heat vaporisation mol
#' @export
utils_latentHeatVaporisationMol <- function(temperature) {
    .Call(`_meteoland_latentHeatVaporisationMol`, temperature)
}

#' @describeIn utils_saturationVP psychrometric constant
#' @export
utils_psychrometricConstant <- function(temperature, Patm) {
    .Call(`_meteoland_psychrometricConstant`, temperature, Patm)
}

#' @describeIn utils_saturationVP Saturation VP curve slope
#' @export
utils_saturationVaporPressureCurveSlope <- function(temperature) {
    .Call(`_meteoland_saturationVaporPressureCurveSlope`, temperature)
}

.getWindFieldIndexAndFactor <- function(windSpeed, windDirection, wfSpeed, wfDirection) {
    .Call(`_meteoland_getWindFieldIndexAndFactor`, windSpeed, windDirection, wfSpeed, wfDirection)
}

#' @describeIn interpolation_temperature Wind 
#' @export
interpolation_wind <- function(Xp, Yp, WS, WD, X, Y, iniRp = 140000, alpha = 2.0, N = 1L, iterations = 3L, directionsAvailable = TRUE) {
    .Call(`_meteoland_interpolateWindStationPoints`, Xp, Yp, WS, WD, X, Y, iniRp, alpha, N, iterations, directionsAvailable)
}

.interpolateWindFieldSeriesPoints <- function(Xp, Yp, WS, WD, X, Y, I, F, iniRp = 140000, alpha = 3.0, N = 30L, iterations = 3L) {
    .Call(`_meteoland_interpolateWindFieldSeriesPoints`, Xp, Yp, WS, WD, X, Y, I, F, iniRp, alpha, N, iterations)
}

.interpolateWindStationSeriesPoints <- function(Xp, Yp, WS, WD, X, Y, iniRp = 140000, alpha = 3.0, N = 30L, iterations = 3L) {
    .Call(`_meteoland_interpolateWindStationSeriesPoints`, Xp, Yp, WS, WD, X, Y, iniRp, alpha, N, iterations)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_meteoland_RcppExport_registerCCallable`)
})

# test data needed


## Radiation functions tests ####
test_that("radiation c internals work as expected", {

  # data for radiation test
  year_test <- 2022
  month_test <- 4
  day_test <- 25
  dateStrings_test <- c("2022-04-25", "2022-04-26")
  J_test <- 2459695
  latrad_test <- 42 * (pi / 180)
  slorad_test <- 2 * (pi / 180)
  asprad_test <- 270 * (pi / 180)
  delta_test <- 0.2288457
  hrad_test <- -1.734900
  solarConstant_test <- 1.352879
  elevation_test <- 900
  diffTemp_test <- 16
  diffTempMonth_test <- 13
  vpa_test <- 2.5
  precipitation_test <- 15
  R_s_test <- 6
  clearday_test <- TRUE
  nsteps_test <- 12L
  Tair_test <- 25
  c_test <- 0.2
  tmin_test <- 5
  tmax_test <- 5 + 16
  alpha_test <- 0.08

  ## radiation_julianDay
  expect_type(
    (julianDay_test <- radiation_julianDay(year_test, month_test, day_test)),
    "integer"
  )
  expect_error(
    radiation_julianDay("2022", "April", "25")
  )

  # radiation_dateStringToJulianDays
  expect_type(
    (dateStringToJulianDays_test <- radiation_dateStringToJulianDays(dateStrings_test)),
    "integer"
  )
  expect_error(
    radiation_dateStringToJulianDays(C("tururu", "larara"))
  )
  expect_identical(julianDay_test, dateStringToJulianDays_test[1])

  # radiation_solarDeclination
  expect_type(
    (solarDeclination_test <- radiation_solarDeclination(J_test)),
    "double"
  )
  expect_error(
    radiation_solarDeclination("tururu")
  )
  expect_error(
    radiation_solarDeclination(c(J_test, J_test + 1))
  )

  # radiation_solarConstant
  expect_type(
    (solarConstant_test <- radiation_solarConstant(J_test)),
    "double"
  )
  expect_error(
    radiation_solarConstant("tururu")
  )
  expect_error(
    radiation_solarConstant(c(J_test, J_test + 1))
  )

  # radiation_sunRiseSet
  expect_type(
    (sunRiseSet_test <- radiation_sunRiseSet(latrad_test, slorad_test, asprad_test, delta_test)),
    "double"
  )
  expect_error(
    radiation_sunRiseSet("tururu", slorad_test, asprad_test, delta_test)
  )
  expect_error(
    radiation_sunRiseSet(c(latrad_test, latrad_test), slorad_test, asprad_test, delta_test)
  )

  # radiation_solarElevation
  expect_type(
    (solarElevation_test <- radiation_solarElevation(latrad_test, delta_test, hrad_test)),
    "double"
  )
  expect_error(
    radiation_solarElevation("tururu", delta_test, hrad_test)
  )
  expect_error(
    radiation_solarElevation(c(latrad_test, latrad_test), delta_test, hrad_test)
  )

  # radiation_daylength
  expect_type(
    (daylength_test <- radiation_daylength(latrad_test, slorad_test, asprad_test, delta_test)),
    "double"
  )
  expect_error(
    radiation_daylength("tururu", slorad_test, asprad_test, delta_test)
  )
  expect_error(
    radiation_daylength(c(latrad_test, latrad_test), slorad_test, asprad_test, delta_test)
  )

  # radiation_daylengthseconds
  expect_type(
    (daylengthseconds_test <- radiation_daylengthseconds(latrad_test, slorad_test, asprad_test, delta_test)),
    "double"
  )
  expect_error(
    radiation_daylengthseconds("tururu", slorad_test, asprad_test, delta_test)
  )
  expect_error(
    radiation_daylengthseconds(c(latrad_test, latrad_test), slorad_test, asprad_test, delta_test)
  )

  # radiation_potentialRadiation
  expect_type(
    (potentialRadiation_test <- radiation_potentialRadiation(
      solarConstant_test, latrad_test, slorad_test, asprad_test, delta_test
    )),
    "double"
  )
  expect_error(
    radiation_potentialRadiation(
      "tururu", latrad_test, slorad_test, asprad_test, delta_test
    )
  )
  expect_error(
    radiation_potentialRadiation(
      c(solarConstant_test, solarConstant_test), latrad_test, slorad_test, asprad_test, delta_test
    )
  )

  # radiation_solarRadiation
  expect_type(
    (solarRadiation_test <- radiation_solarRadiation(
      solarConstant_test, latrad_test, elevation_test, slorad_test, asprad_test,
      delta_test, diffTemp_test, diffTempMonth_test, vpa_test, precipitation_test
    )),
    "double"
  )
  expect_error(
    radiation_solarRadiation(
      "tururu", latrad_test, elevation_test, slorad_test, asprad_test,
      delta_test, diffTemp_test, diffTempMonth_test, vpa_test, precipitation_test
    )
  )
  expect_error(
    radiation_solarRadiation(
      c(solarConstant_test, solarConstant_test), latrad_test, elevation_test,
      slorad_test, asprad_test, delta_test, diffTemp_test, diffTempMonth_test,
      vpa_test, precipitation_test
    )
  )

  # radiation_directDiffuseInstant
  expect_type(
    (directDiffuseInstant_test <- radiation_directDiffuseInstant(
      solarConstant_test, latrad_test, slorad_test, asprad_test,
      delta_test, hrad_test, R_s_test, clearday_test
    )),
    "double"
  )
  expect_length(directDiffuseInstant_test, 8)
  expect_named(directDiffuseInstant_test, c(
    "SolarElevation", "Rpot", "Rpot_flat", "Rg",
    "SWR_direct", "SWR_diffuse", "PAR_direct", "PAR_diffuse"
  ))
  expect_error(
    radiation_directDiffuseInstant(
      "tururu", latrad_test, slorad_test, asprad_test,
      delta_test, hrad_test, R_s_test, clearday_test
    )
  )
  expect_error(
    radiation_directDiffuseInstant(
      c(solarConstant_test, solarConstant_test), latrad_test, slorad_test, asprad_test,
      delta_test, hrad_test, R_s_test, clearday_test
    )
  )

  # radiation_directDiffuseDay
  expect_s3_class(
    (directDiffuseDay_test <- radiation_directDiffuseDay(
      solarConstant_test, latrad_test, slorad_test, asprad_test,
      delta_test, R_s_test, clearday_test, nsteps_test
    )),
    "data.frame"
  )
  expect_identical(nrow(directDiffuseDay_test), nsteps_test)
  expect_named(directDiffuseDay_test, c(
    "SolarHour", "SolarElevation", "Rpot", "Rpot_flat", "Rg",
    "SWR_direct", "SWR_diffuse", "PAR_direct", "PAR_diffuse"
  ))
  expect_error(
    radiation_directDiffuseDay(
      "tururu", latrad_test, slorad_test, asprad_test,
      delta_test, R_s_test, clearday_test, nsteps_test
    )
  )
  expect_error(
    radiation_directDiffuseDay(
      c(solarConstant_test, solarConstant_test), latrad_test, slorad_test, asprad_test,
      delta_test, R_s_test, clearday_test, nsteps_test
    )
  )

  # radiation_skyLongwaveRadiation
  expect_type(
    (skyLongwaveRadiation_test <- radiation_skyLongwaveRadiation(Tair_test, vpa_test, c_test)),
    "double"
  )
  expect_error(
    radiation_skyLongwaveRadiation("tururu", vpa_test, c_test)
  )
  expect_error(
    radiation_skyLongwaveRadiation(c(Tair_test, Tair_test), vpa_test, c_test)
  )

  # radiation_outgoingLongwaveRadiation
  expect_type(
    (outgoingLongwaveRadiation_test <- radiation_outgoingLongwaveRadiation(
      solarConstant_test, latrad_test, elevation_test, slorad_test, asprad_test,
      delta_test, vpa_test, tmin_test, tmax_test, R_s_test
    )),
    "double"
  )
  expect_error(
    radiation_outgoingLongwaveRadiation(
      "tururu", latrad_test, elevation_test, slorad_test, asprad_test,
      delta_test, vpa_test, tmin_test, tmax_test, R_s_test
    )
  )
  expect_error(
    radiation_outgoingLongwaveRadiation(
      c(solarConstant_test, solarConstant_test), latrad_test, elevation_test,
      slorad_test, asprad_test, delta_test, vpa_test, tmin_test, tmax_test,
      R_s_test
    )
  )

  # radiation_netRadiation
  expect_type(
    (netRadiation_test <- radiation_netRadiation(
      solarConstant_test, latrad_test, elevation_test, slorad_test, asprad_test,
      delta_test, vpa_test, tmin_test, tmax_test, R_s_test, alpha_test
    )),
    "double"
  )
  expect_error(
    radiation_netRadiation(
      "tururu", latrad_test, elevation_test, slorad_test, asprad_test,
      delta_test, vpa_test, tmin_test, tmax_test, R_s_test, alpha_test
    )
  )
  expect_error(
    radiation_netRadiation(
      c(solarConstant_test, solarConstant_test), latrad_test, elevation_test,
      slorad_test, asprad_test, delta_test, vpa_test, tmin_test, tmax_test,
      R_s_test, alpha_test
    )
  )
})

## Utils functions tests ####
test_that("utils c internals work as expected", {

  # data for utils test
  temperature_test <- 25
  Tmin_test <- 5
  Tmax_test <- 26
  RHmin_test <- 25
  RHmax_test <- 99
  elevation_test <- 900
  Patm_test <- 100

  # utils_saturationVP
  expect_type(
    (saturationVP_test <- utils_saturationVP(temperature_test)),
    "double"
  )
  expect_error(
    utils_saturationVP("tururu")
  )
  expect_error(
    utils_saturationVP(c(temperature_test, temperature_test))
  )

  # utils_averageDailyVP
  expect_type(
    (averageDailyVP_test <- utils_averageDailyVP(Tmin_test, Tmax_test, RHmin_test, RHmax_test)),
    "double"
  )
  expect_error(
    utils_averageDailyVP("tururu", Tmax_test, RHmin_test, RHmax_test)
  )
  expect_error(
    utils_averageDailyVP(c(Tmin_test, Tmin_test), Tmax_test, RHmin_test, RHmax_test)
  )

  # utils_atmosphericPressure
  expect_type(
    (atmosphericPressure_test <- utils_atmosphericPressure(elevation_test)),
    "double"
  )
  expect_error(
    utils_atmosphericPressure("tururu")
  )
  expect_error(
    utils_atmosphericPressure(c(elevation_test, elevation_test))
  )

  # utils_airDensity
  expect_type(
    (airDensity_test <- utils_airDensity(temperature_test, Patm_test)),
    "double"
  )
  expect_error(
    utils_airDensity("tururu")
  )
  expect_error(
    utils_airDensity(c(temperature_test, temperature_test), Patm_test)
  )

  # utils_averageDaylightTemperature
  expect_type(
    (averageDaylightTemperature_test <- utils_averageDaylightTemperature(Tmin_test, Tmax_test)),
    "double"
  )
  expect_error(
    utils_averageDaylightTemperature("tururu")
  )
  expect_error(
    utils_averageDaylightTemperature(c(Tmin_test, Tmin_test), Tmax_test)
  )

  # utils_latentHeatVaporisation
  expect_type(
    (latentHeatVaporisation_test <- utils_latentHeatVaporisation(temperature_test)),
    "double"
  )
  expect_error(
    utils_latentHeatVaporisation("tururu")
  )
  expect_error(
    utils_latentHeatVaporisation(c(temperature_test, temperature_test))
  )

  # utils_latentHeatVaporisationMol
  expect_type(
    (latentHeatVaporisationMol_test <- utils_latentHeatVaporisationMol(temperature_test)),
    "double"
  )
  expect_error(
    utils_latentHeatVaporisationMol("tururu")
  )
  expect_error(
    utils_latentHeatVaporisationMol(c(temperature_test, temperature_test))
  )

  # utils_psychrometricConstant
  expect_type(
    (psychrometricConstant_test <- utils_psychrometricConstant(temperature_test, Patm_test)),
    "double"
  )
  expect_error(
    utils_psychrometricConstant("tururu")
  )
  expect_error(
    utils_psychrometricConstant(c(temperature_test, temperature_test), Patm_test)
  )

  # utils_saturationVaporPressureCurveSlope
  expect_type(
    (saturationVaporPressureCurveSlope_test <- utils_saturationVaporPressureCurveSlope(temperature_test)),
    "double"
  )
  expect_error(
    utils_saturationVaporPressureCurveSlope("tururu")
  )
  expect_error(
    utils_saturationVaporPressureCurveSlope(c(temperature_test, temperature_test))
  )
})

## PET functions tests ####
test_that("PET c internals work as expected", {

  # data for PET test
  latrad_test <- 42 * (pi / 180)
  elevation_test <- 900
  slorad_test <- 2 * (pi / 180)
  asprad_test <- 270 * (pi / 180)
  J_test <- 2459695
  tmax_test <- 5 + 16
  tmin_test <- 5
  rhmin_test <- 10
  rhmax_test <- 99
  R_s_test <- 6
  u_test <- 5
  z_test <- 2
  z0_test <- 0.001
  alpha_test <- 0.08
  windfun_test <- "1956"
  rc_test <- 80
  Rn_test <- 25

  # penman
  expect_type(
    (penman_test <- penman(
      latrad_test, elevation_test, slorad_test, asprad_test, J_test, tmin_test,
      tmax_test, rhmin_test, rhmax_test, R_s_test, u_test, z_test, z0_test,
      alpha_test, windfun_test
    )),
    "double"
  )

  expect_error(
    penman(
      "latrad_test", elevation_test, slorad_test, asprad_test, J_test, tmin_test,
      tmax_test, rhmin_test, rhmax_test, R_s_test, u_test, z_test, z0_test,
      alpha_test, windfun_test
    )
  )
  expect_error(
    penman(
      c(latrad_test,latrad_test), elevation_test, slorad_test, asprad_test, J_test, tmin_test,
      tmax_test, rhmin_test, rhmax_test, R_s_test, u_test, z_test, z0_test,
      alpha_test, windfun_test
    )
  )
  # expect_error(
  #   penman(
  #     latrad_test, elevation_test, slorad_test, asprad_test, J_test, tmin_test,
  #     tmax_test, rhmin_test, rhmax_test, R_s_test, u_test, z_test, z0_test,
  #     alpha_test, "2022"
  #   )
  # )
  expect_error(
    penman(
      latrad_test, elevation_test, slorad_test, asprad_test, J_test, tmin_test,
      tmax_test, "rhmin_test", rhmax_test, R_s_test, u_test, z_test, z0_test,
      alpha_test, windfun_test
    )
  )
  expect_error(
    penman(
      latrad_test, elevation_test, slorad_test, asprad_test, J_test, tmin_test,
      tmax_test, c(rhmin_test, rhmin_test), rhmax_test, R_s_test, u_test, z_test, z0_test,
      alpha_test, windfun_test
    )
  )

  # penmanmonteith
  expect_type(
    (penmanmonteith_test <- penmanmonteith(
      rc_test, elevation_test, tmin_test, tmax_test, rhmin_test,
      rhmax_test, Rn_test, u_test
    )),
    "double"
  )

  expect_error(
    penmanmonteith(
      "rc_test", elevation_test, tmin_test, tmax_test, rhmin_test,
      rhmax_test, Rn_test, u_test
    )
  )
  expect_error(
    penmanmonteith(
      c(rc_test, rc_test), elevation_test, tmin_test, tmax_test, rhmin_test,
      rhmax_test, Rn_test, u_test
    )
  )
})

## Interpolation functions tests ####
test_that("interpolation c lowlevel internals work as expected", {

  # data for testing
  reference_data <-
    interpolate_data(
      points_to_interpolate_example, meteoland_interpolator_example,
      dates = as.Date("2022-04-01"), verbose = FALSE
    ) |>
    tidyr::unnest(cols = "interpolated_data")
  Xp <- as.numeric(sf::st_coordinates(points_to_interpolate_example)[,1])
  Yp <- as.numeric(sf::st_coordinates(points_to_interpolate_example)[,2])
  Zp <- points_to_interpolate_example$elevation
  X <- as.numeric(
    sf::st_coordinates(stars::st_get_dimension_values(meteoland_interpolator_example, "station"))[,1]
  )
  Y <- as.numeric(
    sf::st_coordinates(stars::st_get_dimension_values(meteoland_interpolator_example, "station"))[,2]
  )
  Z <- as.numeric(meteoland_interpolator_example[["elevation"]][1,])
  Temp <- as.numeric(meteoland_interpolator_example[["MinTemperature"]][1,])
  P <- as.numeric(meteoland_interpolator_example[["Precipitation"]][1,])
  Psmooth <- as.numeric(meteoland_interpolator_example[["SmoothedPrecipitation"]][1,])
  WS <- as.numeric(meteoland_interpolator_example[["WindSpeed"]][1,])
  WD <- as.numeric(meteoland_interpolator_example[["WindDirection"]][1,])
  iniRp <- get_interpolation_params(meteoland_interpolator_example)$initial_Rp
  alpha <- get_interpolation_params(meteoland_interpolator_example)$alpha_MinTemperature
  N <- get_interpolation_params(meteoland_interpolator_example)$N_MinTemperature
  alpha_event <- get_interpolation_params(meteoland_interpolator_example)$alpha_PrecipitationEvent
  N_event <- get_interpolation_params(meteoland_interpolator_example)$N_PrecipitationEvent
  alpha_amount <- get_interpolation_params(meteoland_interpolator_example)$alpha_PrecipitationAmount
  N_amount <- get_interpolation_params(meteoland_interpolator_example)$N_PrecipitationAmount
  alpha_wind <- get_interpolation_params(meteoland_interpolator_example)$alpha_Wind
  N_wind <- get_interpolation_params(meteoland_interpolator_example)$N_Wind
  iterations <- get_interpolation_params(meteoland_interpolator_example)$iterations
  popcrit <- get_interpolation_params(meteoland_interpolator_example)$pop_crit
  fmax <- get_interpolation_params(meteoland_interpolator_example)$f_max
  debug <- get_interpolation_params(meteoland_interpolator_example)$debug

  expect_type(
    (res_temperature_test <- interpolation_temperature(
      Xp, Yp, Zp,
      X[!is.na(Temp)], Y[!is.na(Temp)], Z[!is.na(Temp)],
      Temp[!is.na(Temp)],
      iniRp, alpha, N, iterations, debug
    )),
    "double"
  )
  expect_identical(
    res_temperature_test,
    reference_data$MinTemperature
  )

  expect_type(
    (res_precipitation_test <- interpolation_precipitation(
      Xp, Yp, Zp,
      X[!is.na(P)], Y[!is.na(P)], Z[!is.na(P)],
      P[!is.na(P)], Psmooth[!is.na(P)],
      iniRp, alpha_event, alpha_amount, N_event, N_amount,
      iterations, popcrit, fmax, debug
    )),
    "double"
  )
  expect_identical(
    res_precipitation_test,
    reference_data$Precipitation
  )

  # Wind test
  # directionsAvailable is FALSE because internally in .interpolateWindStationSeriesPoints, when
  # there is more than one missing WD this argument is set to FALSE (wind.cpp, line 236)
  expect_type(
    (res_wind_test <- interpolation_wind(
      Xp, Yp,
      WS[!is.na(WD)], WD[!is.na(WD)],
      X[!is.na(WD)], Y[!is.na(WD)],
      iniRp, alpha_wind, N_wind, iterations, directionsAvailable = FALSE
    )),
    "double"
  )
  expect_equal(res_wind_test[,1], reference_data$WindSpeed, tolerance = 0.001)
  expect_identical(res_wind_test[,2], reference_data$WindDirection)

  # skip("not implemented yet")
})

# helper to check the month filtering
.check_month_filtering <- function(data, months_to_summary) {

  # data.frame/sf
  if (inherits(data, "data.frame")) {
    if (nrow(data) < 1) {
      cli::cli_warn(
        "Selected months ({.val {months_to_summary}}) are not present in the data"
      )
    }
  }

  if (inherits(data, "stars")) {
    if (length(stars::st_get_dimension_values(data, "date")) < 1) {
      cli::cli_abort(
        "Selected months ({.val {months_to_summary}}) are not present in the data"
      )
    }
  }

  return(data)
}

.summary_by_date_and_var <- function(
  meteo_interpolated,
  fun = "mean",
  frequency = NULL,
  vars_to_summary = c(
    "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
    "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
    "Radiation", "WindSpeed", "WindDirection", "PET"
  ),
  dates_to_summary = NULL,
  months_to_summary = 1:12,
  verbose = getOption("meteoland_verbosity", TRUE),
  ...
) {

  # assertions
  assertthat::assert_that(
    has_meteo_names(meteo_interpolated),
    msg = "interpolated_data provided lacks the usual variable names.
    Are you sure it comes from interpolate_data function?"
  )
  if (!is.null(dates_to_summary)) {
    assertthat::assert_that(
      assertthat::is.date(dates_to_summary),
      msg = "dates_to_summary is not a Date object"
    )

    # if not null, filter by the dates supplied
    .verbosity_control(
      cli::cli_alert_info("Filtering the desired dates"),
      verbose
    )
    meteo_interpolated <- meteo_interpolated |>
      dplyr::filter(as.Date(.data$dates) %in% dates_to_summary)
  }

  .group_by_freq <- function(data, frequency) {
    ## frequency alone, if data spans more than one year can group together data from different years
    ## (in the original code it was a cut(dates, breaks = freq), so we have to imitate it)
    ## Also, the frequency can be NULL, so, we have to take that into account. We create a helper to
    ## do the mutate and group_by steps depending on the freq data

    # if there is no frequency, means no frequency_fun, then return data as is
    if (is.null(frequency)) {
      return(
        data |>
          # and select the vars wanted
          dplyr::select(dplyr::any_of(vars_to_summary))
      )
    }

    # match args
    frequency <- match.arg(frequency, c("month", "week", "quarter", "year"))
    # lubridate frequency function
    frequency_fun <- parse(text = paste0("lubridate::", frequency)) |> eval()

    # weekly is fu**ed. If we use lubridate::week, the week number is not
    # the standard ISO. If we use lubridate::isoweek, the week number is
    # the correct iso one, but in fringe cases (first days of the year) when
    # adding also the year, the week number corresponds to the previous year, not
    # the date one. To see this in action:
    #
    # dates <- seq(as.Date("2021-12-26"), as.Date("2022-01-31"), 1)
    # tururu <- dplyr::tibble(dates = dates)
    # tururu |>
    #   dplyr::mutate(
    #     month = lubridate::month(.data$dates),
    #     week = lubridate::week(.data$dates),
    #     isoweek = lubridate::isoweek(.data$dates),
    #     year_wtf = lubridate::year(.data$dates),
    #     year_ok = dplyr::if_else(.data$week - .data$isoweek < 0, lubridate::year(.data$dates) - 1, lubridate::year(.data$dates))
    #   )
    #
    # But we cannot use the fixing if_else for all frequencies, as the month
    # column indicates. We need to apply it only in week frequency, hence the
    # block for itself
    if (frequency == "week") {
      frequency_fun <- lubridate::isoweek
      weekly_case <- data |>
        # create and group by the frequency desired
        dplyr::mutate(
          {{frequency}} := frequency_fun(.data$dates),
          year = dplyr::if_else(
            lubridate::week(.data$dates) - lubridate::isoweek(.data$dates) < 0,
            lubridate::year(.data$dates) - 1, # negative means isoweek is from last year
            lubridate::year(.data$dates) # current year
          )
        ) |>
        dplyr::group_by(.data[[frequency]], .data$year) |>
        # and select the vars wanted
        dplyr::select(dplyr::any_of(c(frequency, "year", vars_to_summary)))

      return(weekly_case)
    }

    # year is another fringe case, as the year var is already created, not like
    # the other frequencies. So we need to remove the year creation and selection
    if (frequency == "year") {
      yearly_case <- data |>
        # create and group by the frequency desired
        dplyr::mutate(
          {{frequency}} := frequency_fun(.data$dates)
        ) |>
        dplyr::group_by(.data[[frequency]]) |>
        # and select the vars wanted
        dplyr::select(dplyr::any_of(c(frequency, vars_to_summary)))

      return(yearly_case)
    }

    # normal process, create and group by the frequency desired and select the
    # vars wanted
    data |>
      dplyr::mutate(
        {{frequency}} := frequency_fun(.data$dates),
        year = lubridate::year(.data$dates)
      ) |>
      dplyr::group_by(.data[[frequency]], .data$year) |>
      dplyr::select(dplyr::any_of(c(frequency, "year", vars_to_summary)))
  }

  meteo_interpolated |>
    # filtering the months to summary
    dplyr::filter(lubridate::month(.data$dates) %in% months_to_summary) |>
    # check the filtering return rows, if not trigger a warning
    .check_month_filtering(months_to_summary) |>
    # group by desired frequency (if any)
    .group_by_freq(frequency) |>
    # summarise by frequency
    dplyr::summarise(
      dplyr::across(
        .cols = dplyr::everything(),
        .fns = \(column) {
          fun_to_apply <- parse(text = fun) |> eval()
          fun_to_apply(column, ...)
        }
      )
    ) |>
    # ungroup any group left
    dplyr::ungroup()
}

.summary_col_name <- function(frequency, fun) {
  freq_fixed <- paste0(frequency, 'ly')
  if (is.null(frequency)) {
    freq_fixed <- 'all'
  }

  return(paste0(c(freq_fixed, fun), collapse = '_'))
}

.summary_interpolated_sf <- function(
    meteo_interpolated,
    fun = "mean",
    frequency = NULL,
    vars_to_summary = c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    ),
    dates_to_summary = NULL,
    months_to_summary = 1:12,
    verbose = getOption("meteoland_verbosity", TRUE),
    ...
) {

  ## sf nested
  if ("interpolated_data" %in% names(meteo_interpolated) & inherits(meteo_interpolated, "sf")) {

    summary_name <- .summary_col_name(frequency, fun)
    res <- meteo_interpolated |>
      dplyr::mutate(
        !!summary_name := purrr::map(
          .data$interpolated_data, .f = .summary_by_date_and_var,
          fun = fun, frequency = frequency, vars_to_summary = vars_to_summary,
          dates_to_summary = dates_to_summary, months_to_summary = months_to_summary,
          verbose = verbose, ...
        )
      )

    return(res)
  }

  # warning if unnested interpolation is supplied, as all locations will be merged
  # together
  if (
    has_meteo_names(meteo_interpolated) &
    inherits(meteo_interpolated, "sf") &
    !"interpolated_data" %in% names(meteo_interpolated)
  ) {
    cli::cli_warn(
      "meteo_interpolated object seen to have meteo variables and more than one location. All locations will be summarise together"
    )
  }

  # data frame not nested
  meteo_interpolated |>
    .summary_by_date_and_var(
      fun = fun, frequency = frequency, vars_to_summary = vars_to_summary,
      dates_to_summary = dates_to_summary, months_to_summary = months_to_summary,
      verbose = verbose, ...
    )
}

.summary_interpolated_stars <- function(
    meteo_interpolated,
    fun = "mean",
    frequency = NULL,
    vars_to_summary = c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    ),
    dates_to_summary = NULL,
    months_to_summary = 1:12,
    verbose = getOption("meteoland_verbosity", TRUE),
    ...
) {

  # assertions
  assertthat::assert_that(
    has_meteo_names(meteo_interpolated),
    msg = "interpolated_data provided lacks the usual variable names.
    Are you sure it comes from interpolate_data function?"
  )

  # first is to filter dates if dates is supplied
  if (!is.null(dates_to_summary)) {
    assertthat::assert_that(
      assertthat::is.date(dates_to_summary),
      msg = "dates_to_summary is not a Date object"
    )

    # if not null, filter by the dates supplied
    .verbosity_control(
      cli::cli_alert_info("Filtering the desired dates"),
      verbose
    )
    meteo_interpolated <- meteo_interpolated |>
      dplyr::filter(as.Date(date) %in% dates_to_summary)
  }

  # check the NULLness of frequency, if frequency is NULL, the aggregation is
  # done between all dates in date dimension
  if (!is.null(frequency)) {
    # match args if frequency is not null
    frequency <- match.arg(frequency, c("month", "week", "quarter", "year"))
  } else {
    # count the number of days and set it as frequency
    frequency <- paste0(
      length(stars::st_get_dimension_values(meteo_interpolated, "date")),
      " days"
    )
  }

  # stars aggregation by frequency
  meteo_interpolated |>
    # select the vars to summarise
    dplyr::select(dplyr::any_of(vars_to_summary)) |>
    # filter the months to aggregate if supplied, if not all
    dplyr::filter(lubridate::month(date) %in% months_to_summary) |>
    # check the filtering return dates, if not trigger a warning
    .check_month_filtering(months_to_summary) |>
    # perform the aggregation
    aggregate(by = frequency, FUN = parse(text = fun) |> eval())
}

#' Summarise interpolated data by temporal dimension
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Summarises the interpolated meteorology in one or more locations by the desired
#' temporal scale
#'
#' @details
#' If \code{interpolated_data} is a nested interpolated data sf object, as
#' returned by \code{\link{interpolate_data}}, temporal summary is done for each
#' location present in the interpolated data. If \code{interpolated_data} is
#' an unnested interpolated data sf object, temporal summary is done for all
#' locations together. If \code{interpolated_data} is a single location data.frame
#' containing the dates and the interpolated variables, temporal summary is done
#' for that location. If \code{interpolated_data} is a stars object as returned
#' by \code{\link{interpolate_data}}, temporal summary is done for all the raster.
#'
#' @param interpolated_data An interpolated data object as returned by
#'   \code{\link{interpolate_data}}.
#' @param fun The function to use for summarising the data.
#' @param frequency A string indicating the interval specification (allowed ones
#'   are "week", "month", "quarter" and "year"). If NULL (default), aggregation
#'   is done in one interval for all the dates present.
#' @param vars_to_summary A character vector with one or more variable names to
#'   summarise. By default, all interpolated variables are summarised.
#' @param dates_to_summary A Date object to define the dates to be summarised.
#'   If NULL (default), all dates in the interpolated data are processed.
#' @param months_to_summary A numeric vector with the month numbers to subset
#'   the data before summarising. (e.g. \code{c(7,8)} for July and August).
#'   This parameter allows studying particular seasons, when combined with
#'   \code{frequency}. For example \code{frequency = "years"} and
#'   \code{months = 6:8} leads to summarizing summer months of each year.
#' @param verbose Logical indicating if the function must show messages and info.
#'   Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
#'   to TRUE. It can be turned off for the function with FALSE, or session wide with
#'   \code{options(meteoland_verbosity = FALSE)}
#' @param ... Arguments needed for \code{fun}
#'
#' @return For a nested interpolated data, the same sf object with a new column
#' with the temporal summaries. For an unnested interpolated
#' data, a data.frame with the summarised meteo variables. For an interpolated
#' raster (stars object), the same raster with the temporal dimension aggregated
#' as desired.
#'
#' @author \enc{Víctor}{Victor} Granda \enc{García}{Garcia}, CREAF
#' @examples
#' \donttest{
#' # points interpolation aggregation
#' points_to_interpolate_example |>
#'   interpolate_data(meteoland_interpolator_example, verbose = FALSE) |>
#'   summarise_interpolated_data()
#'
#' # raster interpolation aggregation
#' raster_to_interpolate_example |>
#'   interpolate_data(meteoland_interpolator_example, verbose = FALSE) |>
#'   summarise_interpolated_data()
#' }
#'
#' @export
summarise_interpolated_data <- function(
  interpolated_data,
  fun = "mean",
  frequency = NULL,
  vars_to_summary = c(
    "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
    "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
    "Radiation", "WindSpeed", "WindDirection", "PET"
  ),
  dates_to_summary = NULL,
  months_to_summary = 1:12,
  verbose = getOption("meteoland_verbosity", TRUE),
  ...
) {

  # assertions
  assertthat::assert_that(
    is.character(fun), msg = "fun must be a character"
  )
  assertthat::assert_that(
    (is.null(frequency)) || is.character(frequency),
    msg = "frequency must be NULL or a character"
  )
  assertthat::assert_that(
    is.character(vars_to_summary), msg = "vars_to_summary must be a character vector"
  )
  assertthat::assert_that(
    any(vars_to_summary %in% c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    )),
    msg = "vars_to_summary must be one or more of
    'MeanTemperature', 'MinTemperature','MaxTemperature', 'Precipitation',
    'MeanRelativeHumidity', 'MinRelativeHumidity', 'MaxRelativeHumidity',
    'Radiation', 'WindSpeed', 'WindDirection', 'PET'"
  )
  assertthat::assert_that(
    is.numeric(months_to_summary), msg = "months_to_summary must be a numeric vector"
  )

  # interpolated data dispatcher
  summary_fun <- NULL

  if (inherits(interpolated_data, "stars")) {
    summary_fun <- .summary_interpolated_stars
  }

  if (inherits(interpolated_data, "sf") || inherits(interpolated_data, "data.frame")) {
    summary_fun <- .summary_interpolated_sf
  }

  if (is.null(summary_fun)) {
    cli::cli_abort(
      "interpolated_data provided is not either a stars object or a sf/data.frame object. Are you sure it comes from interpolate_data function?"
    )
  }

  res <- summary_fun(
    interpolated_data,
    fun = fun,
    frequency = frequency,
    vars_to_summary = vars_to_summary,
    dates_to_summary = dates_to_summary,
    months_to_summary = months_to_summary,
    verbose = verbose,
    ...
  )

  return(res)
}

#' Summarise interpolator objects by temporal dimension
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Summarises an interpolator object by the desired temporal scale.
#'
#' @inheritParams summarise_interpolated_data
#' @param interpolator An interpolator object as created by \code{\link{create_meteo_interpolator}}.
#'
#' @return
#' \code{summarise_interpolator} function returns the same interpolator object provided with the
#' temporal dimension aggregated to desired frequency.
#'
#' @author \enc{Víctor}{Victor} Granda \enc{García}{Garcia}, CREAF
#' @examples
#' \donttest{
#' # example interpolator
#' meteoland_interpolator_example
#'
#' # aggregate all dates in the interpolator, calculating the maximum values
#' summarise_interpolator(meteoland_interpolator_example, fun = "max")
#'
#' # aggregate weekly, calculating mean values
#' summarise_interpolator(meteoland_interpolator_example, frequency = "week")
#'
#' }
#'
#' @export
summarise_interpolator <- function(
  interpolator,
  fun = "mean",
  frequency = NULL,
  vars_to_summary = c(
    "Temperature", "MinTemperature","MaxTemperature", "Precipitation",
    "RelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
    "Radiation", "WindSpeed", "WindDirection", "PET",
    "SmoothedPrecipitation", "SmoothedTemperatureRange",
    "elevation", "slope", "aspect"
  ),
  dates_to_summary = NULL,
  months_to_summary = 1:12,
  verbose = getOption("meteoland_verbosity", TRUE),
  ...
) {

  # assertions
  assertthat::assert_that(.is_interpolator(interpolator))
  assertthat::assert_that(
    is.character(fun), msg = "fun must be a character"
  )
  assertthat::assert_that(
    (is.null(frequency)) || is.character(frequency),
    msg = "frequency must be NULL or a character"
  )
  assertthat::assert_that(
    is.character(vars_to_summary), msg = "vars_to_summary must be a character vector"
  )
  assertthat::assert_that(
    any(vars_to_summary %in% c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "Temperature",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "RelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET",
      "SmoothedPrecipitation", "SmoothedTemperatureRange",
      "elevation", "slope", "aspect"
    )),
    msg = "vars_to_summary must be one or more of
    'MeanTemperature', 'MinTemperature','MaxTemperature', 'Precipitation',
    'MeanRelativeHumidity', 'MinRelativeHumidity', 'MaxRelativeHumidity',
    'RelativeHumidity',
    'Radiation', 'WindSpeed', 'WindDirection', 'PET',
    'SmoothedPrecipitation', 'SmoothedTemperatureRange',
    'elevation', 'aspect', 'slope'"
  )
  assertthat::assert_that(
    is.numeric(months_to_summary), msg = "months_to_summary must be a numeric vector"
  )

  # helper function to put col and row names again after summarising
  .apply_colrows <- function(x, column_names, row_names) {
    colnames(x) <- column_names
    rownames(x) <- row_names
    return(x)
  }

  # summarising (using the .summary_interpolated_stars, because at the end,
  # the interpolator object is a star object)
  temp_res <- .summary_interpolated_stars(
    interpolator,
    fun = fun,
    frequency = frequency,
    vars_to_summary = vars_to_summary,
    dates_to_summary = dates_to_summary,
    months_to_summary = months_to_summary,
    verbose = verbose,
    ...
  ) |>
    # we are summarising an interpolator, dont forget to add again the parameters
    set_interpolation_params(
      params = get_interpolation_params(interpolator),
      verbose = verbose
    )

  # also, aggregating with aggregate.stars method rename the temporal dimension
  # so we need to ensure to maintain the old ones
  dimnames(temp_res) <- dimnames(interpolator)

  # dont forget also to put again the names of the columns and rows in the
  # interpolator data also
  attributes_with_rowcol_names <- temp_res |>
    purrr::map(
      .f = .apply_colrows,
      column_names = colnames(interpolator[["elevation"]]),
      row_names = as.character(stars::st_get_dimension_values(temp_res, "date"))
    )

  for (attribute in names(attributes_with_rowcol_names)) {
    temp_res[[attribute]] <- attributes_with_rowcol_names[[attribute]]
  }

  return(temp_res)

}

test_that("summarise_interpolated_data for sf works as expected", {
  data_test <- interpolate_data(
    points_to_interpolate_example, meteoland_interpolator_example,
    verbose = FALSE
  )

  # errors
  # summarise_interpolated_data arguments errors
  expect_error(
    summarise_interpolated_data(meteoland_topo_example),
    "interpolated_data"
  )
  expect_error(
    summarise_interpolated_data(25),
    "provided is not"
  )
  expect_error(
    summarise_interpolated_data(data_test, fun = 25),
    "fun"
  )
  expect_error(
    summarise_interpolated_data(data_test, fun = "tururu"),
    "tururu"
  )
  expect_error(
    summarise_interpolated_data(data_test, frequency = 25),
    "frequency"
  )
  expect_error(
    summarise_interpolated_data(data_test, frequency = "tururu"),
    "should be one of"
  )
  expect_error(
    summarise_interpolated_data(data_test, vars_to_summary = 25),
    "vars_to_summary"
  )
  expect_error(
    summarise_interpolated_data(data_test, vars_to_summary = "tururu"),
    "vars_to_summary"
  )
  expect_error(
    summarise_interpolated_data(data_test, dates_to_summary = 25),
    "dates_to_summary"
  )
  expect_error(
    summarise_interpolated_data(data_test, months_to_summary = "tururu"),
    "months_to_summary"
  )

  # nested sf
  # defaults
  expect_s3_class(
    (res_nested_defaults_test <- summarise_interpolated_data(data_test)),
    "sf"
  )
  expect_true("all_mean" %in% names(res_nested_defaults_test))
  expect_s3_class(res_nested_defaults_test$all_mean[[1]], "data.frame")
  expect_named(
    res_nested_defaults_test$all_mean[[10]],
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_true(nrow(res_nested_defaults_test$all_mean[[5]]) == 1)
  # test for data summarised, not NA or NaN
  expect_false(
    is.na(res_nested_defaults_test$all_mean[[6]]$MeanTemperature) ||
      is.nan(res_nested_defaults_test$all_mean[[6]]$MeanTemperature)
  )
  # test that summary dfs are not grouped
  expect_false(inherits(res_nested_defaults_test$all_mean[[8]], "grouped_df"))

  # frequency
  expect_s3_class(
    (res_nested_weekly_test <- summarise_interpolated_data(data_test, frequency = "week")),
    "sf"
  )
  expect_true("weekly_mean" %in% names(res_nested_weekly_test))
  expect_s3_class(res_nested_weekly_test$weekly_mean[[1]], "data.frame")
  expect_named(
    res_nested_weekly_test$weekly_mean[[10]],
    c(
      "week", "year", "MeanTemperature", "MinTemperature","MaxTemperature",
      "Precipitation", "MeanRelativeHumidity", "MinRelativeHumidity",
      "MaxRelativeHumidity", "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_true(nrow(res_nested_weekly_test$weekly_mean[[5]]) == 5)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_nested_weekly_test$weekly_mean[[6]]$MeanTemperature)) ||
      all(is.nan(res_nested_weekly_test$weekly_mean[[6]]$MeanTemperature))
  )
  # test that summary dfs are not grouped
  expect_false(inherits(res_nested_weekly_test$weekly_mean[[8]], "grouped_df"))

  # dates to summary
  dates_to_summary_test <- as.Date(unique(data_test$interpolated_data[[3]]$dates)[1:5])
  expect_message(
    (res_nested_dates_test <-
       summarise_interpolated_data(data_test, dates_to_summary = dates_to_summary_test)),
    "Filtering the desired dates"
  )
  expect_s3_class(res_nested_dates_test, "sf")
  expect_true("all_mean" %in% names(res_nested_dates_test))
  expect_s3_class(res_nested_dates_test$all_mean[[1]], "data.frame")
  expect_named(
    res_nested_dates_test$all_mean[[10]],
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature",
      "Precipitation", "MeanRelativeHumidity", "MinRelativeHumidity",
      "MaxRelativeHumidity", "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_true(nrow(res_nested_dates_test$all_mean[[5]]) == 1)
  # test for data summarised, not NA or NaN
  expect_false(
    is.na(res_nested_dates_test$all_mean[[6]]$MeanTemperature) ||
      is.nan(res_nested_dates_test$all_mean[[6]]$MeanTemperature)
  )
  # test that summary dfs are not grouped
  expect_false(inherits(res_nested_dates_test$all_mean[[8]], "grouped_df"))

  # What happens when the dates supplied are not in the data??
  # TODO

  # months_to_summary
  months_to_summary_test <- 4
  expect_s3_class(
    (res_nested_months_test <- summarise_interpolated_data(data_test, months_to_summary = months_to_summary_test)),
    "sf"
  )
  expect_true("all_mean" %in% names(res_nested_months_test))
  expect_s3_class(res_nested_months_test$all_mean[[1]], "data.frame")
  expect_named(
    res_nested_months_test$all_mean[[10]],
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature",
      "Precipitation", "MeanRelativeHumidity", "MinRelativeHumidity",
      "MaxRelativeHumidity", "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_true(nrow(res_nested_months_test$all_mean[[5]]) == 1)
  # test for data summarised, not NA or NaN
  expect_false(
    is.na(res_nested_months_test$all_mean[[6]]$MeanTemperature) ||
      is.nan(res_nested_months_test$all_mean[[6]]$MeanTemperature)
  )

  # as data is only april, this result should be identical to the base one
  expect_identical(res_nested_months_test, res_nested_defaults_test)

  # What happens when the months supplied are not in the data??
  expect_s3_class(
    suppressWarnings(
      res_nested_months_bad_test <- summarise_interpolated_data(data_test, months_to_summary = 1:2)
    ),
    "sf"
  )
  # expecting NA or NaN
  expect_true(
    is.na(res_nested_months_bad_test$all_mean[[6]]$MeanTemperature) ||
      is.nan(res_nested_months_bad_test$all_mean[[6]]$MeanTemperature)
  )
  # test that summary dfs are not grouped
  expect_false(inherits(res_nested_months_test$all_mean[[8]], "grouped_df"))
  expect_false(inherits(res_nested_months_bad_test$all_mean[[8]], "grouped_df"))

  # vars_to_summary
  vars_to_summary_test <- c("Precipitation", "Radiation")
  expect_s3_class(
    (res_nested_vars_test <- summarise_interpolated_data(data_test, vars_to_summary = vars_to_summary_test)),
    "sf"
  )
  expect_true("all_mean" %in% names(res_nested_vars_test))
  expect_s3_class(res_nested_vars_test$all_mean[[1]], "data.frame")
  expect_named(res_nested_vars_test$all_mean[[10]], c("Precipitation", "Radiation"))
  expect_true(nrow(res_nested_vars_test$all_mean[[5]]) == 1)
  # test for data summarised, not NA or NaN
  expect_false(
    is.na(res_nested_vars_test$all_mean[[6]]$Precipitation) ||
      is.nan(res_nested_vars_test$all_mean[[6]]$Precipitation)
  )
  # test that summary dfs are not grouped
  expect_false(inherits(res_nested_vars_test$all_mean[[8]], "grouped_df"))

  # integration tests
  expect_s3_class(
    (res_nested_integration_test <- summarise_interpolated_data(
      data_test,
      fun = "max",
      frequency = "week",
      vars_to_summary = c("Precipitation", "Radiation"),
      dates_to_summary = as.Date(c("2022-04-01", "2022-04-02")),
      months_to_summary = 4,
      na.rm = TRUE,
      verbose = FALSE
    )),
    "sf"
  )
  expect_true("weekly_max" %in% names(res_nested_integration_test))
  expect_s3_class(res_nested_integration_test$weekly_max[[1]], "data.frame")
  expect_named(
    res_nested_integration_test$weekly_max[[10]],
    c("week", "year", "Precipitation", "Radiation")
  )
  expect_true(nrow(res_nested_integration_test$weekly_max[[5]]) == 1)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_nested_integration_test$weekly_max[[6]]$Radiation)) ||
      all(is.nan(res_nested_integration_test$weekly_max[[6]]$Radiation))
  )
  # test that summary dfs are not grouped
  expect_false(inherits(res_nested_integration_test$weekly_max[[8]], "grouped_df"))

  # TODO
  # tests for unnested
  # test for individual data frames

})

test_that("summarise_interpolated_data for stars works as expected", {
  data_test <- interpolate_data(
    raster_to_interpolate_example, meteoland_interpolator_example,
    verbose = FALSE
  )

  data_test_no_interpolation <- data_test[c("elevation", "slope", "aspect"),,,1]

  # errors
  expect_error(
    summarise_interpolated_data(data_test_no_interpolation), "interpolated_data"
  )

  # defaults
  expect_s3_class(
    (res_defaults_test <- summarise_interpolated_data(data_test)),
    "stars"
  )
  expect_named(
    res_defaults_test,
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_length(stars::st_get_dimension_values(res_defaults_test, "time"), 1)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_defaults_test$MeanTemperature)) ||
      all(is.nan(res_defaults_test$MeanTemperature))
  )

  # frequency
  expect_s3_class(
    (res_frequency_test <- summarise_interpolated_data(data_test, frequency = "week")),
    "stars"
  )
  expect_named(
    res_frequency_test,
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_length(stars::st_get_dimension_values(res_frequency_test, "time"), 5)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_frequency_test$MeanTemperature)) ||
      all(is.nan(res_frequency_test$MeanTemperature))
  )

  # dates_to_summary
  dates_to_summary_test <- as.Date(stars::st_get_dimension_values(data_test, "date")[1:5])
  expect_message(
    (res_dates_test <- summarise_interpolated_data(data_test, dates_to_summary = dates_to_summary_test)),
    "Filtering the desired dates"
  )
  expect_s3_class(res_dates_test, "stars")
  expect_named(
    res_dates_test,
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_length(stars::st_get_dimension_values(res_dates_test, "time"), 1)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_dates_test$MeanTemperature)) ||
      all(is.nan(res_dates_test$MeanTemperature))
  )

  # What happens when the dates supplied are not in the data??
  # TODO

  # months_to_summary
  months_to_summary_test <- 4
  expect_s3_class(
    (res_months_test <- summarise_interpolated_data(data_test, months_to_summary = months_to_summary_test)),
    "stars"
  )
  expect_named(
    res_months_test,
    c(
      "MeanTemperature", "MinTemperature","MaxTemperature", "Precipitation",
      "MeanRelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity",
      "Radiation", "WindSpeed", "WindDirection", "PET"
    )
  )
  expect_length(stars::st_get_dimension_values(res_months_test, "time"), 1)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_months_test$MeanTemperature)) ||
      all(is.nan(res_months_test$MeanTemperature))
  )

  # as data is only april, this result should be identical to the base one
  expect_identical(res_months_test, res_defaults_test)

  # What happens when the months supplied are not in the data??
  expect_error(
    summarise_interpolated_data(data_test, months_to_summary = 1:2),
    "Selected months"
  )

  # vars_to_summary
  vars_to_summary_test <- c("Precipitation", "Radiation")
  expect_s3_class(
    (res_vars_test <- summarise_interpolated_data(data_test, vars_to_summary = vars_to_summary_test)),
    "stars"
  )
  expect_named(res_vars_test, c("Precipitation", "Radiation"))
  expect_length(stars::st_get_dimension_values(res_vars_test, "time"), 1)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_vars_test$Precipitation)) ||
      all(is.nan(res_vars_test$Precipitation))
  )

  # integration tests
  expect_s3_class(
    (res_integration_test <- summarise_interpolated_data(
      data_test,
      fun = "max",
      frequency = "week",
      vars_to_summary = c("Precipitation", "Radiation"),
      dates_to_summary = as.Date(c("2022-04-01", "2022-04-02")),
      months_to_summary = 4,
      na.rm = TRUE,
      verbose = FALSE
    )),
    "stars"
  )
  expect_named(res_integration_test, c("Precipitation", "Radiation"))
  expect_length(stars::st_get_dimension_values(res_integration_test, "time"), 1)
  # test for data summarised, not NA or NaN
  expect_false(
    all(is.na(res_integration_test$Precipitation)) ||
      all(is.nan(res_integration_test$Precipitation))
  )

})


test_that("summarise_interpolator works as expected", {

  data_test <- meteoland_interpolator_example

  # errors
  # summarise_interpolator arguments errors
  expect_error(
    summarise_interpolator(meteoland_topo_example),
    "interpolator"
  )
  expect_error(
    summarise_interpolator(
      interpolate_data(
        raster_to_interpolate_example, meteoland_interpolator_example,
        verbose = FALSE
      )
    ),
    "interpolator"
  )
  expect_error(
    summarise_interpolator(25),
    "interpolator"
  )
  expect_error(
    summarise_interpolator(data_test, fun = 25),
    "fun"
  )
  expect_error(
    summarise_interpolator(data_test, fun = "tururu"),
    "tururu"
  )
  expect_error(
    summarise_interpolator(data_test, frequency = 25),
    "frequency"
  )
  expect_error(
    summarise_interpolator(data_test, frequency = "tururu"),
    "should be one of"
  )
  expect_error(
    summarise_interpolator(data_test, vars_to_summary = 25),
    "vars_to_summary"
  )
  expect_error(
    summarise_interpolator(data_test, vars_to_summary = "tururu"),
    "vars_to_summary"
  )
  expect_error(
    summarise_interpolator(data_test, dates_to_summary = 25),
    "dates_to_summary"
  )
  expect_error(
    summarise_interpolator(data_test, months_to_summary = "tururu"),
    "months_to_summary"
  )

  # default tests
  expect_s3_class(
    (res_interp_defaults <- summarise_interpolator(data_test)),
    "stars"
  )
  # result must be an interpolator
  expect_true(.is_interpolator(res_interp_defaults))
  # result must have the same names
  expect_named(res_interp_defaults, names(data_test), ignore.order = TRUE)
  # result has to have a frequency dependent number of dates
  expect_length(stars::st_get_dimension_values(res_interp_defaults, "date"), 1)
  # result has to have the same stations
  expect_identical(ncol(res_interp_defaults), ncol(data_test))
  # result has to have values for the variables that had values in the interpolator
  expect_false(all(is.na(res_interp_defaults[["MinTemperature"]])))
  # result has not gaps in the topo info
  expect_false(any(is.na(res_interp_defaults[["elevation"]])))
  # result shouldn't have change the values for topo, also, colrownames must be the same
  expect_identical(res_interp_defaults[["elevation"]][1,], data_test[["elevation"]][1,])

  # frequency
  expect_s3_class(
    (res_interp_week <- summarise_interpolator(data_test, frequency = "week")),
    'stars'
  )
  # result must be an interpolator
  expect_true(.is_interpolator(res_interp_week))
  # result must have the same names
  expect_named(res_interp_week, names(data_test), ignore.order = TRUE)
  # result has to have a frequency dependent number of dates
  expect_length(stars::st_get_dimension_values(res_interp_week, "date"), 5)
  # result has to have the same stations
  expect_identical(ncol(res_interp_week), ncol(data_test))
  # result has to have values for the variables that had values in the interpolator
  expect_false(all(is.na(res_interp_week[["MinTemperature"]])))
  # result has not gaps in the topo info
  expect_false(any(is.na(res_interp_week[["elevation"]])))
  # result shouldn't have change the values for topo, also, colrownames must be the same
  expect_identical(res_interp_week[["elevation"]][1,], data_test[["elevation"]][1,])

  # dates_to_summary
  dates_to_summary_test <-
    as.Date(stars::st_get_dimension_values(data_test, "date")[1:5])
  expect_s3_class(
    (res_interp_dates <-
       summarise_interpolator(data_test, dates_to_summary = dates_to_summary_test)),
    'stars'
  )
  # result must be an interpolator
  expect_true(.is_interpolator(res_interp_dates))
  # result must have the same names
  expect_named(res_interp_dates, names(data_test), ignore.order = TRUE)
  # result has to have a frequency dependent number of dates
  expect_length(stars::st_get_dimension_values(res_interp_dates, "date"), 1)
  # result has to have the same stations
  expect_identical(ncol(res_interp_dates), ncol(data_test))
  # result has to have values for the variables that had values in the interpolator
  expect_false(all(is.na(res_interp_dates[["MinTemperature"]])))
  # result has not gaps in the topo info
  expect_false(any(is.na(res_interp_dates[["elevation"]])))
  # result shouldn't have change the values for topo, also, colrownames must be the same
  expect_identical(res_interp_dates[["elevation"]][1,], data_test[["elevation"]][1,])
  # result must be different from the default result, as we are only summarising some dates
  expect_false(
    identical(res_interp_dates[["MinTemperature"]], res_interp_defaults[["MinTemperature"]])
  )
  # What happens when the dates supplied are not in the data??
  # TODO

  # months_to_summary
  months_to_summary_test <- 4
  expect_s3_class(
    (res_interp_months <-
       summarise_interpolator(data_test, months_to_summary = months_to_summary_test)),
    'stars'
  )
  # result must be an interpolator
  expect_true(.is_interpolator(res_interp_months))
  # result must have the same names
  expect_named(res_interp_months, names(data_test), ignore.order = TRUE)
  # result has to have a frequency dependent number of dates
  expect_length(stars::st_get_dimension_values(res_interp_months, "date"), 1)
  # result has to have the same stations
  expect_identical(ncol(res_interp_months), ncol(data_test))
  # result has to have values for the variables that had values in the interpolator
  expect_false(all(is.na(res_interp_months[["MinTemperature"]])))
  # result has not gaps in the topo info
  expect_false(any(is.na(res_interp_months[["elevation"]])))
  # result shouldn't have change the values for topo, also, colrownames must be the same
  expect_identical(res_interp_months[["elevation"]][1,], data_test[["elevation"]][1,])
  # result must be equal from the default result, as we are summarising the only
  # present month
  expect_identical(
    res_interp_months[["MinTemperature"]], res_interp_defaults[["MinTemperature"]]
  )
  # What happens when the months supplied are not in the data??
  expect_error(
    summarise_interpolator(data_test, months_to_summary = 1:2),
    "Selected months"
  )

  # vars_to_summary
  vars_to_summary_test <- c("Precipitation", "Radiation")
  expect_s3_class(
    (res_interp_vars <-
       summarise_interpolator(data_test, vars_to_summary = vars_to_summary_test)),
    'stars'
  )
  # result must be an interpolator, but as we remove the mandatory variables, then
  # is not
  expect_error(.is_interpolator(res_interp_vars), "Names found")
  # result must have the same names
  expect_named(res_interp_vars, c("Precipitation", "Radiation"), ignore.order = TRUE)
  # result has to have a frequency dependent number of dates
  expect_length(stars::st_get_dimension_values(res_interp_vars, "date"), 1)
  # result has to have the same stations
  expect_identical(ncol(res_interp_vars), ncol(data_test))
  # result has to have values for the variables that had values in the interpolator
  expect_false(all(is.na(res_interp_vars[["Precipitation"]])))
  # result must be equal from the default result, as we are summarising the only
  # present month
  expect_identical(
    res_interp_vars[["Precipitation"]], res_interp_defaults[["Precipitation"]]
  )

  # integration tests
  expect_s3_class(
    (res_interp_integration <- summarise_interpolator(
      data_test,
      fun = "max",
      frequency = "week",
      vars_to_summary = c("Precipitation", "Radiation"),
      dates_to_summary = as.Date(c("2022-04-01", "2022-04-02")),
      months_to_summary = 4,
      na.rm = TRUE,
      verbose = FALSE
    )),
    "stars"
  )
  # result must be an interpolator, but as we remove the mandatory variables, then
  # is not
  expect_error(.is_interpolator(res_interp_integration), "Names found")
  # result must have the same names
  expect_named(res_interp_integration, c("Precipitation", "Radiation"), ignore.order = TRUE)
  # result has to have a frequency dependent number of dates
  expect_length(stars::st_get_dimension_values(res_interp_integration, "date"), 1)
  # result has to have the same stations
  expect_identical(ncol(res_interp_integration), ncol(data_test))
  # result has to have values for the variables that had values in the interpolator
  expect_false(all(is.na(res_interp_integration[["Precipitation"]])))
  # result must be different from the week result, as we are using only two dates
  expect_false(
    identical(res_interp_integration[["Precipitation"]], res_interp_week[["Precipitation"]])
  )

})

#' Safely removing a station from the interpolator object
#'
#' Safely remove a station from the interpolator object for cross validation
#' processes
#'
#' This function removes the desired stations from the interpolator while
#' maintaining interpolation parameters attribute
#'
#' @param interpolator meteoland interpolator object
#' @param station_index numeric index for the station to remove
#'
#' @return An interpolator object without the station provided
#'
#' @noRd
.remove_station_from_interpolator <- function(interpolator, station_index) {
  res <- interpolator[, , -station_index]
  attr(res, "params") <- attr(interpolator, "params")
  return(res)
}

#' Cross validation process for a single station
#'
#' Cross validation process for a single station
#'
#' This function performs the cross validation process for a single stations,
#' removing the station from the interpolator and using the spatial station
#' info to simulate a spatial data object
#'
#' @param station_index Numeric value with the station index
#' @param interpolator meteoland interpolator object
#'
#' @return A list with the interpolation results for the desired station
#'
#' @noRd
.station_cross_validation <- function(station_index, interpolator) {

  # get the point sf with topography info
  station_sf <- dplyr::tibble(
    elevation = interpolator[["elevation"]][1, station_index],
    aspect = interpolator[["aspect"]][1, station_index],
    slope = interpolator[["slope"]][1, station_index],
    geometry = stars::st_get_dimension_values(interpolator, 'station')[station_index]
  ) |>
    sf::st_as_sf()

  # remove the station from the interpolator
  interpolator_station_removed <-
    .remove_station_from_interpolator(interpolator, station_index)

  # return the interpolation
  suppressMessages(suppressWarnings(
    .interpolation_point(
      station_sf, interpolator_station_removed, verbose = FALSE,
      ignore_convex_hull_check = TRUE
    )
  ))[[1]] |>
    dplyr::mutate(
      RangeTemperature = .data$MaxTemperature - .data$MinTemperature,
      station = station_index
    ) |>
    dplyr::rename(RelativeHumidity = "MeanRelativeHumidity")
}

#' Transform interpolator values to df
#'
#' Transform interpolator values to a data frame
#'
#' This function transform the observed meteorological values in the interpolator
#' object to a data frame for the cross validation process
#'
#' @param interpolator meteoland interpolator object
#'
#' @return A data frame with the station index, dates and meteorological observed
#'   values
#'
#' @noRd
.interpolator2tibble <- function(interpolator) {
  purrr::map(
    1:length(stars::st_get_dimension_values(interpolator, 'station')),
    function(station_index) {
      res <-
        dplyr::tibble(dates = stars::st_get_dimension_values(interpolator, 'date'))
      for (variable in names(interpolator)) {
        res[[variable]] <- interpolator[[variable]][,station_index]
      }
      res |>
        dplyr::mutate(
          RangeTemperature = .data$MaxTemperature - .data$MinTemperature,
          station = station_index,
          stationID = colnames(interpolator[[1]])[station_index]
        )
    }
  ) |>
    purrr::list_rbind()
}

#' Helper to convert predicted values to NA
#'
#' Helper to convert predicted values to NA
#'
#' This function convert predicted values to NA where no observed values are
#' recorded, to allow for correct cross validation calculations
#'
#' @param predicted_df data frame with the predicted (interpolated) values
#' @param observed_df data frame with the observed values
#'
#' @return \code{predicted_df} data frame with NAs where the \code{observed_df}
#'   has NAs
#'
#' @noRd
.set_predicted_nas <- function(predicted_df, observed_df) {

  predicted_df$MinTemperature[is.na(observed_df$MinTemperature)] <- NA
  predicted_df$MaxTemperature[is.na(observed_df$MaxTemperature)] <- NA
  predicted_df$RangeTemperature[is.na(observed_df$RangeTemperature)] <- NA
  predicted_df$Precipitation[is.na(observed_df$Precipitation)] <- NA
  predicted_df$Radiation[is.na(observed_df$Radiation)] <- NA
  predicted_df$RelativeHumidity[is.na(observed_df$RelativeHumidity)] <- NA

  return(predicted_df)

}

#' Correlation between observed and predicted values
#'
#' Correlation between observed and predicted values
#'
#' This function return the correlation values for the provided meteorolgical
#' variable
#'
#' @param variable character with the variable name
#' @param predicted_df data frame with predicted values
#' @param observed_df data frame with observed values
#'
#' @return correlation value (numeric)
#'
#' @noRd
.cv_correlation <- function(variable, predicted_df, observed_df) {

  res <- NA

  if (sum(
    complete.cases(cbind(predicted_df[[variable]], observed_df[[variable]]))
  ) > 0) {
    res <-
      cor(predicted_df[[variable]], observed_df[[variable]], use = "complete.obs")
  }

  return(res)
}

#' Cross validation results processing
#'
#' Cross validation results processing
#'
#' This function creates the results elements for the cross validation returned
#' list
#'
#' @param predicted_df data frame with predicted values
#' @param observed_df data frame with observed values
#'
#' @return A list with the cross validation results (total errors, stations
#' stats, dates stats)
#'
#' @noRd
.cv_processing <- function(predicted_df, observed_df) {

  # calculate errors
  total_errors <- dplyr::tibble(
    dates = predicted_df[["dates"]],
    station = predicted_df[["station"]],
    stationID = observed_df[["stationID"]],
    MinTemperature_error =
      predicted_df[["MinTemperature"]] - observed_df[["MinTemperature"]],
    MaxTemperature_error =
      predicted_df[["MaxTemperature"]] - observed_df[["MaxTemperature"]],
    RangeTemperature_error =
      predicted_df[["RangeTemperature"]] - observed_df[["RangeTemperature"]],
    RelativeHumidity_error =
      predicted_df[["RelativeHumidity"]] - observed_df[["RelativeHumidity"]],
    Radiation_error =
      predicted_df[["Radiation"]] - observed_df[["Radiation"]],
    Precipitation_error =
      predicted_df[["Precipitation"]] - observed_df[["Precipitation"]]
  ) |>
    dplyr::mutate(
      MinTemperature_predicted = predicted_df[["MinTemperature"]],
      MaxTemperature_predicted = predicted_df[["MaxTemperature"]],
      RangeTemperature_predicted = predicted_df[["RangeTemperature"]],
      RelativeHumidity_predicted = predicted_df[["RelativeHumidity"]],
      Radiation_predicted = predicted_df[["Radiation"]],
      Precipitation_predicted = predicted_df[["Precipitation"]],
      MinTemperature_observed = observed_df[["MinTemperature"]],
      MaxTemperature_observed = observed_df[["MaxTemperature"]],
      RangeTemperature_observed = observed_df[["RangeTemperature"]],
      RelativeHumidity_observed = observed_df[["RelativeHumidity"]],
      Radiation_observed = observed_df[["Radiation"]],
      Precipitation_observed = observed_df[["Precipitation"]]
    )

  # bias and mae for stations
  station_stats <- total_errors |>
    dplyr::group_by(.data$station) |>
    dplyr::summarise(
      stationID = dplyr::first(.data$stationID),
      MinTemperature_station_bias = mean(.data$MinTemperature_error, na.rm = TRUE),
      MaxTemperature_station_bias = mean(.data$MaxTemperature_error, na.rm = TRUE),
      RangeTemperature_station_bias = mean(.data$RangeTemperature_error, na.rm = TRUE),
      RelativeHumidity_station_bias = mean(.data$RelativeHumidity_error, na.rm = TRUE),
      Radiation_station_bias = mean(.data$Radiation_error, na.rm = TRUE),
      MinTemperature_station_mae = mean(abs(.data$MinTemperature_error), na.rm = TRUE),
      MaxTemperature_station_mae = mean(abs(.data$MaxTemperature_error), na.rm = TRUE),
      RangeTemperature_station_mae = mean(abs(.data$RangeTemperature_error), na.rm = TRUE),
      RelativeHumidity_station_mae = mean(abs(.data$RelativeHumidity_error), na.rm = TRUE),
      Radiation_station_mae = mean(abs(.data$Radiation_error), na.rm = TRUE),
      TotalPrecipitation_station_observed = sum(.data$Precipitation_observed, na.rm = TRUE),
      TotalPrecipitation_station_predicted = sum(.data$Precipitation_predicted, na.rm = TRUE),
      TotalPrecipitation_station_bias =
        sum(.data$Precipitation_predicted, na.rm = TRUE) -
        sum(.data$Precipitation_observed, na.rm = TRUE),
      TotalPrecipitation_station_relative_bias = 100*(
        sum(.data$Precipitation_predicted, na.rm = TRUE) -
          sum(.data$Precipitation_observed, na.rm = TRUE)
      ) / sum(.data$Precipitation_observed, na.rm = TRUE),
      DaysPrecipitation_station_bias =
        sum(.data$Precipitation_predicted > 0, na.rm = TRUE) -
        sum(.data$Precipitation_observed > 0, na.rm = TRUE),
      DaysPrecipitation_station_relative_bias = 100*(
        sum(.data$Precipitation_predicted > 0, na.rm = TRUE) -
          sum(.data$Precipitation_observed > 0, na.rm = TRUE)
      ) / sum(.data$Precipitation_observed > 0, na.rm = TRUE),
      FreqPrecipitation_station_observed =
        mean(.data$Precipitation_observed > 0, na.rm = TRUE),
      FreqPrecipitation_station_predicted =
        mean(.data$Precipitation_predicted > 0, na.rm = TRUE)
    ) |>
    dplyr::mutate(
      TotalPrecipitation_station_observed = dplyr::if_else(
        .data$TotalPrecipitation_station_observed == 0, NA_real_, .data$TotalPrecipitation_station_observed
      ),
      TotalPrecipitation_station_predicted = dplyr::if_else(
        .data$TotalPrecipitation_station_predicted == 0, NA_real_, .data$TotalPrecipitation_station_predicted
      ),
      TotalPrecipitation_station_bias = dplyr::if_else(
        .data$TotalPrecipitation_station_bias == 0, NA_real_, .data$TotalPrecipitation_station_bias
      ),
      TotalPrecipitation_station_relative_bias = dplyr::if_else(
        .data$TotalPrecipitation_station_relative_bias == 0, NA_real_, .data$TotalPrecipitation_station_relative_bias
      ),
      TotalPrecipitation_station_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_station_predicted), NA_real_, .data$TotalPrecipitation_station_bias
      ),
      TotalPrecipitation_station_relative_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_station_predicted), NA_real_, .data$TotalPrecipitation_station_relative_bias
      ),
      DaysPrecipitation_station_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_station_predicted), NA_integer_, .data$DaysPrecipitation_station_bias
      ),
      DaysPrecipitation_station_relative_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_station_predicted), NA_real_, .data$DaysPrecipitation_station_relative_bias
      ),
      FreqPrecipitation_station_observed = dplyr::if_else(
        is.na(.data$TotalPrecipitation_station_observed), NA_real_, .data$FreqPrecipitation_station_observed
      ),
      FreqPrecipitation_station_predicted = dplyr::if_else(
        is.na(.data$TotalPrecipitation_station_predicted), NA_real_, .data$FreqPrecipitation_station_predicted
      )
    )

  dates_stats <- total_errors |>
    dplyr::group_by(.data$dates) |>
    dplyr::summarise(
      MinTemperature_date_bias = mean(.data$MinTemperature_error, na.rm = TRUE),
      MaxTemperature_date_bias = mean(.data$MaxTemperature_error, na.rm = TRUE),
      RangeTemperature_date_bias = mean(.data$RangeTemperature_error, na.rm = TRUE),
      RelativeHumidity_date_bias = mean(.data$RelativeHumidity_error, na.rm = TRUE),
      Radiation_date_bias = mean(.data$Radiation_error, na.rm = TRUE),
      MinTemperature_date_mae = mean(abs(.data$MinTemperature_error), na.rm = TRUE),
      MaxTemperature_date_mae = mean(abs(.data$MaxTemperature_error), na.rm = TRUE),
      RangeTemperature_date_mae = mean(abs(.data$RangeTemperature_error), na.rm = TRUE),
      RelativeHumidity_date_mae = mean(abs(.data$RelativeHumidity_error), na.rm = TRUE),
      Radiation_date_mae = mean(abs(.data$Radiation_error), na.rm = TRUE),
      TotalPrecipitation_date_observed = sum(.data$Precipitation_observed, na.rm = TRUE),
      TotalPrecipitation_date_predicted = sum(.data$Precipitation_predicted, na.rm = TRUE),
      TotalPrecipitation_date_bias =
        sum(.data$Precipitation_predicted, na.rm = TRUE) -
        sum(.data$Precipitation_observed, na.rm = TRUE),
      TotalPrecipitation_date_relative_bias = 100*(
        sum(.data$Precipitation_predicted, na.rm = TRUE) -
          sum(.data$Precipitation_observed, na.rm = TRUE)
      ) / sum(.data$Precipitation_observed, na.rm = TRUE),
      StationsPrecipitation_date_bias =
        sum(.data$Precipitation_predicted > 0, na.rm = TRUE) -
        sum(.data$Precipitation_observed > 0, na.rm = TRUE),
      StationsPrecipitation_date_relative_bias = 100*(
        sum(.data$Precipitation_predicted > 0, na.rm = TRUE) -
          sum(.data$Precipitation_observed > 0, na.rm = TRUE)
      ) / sum(.data$Precipitation_observed > 0, na.rm = TRUE),
      FreqPrecipitation_date_observed =
        mean(.data$Precipitation_observed > 0, na.rm = TRUE),
      FreqPrecipitation_date_predicted =
        mean(.data$Precipitation_predicted > 0, na.rm = TRUE)
    ) |>
    dplyr::mutate(
      TotalPrecipitation_date_observed = dplyr::if_else(
        .data$TotalPrecipitation_date_observed == 0, NA_real_, .data$TotalPrecipitation_date_observed
      ),
      TotalPrecipitation_date_predicted = dplyr::if_else(
        .data$TotalPrecipitation_date_predicted == 0, NA_real_, .data$TotalPrecipitation_date_predicted
      ),
      TotalPrecipitation_date_bias = dplyr::if_else(
        .data$TotalPrecipitation_date_bias == 0, NA_real_, .data$TotalPrecipitation_date_bias
      ),
      TotalPrecipitation_date_relative_bias = dplyr::if_else(
        .data$TotalPrecipitation_date_relative_bias == 0, NA_real_, .data$TotalPrecipitation_date_relative_bias
      ),
      TotalPrecipitation_date_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_date_predicted), NA_real_, .data$TotalPrecipitation_date_bias
      ),
      TotalPrecipitation_date_relative_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_date_predicted), NA_real_, .data$TotalPrecipitation_date_relative_bias
      ),
      StationsPrecipitation_date_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_date_predicted), NA_integer_, .data$StationsPrecipitation_date_bias
      ),
      StationsPrecipitation_date_relative_bias = dplyr::if_else(
        is.na(.data$TotalPrecipitation_date_predicted), NA_real_, .data$StationsPrecipitation_date_relative_bias
      ),
      FreqPrecipitation_date_observed = dplyr::if_else(
        is.na(.data$TotalPrecipitation_date_observed), NA_real_, .data$FreqPrecipitation_date_observed
      ),
      FreqPrecipitation_date_predicted = dplyr::if_else(
        is.na(.data$TotalPrecipitation_date_predicted), NA_real_, .data$FreqPrecipitation_date_predicted
      )
    )

  return(list(
    errors = total_errors,
    station_stats = station_stats,
    dates_stats = dates_stats
  ))
}

#' @rdname interpolator_calibration
#'
#' @return \code{interpolation_cross_validation} returns a list with the
#' following items
#' \itemize{
#'   \item{errors: Data frame with each combination of station and date with
#'   observed variables, predicated variables and the total error
#'   (predicted - observed) calculated for each variable}
#'   \item{station_stats: Data frame with error and bias statistics aggregated by
#'   station}
#'   \item{dates_stats: Data frame with error and bias statistics aggregated by
#'   date}
#'   \item{r2: correlation indexes between observed and predicted values for each
#'   meteorological variable}
#' }
#'
#' @examples
#'
#' \donttest{
#' # example interpolator
#' data("meteoland_interpolator_example")
#'
#' # As the cross validation for all stations can be time consuming, we are
#' # gonna use only for the first 5 stations of the 198
#' cv <- interpolation_cross_validation(meteoland_interpolator_example, stations = 1:5)
#'
#' # Inspect the results
#' cv$errors
#' cv$station_stats
#' cv$dates_stats
#' cv$r2
#' }
#'
#' @export
interpolation_cross_validation <- function(interpolator, stations = NULL, verbose = getOption("meteoland_verbosity", TRUE)) {

  # debug
  # browser()

  ### assertions
  # interpolator
  assertthat::assert_that(.is_interpolator(interpolator))
  # stations
  assertthat::assert_that(
    is.null(stations) || is.numeric(stations) || is.character(stations),
    msg = "stations must be NULL or a numeric vector with the stations indexes"
  )

  stations <- .station_indexes_converter(stations, interpolator)

  .verbosity_control(
    cli::cli_alert_info("Starting Cross Validation process..."),
    verbose
  )

  observed_values <- .interpolator2tibble(interpolator) |>
    dplyr::filter(.data$station %in% stations)

  .verbosity_control(
    cli::cli_ul("Interpolating stations..."),
    verbose
  )
  predicted_values <-
    purrr::map(stations, .station_cross_validation, interpolator = interpolator, .progress = verbose) |>
    purrr::list_rbind() |>
    # set predicted to NA when observed is NA
    .set_predicted_nas(observed_values)

  ### Processing results
  .verbosity_control(
    cli::cli_ul("Calculating R squared..."),
    verbose
  )

  r2_list <- c(
    MinTemperature = "MinTemperature", MaxTemperature = "MaxTemperature",
    RangeTemperature = "RangeTemperature", RelativeHumidity = "RelativeHumidity",
    Radiation = "Radiation"
  ) |>
    purrr::map(
      .cv_correlation,
      predicted_df = predicted_values, observed_df = observed_values,
      .progress = verbose
    )

  .verbosity_control(
    cli::cli_ul("calculating errors, MAE and bias for interpolated variables..."),
    verbose
  )

  res <- .cv_processing(predicted_values, observed_values)

  res$r2 <- r2_list

  .verbosity_control(
    cli::cli_alert_success("Cross validation done."),
    verbose
  )

  return(res)

}

.station_indexes_converter <- function(stations, interpolator) {

  res <- stations

  if (is.null(stations)) {
    res <- 1:length(stars::st_get_dimension_values(interpolator, "station"))
  }

  if (is.character(stations)) {
    res <- which(colnames(interpolator[[1]]) %in% stations)
    if (length(res) < 1) {
      cli::cli_abort("Station names not found in interpolator")
    }
  }

  return(res)
}

#' Precipitation rainfall erosivity
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' Function \code{precipitation_rainfall_erosivity()} calculates a multi-year
#' average of monthly rainfall erosivity using the MedREM model proposed by
#' Diodato and Bellochi (2010) for the Mediterranean area (see also Guerra et
#' al. 2016).
#'
#' @details
#' MedREM model is: Rm = b0·P·sqrt(d)·(alpha + b1*longitude), where P is
#' accumulated precipitation and d is maximum daily precipitation. Parameters
#' used for the MedREM model are b0 = 0.117, b1 = -0.015, alpha = 2. Note that
#' there is a mistake in Guerra et al. (2016) regarding parameters b1 and a.
#'
#' @param meteo_data A meteo tibble as with the dates and meteorological variables
#'   as returned by \code{\link{interpolate_data}} in the "interpolated_data"
#'   column.
#' @param longitude Longitude in degrees.
#' @param scale Character, either 'month' or 'year'. Default to 'month'
#' @param average Boolean flag to calculate multi-year averages before applying
#'   MedREM's formula.
#'
#' @return A vector of values for each month (in MJ·mm·ha-1·h-1·month-1) or each
#'   year (in MJ·mm·ha-1·h-1·yr-1), depending on the scale
#'
#' @author
#'   Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF.
#'
#'   \enc{Víctor}{Victor} Granda \enc{García}{Garcia}, CREAF.
#'
#' @references
#' Diodato, N., Bellocchi, G., 2010. MedREM, a rainfall erosivity
#' model for the Mediterranean region. J. Hydrol. 387, 119–127,
#' doi:10.1016/j.jhydrol.2010.04.003.
#'
#' Guerra CA, Maes J, Geijzendorffer I, Metzger MJ (2016) An assessment of soil
#' erosion prevention by vegetation in Mediterranean Europe: Current trends of
#' ecosystem service provision. Ecol Indic 60:213–222. doi:
#' 10.1016/j.ecolind.2015.06.043.
#'
#' @examples
#' \donttest{
#' interpolated_example <-
#'   interpolate_data(points_to_interpolate_example, meteoland_interpolator_example)
#'
#' precipitation_rainfall_erosivity(
#'   meteo_data = interpolated_example$interpolated_data[[1]],
#'   longitude = 2.32,
#'   scale = "month",
#'   average = TRUE
#' )
#' }
#'
#' @export
precipitation_rainfall_erosivity <- function(
  meteo_data,
  longitude,
  scale = c("month", "year"),
  average = TRUE
) {

  # recursive call to be able to call precipitation_rainfall_erosivity in mutates or
  # with lists of meteo_data and longitudes
  if ((!is.data.frame(meteo_data) & is.list(meteo_data)) && length(meteo_data) == length(longitude)) {
    res <- purrr::map2(
      .x = meteo_data, .y = longitude,
      .f = precipitation_rainfall_erosivity,
      scale = scale, average = average
    )

    return(res)
  }

  # assertions
  assertthat::assert_that(
    assertthat::is.number(longitude), msg = "longitude must be numeric"
  )
  # assertthat::assert_that(
  #   assertthat::is.string(scale), length(scale) == 1,
  #   msg = "scale must be a string of length 1"
  # )
  assertthat::assert_that(
    assertthat::is.flag(average), msg = "average must be a logical value"
  )

  # match.arg
  scale <- match.arg(scale)

  # constants
  b0 <- 0.117
  b1 <- -0.015
  a <- 2.000

  precipitation_sum <- summarise_interpolated_data(
    meteo_data, fun = 'sum', frequency = scale,
    vars_to_summary = "Precipitation", na.rm = TRUE
  )

  precipitation_max <- summarise_interpolated_data(
    meteo_data, fun = 'max', frequency = scale,
    vars_to_summary = "Precipitation", na.rm = TRUE
  )

  if (isTRUE(average)) {
    ## TODO there is a problem in multiyear datasets and yearly scale, because in theory
    ## in those situations all years must be summarised together, but that's not
    ## happening with the actual code

    # average by scale.
    # This depends on the scale and is done for compatibility with older versions of meteoland
    # If monthly, group by month and average.
    if (scale == 'month') {
      precipitation_sum <- dplyr::group_by(precipitation_sum, .data[[scale]]) |>
        dplyr::summarise(p_s = mean(.data$Precipitation, na.rm = TRUE))
      precipitation_max <- dplyr::group_by(precipitation_max, .data[[scale]]) |>
        dplyr::summarise(d_s = mean(.data$Precipitation, na.rm = TRUE))
    } else {
      # If yearly, average without grouping
      precipitation_sum <- precipitation_sum |>
        dplyr::summarise(p_s = mean(.data$Precipitation, na.rm = TRUE))
      precipitation_max <- precipitation_max |>
        dplyr::summarise(d_s = mean(.data$Precipitation, na.rm = TRUE))
    }

  } else {
    # no average, just rename the Precipitation var for the next steps
    precipitation_sum <- dplyr::rename(precipitation_sum, p_s = "Precipitation")
    precipitation_max <- dplyr::rename(precipitation_max, d_s = "Precipitation")
  }

  p_s <- precipitation_sum |>
    dplyr::pull(.data$p_s) |>
    purrr::set_names(unique(precipitation_sum[[scale]]))

  d_s <- precipitation_max |>
    dplyr::pull(.data$d_s) |>
    purrr::set_names(unique(precipitation_max[[scale]]))

  # build the result with the formula and return it
  res <- b0 * p_s * sqrt(d_s) * (a + b1 * longitude)
  return(res)
}

#' Helper to check for nullness or specific character vector
#' @noRd
is_null_or_variable <- function(object, variable_names) {
  is.null(object) || any(variable_names %in% object)
}

#' Helper to check for nullness and return a numeric NA if it is
#' @noRd
null2na <- function(object) {
  if (is.null(object)) {
    return(NA_real_)
  }
  return(object)
}

#' Verbosity control
#'
#' Verbose check for meteoland messages
#'
#' Function print to console the usethis expression if the verbose control is TRUE. Always returns
#' \code{invisible()}
#'
#' @param expr usethis expression to print if verbose is TRUE
#' @param verbose verbosity user control
#'
#' @return invisible()
#' @noRd
.verbosity_control <- function(expr, verbose) {
  if (verbose) {
    expr
  }
  return(invisible())
}

.get_geometry_name <- function(sf_object) {
  return(attr(sf_object, "sf_column"))
}

---
title: "Tidy meteoland"
author: "Victor Granda"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Tidy meteoland}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(meteoland)
library(stars)
library(dplyr)
```

## A new way of working with `meteoland`

With the [retirement](https://r-spatial.org/r/2022/04/12/evolution.html)
of `rgdal`, `rgeos` and `maptools` R packages, a complete update of `meteoland` was necessary 
to remove the hard dependency `meteoland` has with `sp` and `raster` R packages. Starting with
version 2.0.0 of `meteoland`, any hard dependency on retired packages as well as `sp` and `raster`
has been removed, and now `sf` and `stars` packages are internally used for working with simple
features and raster data.

By June 2023, `sp` and `raster` packages will be completely removed from the dependency list.

As fundamental changes were required (`meteoland` meteorology classes were based on `sp` ones), a
decision to improve and make meteorological interpolation process simpler was taken. In this
vignette, we provide insights on the new ways of working with `meteoland`.

If you are interested in the equivalences between older and newer functions of `meteoland`,
please see the [appendix](#appendix) at the end of the vignette.

## Example datasets

`meteoland` now ships with new example objects, based on the `sf` and `stars` packages. The
following table describes the new data examples:

Name | Description
-----|------------
meteoland_interpolator_example | A `meteoland` interpolator object, with daily meteorological data in Catalonia (Spain) for April 2022.
meteoland_meteo_example | Data from meteorological stations in Catalonia (Spain) for April 2022.
meteoland_meteo_no_topo_example | Data from meteorological stations in Catalonia (Spain) for April 2022, without topographical information.
meteoland_topo_example | Topographical information for meteorological stations in Catalonia (Spain).
points_to_interpolate_example | Topographical information for 15 plots located in Catalonia (Spain).
raster_to_interpolate_example | Topographical information for a 0.01 degree grid (10x10 cells) raster located in central Catalonia (Spain)

## The new interpolation process

The interpolation of meteorological information requires two kinds of information:

  1. The **topographical information** of the target locations to interpolate. This includes elevation,
  aspect and slope. Elevation is the only mandatory topography variable, but interpolation
  results improve when also aspect and slope are provided in mountain areas.
  
  2. The reference **meteorological information** that we will use to build the interpolator
  object. This information can come from meteorological stations in the area we are interested
  on or, alternatively, can be extracted from available rasters with meteorological variables.

### Topographical information

In order to interpolate, we need our locations in a format that `meteoland` can understand. For
point locations, this is a `sf` object including the `elevation` (in m.a.s.l.), `slope` (in degrees) and `aspect` (in degrees) variables, such as:

```{r points_to_interpolate_example}
points_to_interpolate_example
```

For spatially-continuous data (i.e. a raster), we need a `stars` object including `elevation` (in m.a.s.l.), `slope` (in degrees) and
`aspect` (in degrees) as attributes, such as:

```{r raster_to_interpolate_example}
raster_to_interpolate_example
```

Both, `sf` and `stars` R packages have the necessary functions to read most spatial formats,
the only thing to consider is ensuring that the topographical variables are included, have the proper units and mandatory names
(`elevation`, `aspect`, `slope`). 

### Meteorological information

For interpolating the meteorological variables in our locations (see above), we need a
reference meteorological data. This is a `sf` object with the reference locations, and daily values of the
weather variables needed to perform the interpolation, for example:

```{r meteoland_meteo_example}
meteoland_meteo_example
```

`meteoland` expects variable names to be as indicated in the example:

```{r meteo_names}
names(meteoland_meteo_example)
```

The only mandatory variables are `MinTemperature` and `MaxTemperature`. Other variables
(`Precipitation`, `WindSpeed`...), when present, allow for a more complete weather
interpolation.

For more information on preparing meteorological data for `meteoland`, see
`vignette("reshaping-meteo", package = "meteoland")`

### Quick interpolation (if everything is ok)

With the necessary data in the correct format we can perform the interpolation right away:

```{r quick_interpolation}
# creating the interpolator object
interpolator <- with_meteo(meteoland_meteo_example) |>
  create_meteo_interpolator()

# performing the interpolation
points_interpolated <- points_to_interpolate_example |>
  interpolate_data(interpolator)
points_interpolated
```

Let's see this step by step.

  - `with_meteo(...)` ensures that the provided meteorological information is in the correct format
  for using it with `meteoland`. It performs several checks and informs of any error found.
  For example, if the meteorological information doesn't have the mandatory variables, an informative
  error is shown:
  
```{r non_mandatory_vars_in_meteo, error=TRUE}
meteo_without_temp <- meteoland_meteo_example
meteo_without_temp[["MinTemperature"]] <- NULL
meteo_without_temp[["MaxTemperature"]] <- NULL
with_meteo(meteo_without_temp)
```
  
  - `create_meteo_interpolator(...)` creates the interpolator object from the meteorological information.
  This object stores not only the meteorological information, but also the parameters that will be used in the interpolation
  process. These parameters can be supplied as a list in the `params` argument from
  `create_meteo_interpolator()` function. If not supplied, a default set of parameters is
  used. At any point, we can display the interpolation parameters using `get_interpolation_params()`:
  
```{r interpolatior_params}
# parameters
get_interpolation_params(interpolator)
```

  - `interpolate_data(...)` performs the interpolation using the topography provided and the
  interpolator object. If the topographical information is in an `sf` object, as in the example above,
  `interpolate_data` returns the same `sf` object with an additional column called
  `interpolated_data`:
  
```{r interpolated_data}
# interpolated meteo for the first location
points_interpolated[["interpolated_data"]][1]
```

We can "unnest" the results to get the data in a *long* format (each combination of
location and date in a different row):

```{r long}
tidyr::unnest(points_interpolated, cols = "interpolated_data")
```


## Interpolator object

In older versions of meteoland, the interpolator object inherited the `MeteorologyInterpolationData` class (based on `sp` classes). Starting on `meteoland v2.0.0`, `MeteorologyInterpolationData` class is deprecated, and
the interpolator object created by `create_meteo_interpolator()` inherits directly from
`stars` class.

```{r interpolator_class}
class(interpolator)
```

This object is a data cube, with reference weather locations and dates as dimensions, and meteorological
and topographical variables as attributes.

```{r interpolator_description}
interpolator
```

The object also contains the interpolation parameters as an attribute, that can be accessed with
`get_interpolation_params()`.

```{r get_interpolation_params}
get_interpolation_params(interpolator)
```

Interpolation parameters can also be changed with `set_interpolation_params()`.

```{r set_interpolation_params}
# wind_height parameter
get_interpolation_params(interpolator)$wind_height

# set a new wind_height parameter and check
interpolator <- set_interpolation_params(interpolator, params = list(wind_height = 5))
get_interpolation_params(interpolator)$wind_height
```

### Writing and reading interpolator objects

Interpolator objects can be reused for different interpolations exercises within the area covered
by the interpolator. To allow interpolator objects to be shared between sessions,
`meteoland` offers functions to write and read these objects. The interpolator
is saved in NetCDF-CF format (https://cfconventions.org/cf-conventions/cf-conventions.html)
and can be also opened with any GIS software that supports NetCDF-CF.

```{r writing_interpolator}
temporal_folder <- tempdir()
write_interpolator(interpolator, file.path(temporal_folder, "interpolator.nc"))
# file should exists now
file.exists(file.path(temporal_folder, "interpolator.nc"))
```

To load the interpolator in your session again, you can use the `read_interpolator()`
function.

```{r reading_interpolator}
file_interpolator <- read_interpolator(file.path(temporal_folder, "interpolator.nc"))
# the read interpolator should be identical to the one we have already
identical(file_interpolator, interpolator)
```

### Interpolator calibration

Interpolation parameters can be calibrated for individual variables before performing the
interpolation process. In fact, **it's recommended** to calibrate the interpolator object
before using it, as the default interpolation parameters can be not adequate for the
studied area. `meteoland` offers a calibration process with the `interpolator_calibration()`
function.

    **Important!** Calibration process for one variable can take a long time to finish,
    as it performs a *leave-one-out* interpolation for all stations present in the
    interpolator and all combinations of N and alpha sequences provided. In this example
    we reduce the N and alpha test values for the process to be faster, but is recommended
    to explore a wider range of these values.

```{r interpolator_calibration}
# min temperature N and alpha before calibration
get_interpolation_params(interpolator)$N_MinTemperature
get_interpolation_params(interpolator)$alpha_MinTemperature

# calibration
interpolator <- interpolator_calibration(
  interpolator,
  variable = "MinTemperature",
  N_seq = c(10, 20),
  alpha_seq = c(1, 10),
  update_interpolation_params = TRUE
)

# parameters after calibration
get_interpolation_params(interpolator)$N_MinTemperature
get_interpolation_params(interpolator)$alpha_MinTemperature
```

One advantage of the new data flows in `meteoland` is that we can *pipe* the creation and the
calibration of the interpolator, as well as the writing:

```{r preparing_interpolator}
interpolator <- with_meteo(meteoland_meteo_example) |>
  create_meteo_interpolator() |>
  interpolator_calibration(
    variable = "MinTemperature",
    N_seq = c(10, 20),
    alpha_seq = c(1, 10),
    update_interpolation_params = TRUE
  ) |>
  interpolator_calibration(
    variable = "MaxTemperature",
    N_seq = c(10, 20),
    alpha_seq = c(1, 10),
    update_interpolation_params = TRUE
  ) |>
  interpolator_calibration(
    variable = "DewTemperature",
    N_seq = c(10, 20),
    alpha_seq = c(1, 10),
    update_interpolation_params = TRUE
  ) |>
  write_interpolator(
    filename = file.path(temporal_folder, "interpolator.nc"),
    .overwrite = TRUE
  )
```

This way we can create and calibrate the interpolator once, and using it in future
sessions, avoiding the time consuming step of calibrating every time.

## Interpolation validation

The interpolation process can be cross validated. `meteoland` offers the possibility with the
`interpolation_cross_validation()` function.
This function takes an interpolator object and calculates different error measures.

```{r cross_validation}
cross_validation <- interpolation_cross_validation(interpolator, verbose = FALSE)
cross_validation$errors
cross_validation$station_stats
cross_validation$dates_stats
cross_validation$r2
```

## Interpolation utils

`meteoland` also offers some utilities to work with the interpolated data.

### Temporal summary of interpolated data

`meteoland` works at the daily scale. But sometimes the data needs to be aggregated into
bigger temporal scales (monthly, quarterly, yearly...). This can be done with the
`summarise_interpolated_data()` function. This function takes the result of
`interpolate_data` and creates summaries in the desired frequency.  
The function returns the same interpolated data given as input, but with the weekly summary in an additional column.

```{r summarise_interpolated_data}
summarise_interpolated_data(
  points_interpolated,
  fun = "mean",
  frequency = "week"
)
```

### Calculating rainfall erosivity

`meteoland` also offers the possibility of calculating the rainfall erosivity value, with
the `precipitation_rainfall_erosivity()` function. This can be used for individual locations:

```{r erosivity_one_location}
precipitation_rainfall_erosivity(
  points_interpolated$interpolated_data[[1]],
  longitude = sf::st_coordinates(points_interpolated$geometry[[1]])[,1],
  scale = 'month'
)
```

But also for all locations in the results obtained from the call to `interpolate_data()`:

```{r erosivity_mutate}
points_interpolated |>
  mutate(erosivity = precipitation_rainfall_erosivity(
    interpolated_data,
    longitude = sf::st_coordinates(geometry)[,1],
    scale = 'month'
  ))
```

### Piping all together

`meteoland` new data flows also allows for piping all processes:

```{r interpolation_piped}
points_interpolated <- points_to_interpolate_example |>
  interpolate_data(interpolator) |>
  summarise_interpolated_data(
    fun = "mean",
    frequency = "week"
  ) |>
  summarise_interpolated_data(
    fun = "max",
    frequency = "month"
  ) |>
  mutate(
    monthly_erosivity = precipitation_rainfall_erosivity(
      interpolated_data,
      longitude = sf::st_coordinates(geometry)[,1],
      scale = 'month'
    )
  )

points_interpolated
```

## Interpolation on raster data

We can use the `interpolate_data()` function in a raster type of data. All we need in this case is the topography information
in a `stars` object, with `elevation`, `aspect` and `slope` variables as raster attributes:

```{r raster_to_interpolate}
raster_to_interpolate_example
```

In this case, the raster is a 0.01 degree grid (10x10 cells) in central Catalonia. As the raster is
inside the area covered by the interpolator object we created before, we will use it.

```{r raster_interpolation}
raster_interpolated <- raster_to_interpolate_example |>
  interpolate_data(interpolator)

raster_interpolated
```

As we can see, the returned object is the same `stars` raster provided to the function, with the
interpolated meteorological variables as new attributes. Dimensions now include the date, besides the input's
latitude and longitude.

### Temporal aggregation in raster

Like with the point location example, interpolated data in raster format can also be aggregated temporally.

```{r raster_temporal_agg}
summarise_interpolated_data(
  raster_interpolated,
  fun = "mean",
  frequency = "week"
)
```

In this case the result is the aggregated variables as attributes, and the time dimension is aggregated in the desired frequency.

### Piping raster interpolation

As with points, raster interpolation and utilities can also be piped. This way, if we are only
interested in the mean monthly temperature in our study area, we can do:

```{r raster_piped}
monthly_mean_temperature <- raster_to_interpolate_example |>
  interpolate_data(interpolator, variables = "Temperature") |>
  summarise_interpolated_data(
    fun = "max",
    frequency = "month",
    variable = "MeanTemperature"
  )

plot(monthly_mean_temperature)
```

# Appendix {#appendix}

## Equivalence table

Function (< 2.0.0) | Equivalence (>= 2.0.0) | deprecated
--------------|----------------|-------------
`averagearea` | No equivalence, aggregating by area can be done with the `sf` package | `TRUE`
`correctionpoint`, `correctionpoints`, `correctionpoints.errors`, `correction_series`, `defaultCorrectionParams` | No equivalence, better bias correction methods are provided by other packages (see package `MBC` for example) | `TRUE`
`defaultInterpolationParams` | No change | `FALSE`
`download_*` functions | No equivalence, weather download functions are now provided by `meteospain` package | `TRUE`
`extractdates`, `extractgridindex`, `extractgridpoints`, `extractNetCDF`, `extractvars` | No equivalence, not needed as the meteo objects are now `sf` objects | `TRUE`
`humidity_*` conversion tools | No change | `FALSE`
`interpolation.calibration` | `interpolator_calibration` | `TRUE`
`interpolation.calibration.fmax` | `interpolator_calibration` | `TRUE`
`interpolation.coverage` | No equivalence | `TRUE`
`interpolation.cv` | `interpolation_cross_validation` | `TRUE`
`interpolationgrid`, `interpolationpixels`, `interpolationpoints` | `interpolate_data` | `TRUE`
`mergegrid`, `mergepoints` | No equivalence, meteorological objects are now `sf` objects and can be merged, joined or filtered as any data.frame | `TRUE`
`meteocomplete` | `complete_meteo` | `TRUE`
`meteoplot` | No equivalence, meteo objects are now sf objects and can be plotted as any other data.frame | `TRUE`
`Meteorology_*_Data` | No equivalence, spatial classes based on `sp` are now deprecated in `meteoland` | `TRUE`
`penman`, `penmanmonteith` | No changes | `FALSE`
`plot.interpolation.cv` | No equivalence | `TRUE`
`precipitation_concentration` | No equivalence, precipitation_concentration utility is deprecated and will be removed in future versions | `TRUE`
`precipitation_rainfallErosivity` | `precipitation_rainfall_erosivity` | `TRUE`
`radiation_*` utility functions | No change | `FALSE`
`readmeteorology*` | No equivalence, spatial classes based on `sp` are now deprecated in `meteoland` | `TRUE`
`readNetCDF*` | No equivalence, NetCDF files can be managed with more recent and up to date R packages (ncmeta, stars...) | `TRUE`
`readWindNinjaWindFields` | No equivalence | `TRUE`
`reshapemeteospain` | `meteospain2meteoland` | `TRUE`
`reshapeworldmet` | `worldmet2meteoland` | `TRUE`
`reshapeweathercan` | No equivalence, `weathercan` package was removed from CRAN and the functions are deprecated | `TRUE`
`Spatial**Meteorology`, `Spatial**Topography` | No equivalence, spatial classes based on `sp` are now deprecated in `meteoland` | `TRUE`
`summary*` | `summarise_interpolate_data`, `summarise_interpolator` | `TRUE`
`utils_*` | No change | `FALSE`
`weathergeneration`, `defaultGenerationParams` | No equivalence, current weather generation methods are currently deprecated because they operate with classes that are deprecated themselves, but for future versions, we plan to keep the functionality in new functions. | `TRUE`
`writemeteorology*` | No equivalence, spatial classes based on `sp` are now deprecated in `meteoland` | `TRUE`

---
title: "Reshaping meteorological data for meteoland"
author: "Victor Granda"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Reshaping meteorological data for meteoland}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(meteoland)
library(stars)
library(dplyr)
```

```{r data_preparation, echo=FALSE}
unformatted_meteo <- tibble(
  date = rep(seq(from = as.Date("2022-12-01"), to = as.Date("2022-12-05"), by = 1), 3),
  station = rep(letters[1:3], each = 5),
  latitude = rep(c(41.35, 40.11, 42.00), each = 5),
  longitude = rep(c(-0.33, 0.12, 1.12), each = 5),
  min_temp = rnorm(15, 10, 6),
  max_temp = min_temp + rnorm(15, 10, 3),
  rh = rnorm(15, 50, 25)
) |>
  dplyr::mutate(
    rh = dplyr::if_else(rh > 100, 100, rh),
    rh = dplyr::if_else(rh < 20, 20, rh)
  )
```


## Meteorological data format

For the interpolation of meteorological variables on our target locations, we need meteorological data for a reference set of locations. In `meteoland` this is a `sf` object with the spatial coordinates of our reference
locations (usually meteorological stations) and daily values of the meteorological
variables needed to perform the interpolation, *i.e.*:

```{r meteoland_meteo_example}
meteoland_meteo_example
```

`meteoland` expects names to be as in the example:

```{r meteo_names}
names(meteoland_meteo_example)
```

The only mandatory variables are `MinTemperature` and `MaxTemperature`. Other variables
(`Precipitation`, `WindSpeed`...), when present, allow for a more complete
interpolation.

## Converting meteorological data to `meteoland` format

Meteorological data can come in many formats and with different names for the same
variables. As we saw above, we need to convert it to a `meteoland` compliant
format (`sf` object with correct names).

### Converting from data frames

If we have our meteorological data in a data.frame (*i.e.* we obtained it from
our own weather stations, or download it from other source in this format) we can
just simply transform it to the desired format:

```{r unformatted_meteo}
unformatted_meteo
```

```{r unformatted_transformation}
ready_meteo <- unformatted_meteo |>
  # convert names to correct ones
  dplyr::mutate(
    MinTemperature = min_temp,
    MaxTemperature = max_temp,
    MeanRelativeHumidity = rh
  ) |>
  # transform to sf (WGS84)
  sf::st_as_sf(
    coords = c("longitude", "latitude"),
    crs = sf::st_crs(4326)
  )

ready_meteo
```

And *voilà*, we have our meteo data in the correct format

### Meteorological data from other R packages

`meteoland` offers transformation functions for meteorological data downloaded from the
[`meteospain`](https://emf-creaf.github.io/meteospain/) and
[`worldmet`](https://openair-project.github.io/worldmet/) R packages.

#### `meteospain` data

For data coming from `meteospain` package we have the `meteospain2meteoland`
function that transforming the data for us:

```{r meteospain, eval = FALSE}
library(meteospain)
get_meteo_from(
  "meteogalicia",
  meteogalicia_options('daily', as.Date("2022-12-01"), as.Date("2022-12-05"))
) |>
  meteospain2meteoland()
```

#### `worldmet` data

For data coming from `worldmet` package we have the `worldmet2meteoland`
function that do the reshaping for us:

```{r worldmet, eval = FALSE}
library(worldmet)
worldmet::importNOAA("081120-99999", year = 2022) |>
  worldmet2meteoland()
```

### Meteorological data from raster sources

As we have seen, we need the meteorological reference data in a `sf` object (points).
To be able to create an interpolator from a raster, we need to transform the cell values to points
(*i.e.* using the cell center coordinates) for each variable and day and use it to create the
interpolator.

So if we have a multi-layered raster with several dates of meteorological data:

    > Remember that the raster, besides the meteo data, needs to contain also the topographic
    (elevation, aspect and slope) data for the interpolator to work.

```{r raster_meteo_preparing, echo = FALSE}
raster_meteo_reference <- interpolate_data(
  raster_to_interpolate_example, meteoland_interpolator_example, verbose = FALSE
)
```

```{r}
raster_meteo_reference
```

we need to convert it to points:

```{r raster_to_points}
points_meteo_reference <- names(raster_meteo_reference) |>
  # for each variable
  purrr::map(
    # take the variable raster
    ~ raster_meteo_reference[.x] |>
      # convert to sf
      sf::st_as_sf(as_points = TRUE, na.rm = FALSE) |>
      # pivot the data for dates to be in one column
      tidyr::pivot_longer(cols = -geometry, names_to = "dates", values_to = .x) |>
      # convert to tibble to fasten the process
      dplyr::as_tibble() |>
      # convert to date and create stationID
      dplyr::mutate(
        dates = as.Date(dates),
        stationID = as.character(geometry)
      )
  ) |>
  # join all variables
  purrr::reduce(dplyr::left_join) |>
  # create the points sf object
  sf::st_as_sf()

points_meteo_reference
```

And now we can use it to build an interpolator object:

```{r raster_interpolator}
with_meteo(points_meteo_reference) |>
  create_meteo_interpolator()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_meteo_and_topo.R
\name{add_topo}
\alias{add_topo}
\title{Add topography data to meteo object}
\usage{
add_topo(meteo, topo, verbose = getOption("meteoland_verbosity", TRUE))
}
\arguments{
\item{meteo}{meteo object}

\item{topo}{topo object}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
meteo with the topography info added
}
\description{
Add topography data to meteo object
}
\details{
When using meteo data without topography info to create an interpolator,
topography must be added
}
\examples{
# example meteo
data(meteoland_meteo_no_topo_example)
# example topo
data(meteoland_topo_example)
# add topo
with_meteo(meteoland_meteo_no_topo_example) |>
  add_topo(meteoland_topo_example)

}
\seealso{
Other interpolator functions:
\code{\link{create_meteo_interpolator}()},
\code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
\code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()},
\code{\link{write_interpolator}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolator.R
\name{write_interpolator}
\alias{write_interpolator}
\title{Write the interpolator object}
\usage{
write_interpolator(
  interpolator,
  filename,
  .overwrite = FALSE,
  .verbose = getOption("meteoland_verbosity", TRUE)
)
}
\arguments{
\item{interpolator}{meteoland interpolator object, as created by
\code{\link{create_meteo_interpolator}}}

\item{filename}{file name for the interpolator nc file}

\item{.overwrite}{logical indicating if the file should be overwritten if it
already exists}

\item{.verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
invisible interpolator object, to allow using this function as a
step in a pipe
}
\description{
Write the interpolator object to a file
}
\details{
This function writes the interpolator object created with
\code{\link{create_meteo_interpolator}} in a NetCDF-CF standard
compliant format, as specified in
https://cfconventions.org/cf-conventions/cf-conventions.html
}
\examples{

\donttest{
# example interpolator
data(meteoland_interpolator_example)

# temporal folder
tmp_dir <- tempdir()

# write interpolator
write_interpolator(
  meteoland_interpolator_example,
  file.path(tmp_dir, "meteoland_interpolator_example.nc"),
  .overwrite = TRUE
)

# check file exists
file.exists(file.path(tmp_dir, "meteoland_interpolator_example.nc"))

# read it again
read_interpolator(file.path(tmp_dir, "meteoland_interpolator_example.nc"))
}

}
\seealso{
Other interpolator functions: \code{\link{add_topo}()},
\code{\link{create_meteo_interpolator}()},
\code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
\code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()}
}
\author{
Victor Granda \enc{García}{Garcia}, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_reshape_meteo.R
\name{complete_meteo}
\alias{complete_meteo}
\title{Complete missing meteo variables}
\usage{
complete_meteo(
  meteo,
  params = defaultInterpolationParams(),
  verbose = getOption("meteoland_verbosity", TRUE)
)
}
\arguments{
\item{meteo}{meteoland weather data}

\item{params}{A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
the same \code{meteo} data provided with the the variables completed
}
\description{
Calculates missing values of relative humidity, radiation and potential
evapotranspiration from a data frame with daily values of
minimum/maximum/mean temperature and precipitation. The function
takes a meteo object (with meteoland names) and complete any
missing variable if it is possible
}
\details{
#' The function fills values for humidity, radiation and PET only if they are
missing in the input data frame. If a column 'SpecificHumidity' is present
in the input data, relative humidity is calculated from it. Otherwise,
relative humidity is calculated assuming that dew point temperature equals
the minimum temperature. Potential solar radiation is calculated from
latitude, slope and aspect. Incoming solar radiation is then corrected
following Thornton & Running (1999) and potential evapotranspiration
following Penman (1948).
}
\examples{

\donttest{
# example data
data("meteoland_meteo_example")

# remove MinRelativeHumidity
meteoland_meteo_example$MinRelativeHumidity <- NULL
# complete vars
completed_meteo <- complete_meteo(meteoland_meteo_example)
# check MinRelativeHumidity
completed_meteo$MinRelativeHumidity
}

}
\references{
Thornton, P.E., Running, S.W., 1999. An improved algorithm for
estimating incident daily solar radiation from measurements of temperature,
humidity, and precipitation. Agric. For. Meteorol. 93, 211-228.

Penman, H. L. 1948. Natural evaporation from open water, bare soil and
grass. Proceedings of the Royal Society of London. Series A. Mathematical
and Physical Sciences, 193, 120-145.
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF

Victor Granda \enc{García}{Garcia}, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{raster_to_interpolate_example}
\alias{raster_to_interpolate_example}
\title{Example raster data set for interpolation of weather variables}
\format{
stars object
}
\source{
ICGC
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Example raster data set of spatial location and topography
records from Catalonia (NE Spain). Cell size is 1km x 1km and raster size is 10x10 cells.
}
\examples{

data(raster_to_interpolate_example)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{points_to_interpolate_example}
\alias{points_to_interpolate_example}
\title{Example data set of points for interpolation of weather variables}
\format{
sf object
}
\source{
Spanish National Forest Inventory
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Example data set of spatial location and topography
records from 15 experimental plots in Catalonia (NE Spain).
}
\examples{

data(points_to_interpolate_example)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolator.R
\name{set_interpolation_params}
\alias{set_interpolation_params}
\title{Setting interpolation parameters in an interpolator object}
\usage{
set_interpolation_params(
  interpolator,
  params = NULL,
  verbose = getOption("meteoland_verbosity", TRUE)
)
}
\arguments{
\item{interpolator}{interpolator object to update}

\item{params}{list with the parameters provided by the user}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
The same interpolator object provided, with the updated
interpolation parameters
}
\description{
Changing or updating interpolation parameters in an interpolator object
}
\details{
This function ensures that if no parameters are provided, the default ones
are used (see \code{\link{defaultInterpolationParams}}). Also, if params are
partially provided, this function ensures that the rest of the parameters
are not changed.
}
\examples{
# example interpolator
data(meteoland_interpolator_example)
# store the actual parameters
old_parameters <- get_interpolation_params(meteoland_interpolator_example)
# we can provide only the parameter we want to change
meteoland_interpolator_example <- set_interpolation_params(
  meteoland_interpolator_example,
  list(debug = TRUE)
)
# check
get_interpolation_params(meteoland_interpolator_example)$debug
# compare with old
old_parameters$debug
# the rest should be the same
setdiff(old_parameters, get_interpolation_params(meteoland_interpolator_example))

}
\seealso{
Other interpolator functions: \code{\link{add_topo}()},
\code{\link{create_meteo_interpolator}()},
\code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
\code{\link{with_meteo}()}, \code{\link{write_interpolator}()}
}
\author{
Victor Granda \enc{García}{Garcia}, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolation.R
\name{interpolate_data}
\alias{interpolate_data}
\title{Interpolation process for spatial data}
\usage{
interpolate_data(
  spatial_data,
  interpolator,
  dates = NULL,
  variables = NULL,
  ignore_convex_hull_check = FALSE,
  verbose = getOption("meteoland_verbosity", TRUE)
)
}
\arguments{
\item{spatial_data}{An sf or stars raster object to interpolate}

\item{interpolator}{A meteoland interpolator object, as created by
\code{\link{create_meteo_interpolator}}}

\item{dates}{vector with dates to interpolate (must be within the
interpolator date range). Default to NULL (all dates present in the
interpolator object)}

\item{variables}{vector with variable names to be interpolated. NULL (default),
will interpolate all variables. Accepted names are "Temperature", "Precipitation",
"RelativeHumidity", "Radiation" and "Wind"}

\item{ignore_convex_hull_check}{Logical indicating whether errors in convex hull checks should be ignored.
Checking for points to be inside the convex hull will normally raise an error
if >10\% of points are outside. Setting \code{ignore_convex_hull_check = TRUE} means
that a warning is raised but interpolation is performed, which can be useful to users interpolating
on a few points close but outside of the convex hull.}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
an object with the same class and structure as the provided spatial
data with the results of the interpolation joined. In the case of spatial
data being an sf, the results are added as a list-type column that can be
unnested with \code{\link[tidyr]{unnest}}. In the case of a stars raster
object, interpolation results are added as attributes (variables)
}
\description{
Interpolate spatial data to obtain downscaled meteorologic variables
}
\details{
This function takes a spatial data object (sf or stars raster), an
interpolator object (\code{\link{create_meteo_interpolator}}) and a
vector of dates to perform the interpolation of the meteorologic variables
for the spatial locations present in the \code{spatial_data} object.
}
\section{Spatial data}{


The spatial data provided must be of two types. (I) A sf object containing
POINT for each location to interpolate or (II) a stars raster object for
which the interpolation should be done. Independently of the class of
\code{spatial_data} it has to have some mandatory variables, namely
\code{elevation}. It should also contain \code{aspect} and \code{slope} for
a better interpolation process, though this two variables are not mandatory.
}

\examples{

\donttest{
# example of data to interpolate and example interpolator
data("points_to_interpolate_example")
data("meteoland_interpolator_example")

# interpolate data
res <- interpolate_data(points_to_interpolate_example, meteoland_interpolator_example)

# check result
# same class as input data
class(res)
# data
res
# results for the first location
res[["interpolated_data"]][1]
# unnest results
tidyr::unnest(res, cols = "interpolated_data")
}

}
\author{
Victor Granda \enc{García}{Garcia}, EMF-CREAF

Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{interpolation_precipitation}
\alias{interpolation_precipitation}
\alias{interpolation_dewtemperature}
\alias{interpolation_temperature}
\alias{interpolation_wind}
\title{Low-level interpolation functions}
\usage{
interpolation_precipitation(
  Xp,
  Yp,
  Zp,
  X,
  Y,
  Z,
  P,
  Psmooth,
  iniRp = 140000,
  alpha_event = 6.25,
  alpha_amount = 6.25,
  N_event = 20L,
  N_amount = 20L,
  iterations = 3L,
  popcrit = 0.5,
  fmax = 0.95,
  debug = FALSE
)

interpolation_dewtemperature(
  Xp,
  Yp,
  Zp,
  X,
  Y,
  Z,
  T,
  iniRp = 140000,
  alpha = 3,
  N = 30L,
  iterations = 3L,
  debug = FALSE
)

interpolation_temperature(
  Xp,
  Yp,
  Zp,
  X,
  Y,
  Z,
  T,
  iniRp = 140000,
  alpha = 3,
  N = 30L,
  iterations = 3L,
  debug = FALSE
)

interpolation_wind(
  Xp,
  Yp,
  WS,
  WD,
  X,
  Y,
  iniRp = 140000,
  alpha = 2,
  N = 1L,
  iterations = 3L,
  directionsAvailable = TRUE
)
}
\arguments{
\item{Xp, Yp, Zp}{Spatial coordinates and elevation (Zp; in m.a.s.l) of target
points.}

\item{X, Y, Z}{Spatial coordinates and elevation (Zp; in m.a.s.l) of reference
locations (e.g. meteorological stations).}

\item{P}{Precipitation at the reference locations (in mm).}

\item{Psmooth}{Temporally-smoothed precipitation at the reference locations
(in mm).}

\item{iniRp}{Initial truncation radius.}

\item{iterations}{Number of station density iterations.}

\item{popcrit}{Critical precipitation occurrence parameter.}

\item{fmax}{Maximum value for precipitation regression extrapolations (0.6
equals to a maximum of 4 times extrapolation).}

\item{debug}{Boolean flag to show extra console output.}

\item{T}{Temperature (e.g., minimum, maximum or dew temperature) at the
reference locations (in degrees).}

\item{alpha, alpha_amount, alpha_event}{Gaussian shape parameter.}

\item{N, N_event, N_amount}{Average number of stations with non-zero weights.}

\item{WS, WD}{Wind speed (in m/s) and wind direction (in degrees from north
clock-wise) at the reference locations.}

\item{directionsAvailable}{A flag to indicate that wind directions are
available (i.e. non-missing) at the reference locations.}
}
\value{
All functions return a vector with interpolated values for the
target points.
}
\description{
Low-level functions to interpolate meteorology (one day) on a set of points.
}
\details{
This functions exposes internal low-level interpolation functions written in C++
not intended to be used directly in any script or function. The are maintained for
compatibility with older versions of the package and future versions of meteoland
will remove this functions (they will be still accessible through the triple colon
notation (\code{:::}), but their use is not recommended)
}
\section{Functions}{
\itemize{
\item \code{interpolation_precipitation()}: Precipitation

\item \code{interpolation_dewtemperature()}: Dew temperature

\item \code{interpolation_wind()}: Wind

}}
\examples{

Xp <- as.numeric(sf::st_coordinates(points_to_interpolate_example)[,1])
Yp <- as.numeric(sf::st_coordinates(points_to_interpolate_example)[,2])
Zp <- points_to_interpolate_example$elevation
X <- as.numeric(
  sf::st_coordinates(stars::st_get_dimension_values(meteoland_interpolator_example, "station"))[,1]
)
Y <- as.numeric(
  sf::st_coordinates(stars::st_get_dimension_values(meteoland_interpolator_example, "station"))[,2]
)
Z <- as.numeric(meteoland_interpolator_example[["elevation"]][1,])
Temp <- as.numeric(meteoland_interpolator_example[["MinTemperature"]][1,])
P <- as.numeric(meteoland_interpolator_example[["Precipitation"]][1,])
Psmooth <- as.numeric(meteoland_interpolator_example[["SmoothedPrecipitation"]][1,])
WS <- as.numeric(meteoland_interpolator_example[["WindSpeed"]][1,])
WD <- as.numeric(meteoland_interpolator_example[["WindDirection"]][1,])
iniRp <- get_interpolation_params(meteoland_interpolator_example)$initial_Rp
alpha <- get_interpolation_params(meteoland_interpolator_example)$alpha_MinTemperature
N <- get_interpolation_params(meteoland_interpolator_example)$N_MinTemperature
alpha_event <- get_interpolation_params(meteoland_interpolator_example)$alpha_PrecipitationEvent
N_event <- get_interpolation_params(meteoland_interpolator_example)$N_PrecipitationEvent
alpha_amount <- get_interpolation_params(meteoland_interpolator_example)$alpha_PrecipitationAmount
N_amount <- get_interpolation_params(meteoland_interpolator_example)$N_PrecipitationAmount
alpha_wind <- get_interpolation_params(meteoland_interpolator_example)$alpha_Wind
N_wind <- get_interpolation_params(meteoland_interpolator_example)$N_Wind
iterations <- get_interpolation_params(meteoland_interpolator_example)$iterations
popcrit <- get_interpolation_params(meteoland_interpolator_example)$pop_crit
fmax <- get_interpolation_params(meteoland_interpolator_example)$f_max
debug <- get_interpolation_params(meteoland_interpolator_example)$debug

interpolation_temperature(
  Xp, Yp, Zp,
  X[!is.na(Temp)], Y[!is.na(Temp)], Z[!is.na(Temp)],
  Temp[!is.na(Temp)],
  iniRp, alpha, N, iterations, debug
)

interpolation_wind(
  Xp, Yp,
  WS[!is.na(WD)], WD[!is.na(WD)],
  X[!is.na(WD)], Y[!is.na(WD)],
  iniRp, alpha_wind, N_wind, iterations, directionsAvailable = FALSE
)

interpolation_precipitation(
  Xp, Yp, Zp,
  X[!is.na(P)], Y[!is.na(P)], Z[!is.na(P)],
  P[!is.na(P)], Psmooth[!is.na(P)],
  iniRp, alpha_event, alpha_amount, N_event, N_amount,
  iterations, popcrit, fmax, debug
)

}
\references{
Thornton, P.E., Running, S.W., White, M. A., 1997. Generating
surfaces of daily meteorological variables over large regions of complex
terrain. J. Hydrol. 190, 214–251. doi:10.1016/S0022-1694(96)03128-9.

De Caceres M, Martin-StPaul N, Turco M, Cabon A, Granda V (2018) Estimating
daily meteorological data and downscaling climate models over landscapes.
Environmental Modelling and Software 108: 186-196.
}
\seealso{
\code{\link{defaultInterpolationParams}}
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_summary_interpolated_data.R
\name{summarise_interpolated_data}
\alias{summarise_interpolated_data}
\title{Summarise interpolated data by temporal dimension}
\usage{
summarise_interpolated_data(
  interpolated_data,
  fun = "mean",
  frequency = NULL,
  vars_to_summary = c("MeanTemperature", "MinTemperature", "MaxTemperature",
    "Precipitation", "MeanRelativeHumidity", "MinRelativeHumidity",
    "MaxRelativeHumidity", "Radiation", "WindSpeed", "WindDirection", "PET"),
  dates_to_summary = NULL,
  months_to_summary = 1:12,
  verbose = getOption("meteoland_verbosity", TRUE),
  ...
)
}
\arguments{
\item{interpolated_data}{An interpolated data object as returned by
\code{\link{interpolate_data}}.}

\item{fun}{The function to use for summarising the data.}

\item{frequency}{A string indicating the interval specification (allowed ones
are "week", "month", "quarter" and "year"). If NULL (default), aggregation
is done in one interval for all the dates present.}

\item{vars_to_summary}{A character vector with one or more variable names to
summarise. By default, all interpolated variables are summarised.}

\item{dates_to_summary}{A Date object to define the dates to be summarised.
If NULL (default), all dates in the interpolated data are processed.}

\item{months_to_summary}{A numeric vector with the month numbers to subset
the data before summarising. (e.g. \code{c(7,8)} for July and August).
This parameter allows studying particular seasons, when combined with
\code{frequency}. For example \code{frequency = "years"} and
\code{months = 6:8} leads to summarizing summer months of each year.}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}

\item{...}{Arguments needed for \code{fun}}
}
\value{
For a nested interpolated data, the same sf object with a new column
with the temporal summaries. For an unnested interpolated
data, a data.frame with the summarised meteo variables. For an interpolated
raster (stars object), the same raster with the temporal dimension aggregated
as desired.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Summarises the interpolated meteorology in one or more locations by the desired
temporal scale
}
\details{
If \code{interpolated_data} is a nested interpolated data sf object, as
returned by \code{\link{interpolate_data}}, temporal summary is done for each
location present in the interpolated data. If \code{interpolated_data} is
an unnested interpolated data sf object, temporal summary is done for all
locations together. If \code{interpolated_data} is a single location data.frame
containing the dates and the interpolated variables, temporal summary is done
for that location. If \code{interpolated_data} is a stars object as returned
by \code{\link{interpolate_data}}, temporal summary is done for all the raster.
}
\examples{
\donttest{
# points interpolation aggregation
points_to_interpolate_example |>
  interpolate_data(meteoland_interpolator_example, verbose = FALSE) |>
  summarise_interpolated_data()

# raster interpolation aggregation
raster_to_interpolate_example |>
  interpolate_data(meteoland_interpolator_example, verbose = FALSE) |>
  summarise_interpolated_data()
}

}
\author{
\enc{Víctor}{Victor} Granda \enc{García}{Garcia}, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_reshape_meteo.R
\name{worldmet2meteoland}
\alias{worldmet2meteoland}
\title{From worldmet to meteoland meteo objects}
\usage{
worldmet2meteoland(
  meteo,
  complete = FALSE,
  params = defaultInterpolationParams()
)
}
\arguments{
\item{meteo}{worldmet meteo object.}

\item{complete}{logical indicating if the meteo data missing variables
should be calculated (if possible). Default to FALSE.}

\item{params}{A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.}
}
\value{
a compatible meteo object to use with meteoland.
}
\description{
Adapting \code{\link[worldmet]{importNOAA}} meteo objects to meteoland meteo objects
}
\details{
This function converts \code{\link[worldmet]{importNOAA}} meteo objects to
compatible meteoland meteo objects by selecting the needed variables and
adapting the names to comply with meteoland requirements. Also it aggregates
subdaily data as well as complete missing variables if possible (setting
\code{complete = TRUE})
}
\examples{

if (interactive()) {
  # worldmet data
  library(worldmet)
  worldmet_stations <- worldmet::getMeta(lat = 42, lon = 0, n = 2, plot = FALSE)
  worldmet_subdaily_2022 <-
    worldmet::importNOAA(worldmet_stations$code, year = 2022, hourly = TRUE)

  # just convert
  worldmet2meteoland(worldmet_subdaily_2022)
  # convert and complete
  worldmet2meteoland(worldmet_subdaily_2022, complete = TRUE)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{radiation_julianDay}
\alias{radiation_julianDay}
\alias{radiation_dateStringToJulianDays}
\alias{radiation_daylength}
\alias{radiation_daylengthseconds}
\alias{radiation_directDiffuseInstant}
\alias{radiation_directDiffuseDay}
\alias{radiation_potentialRadiation}
\alias{radiation_skyLongwaveRadiation}
\alias{radiation_outgoingLongwaveRadiation}
\alias{radiation_netRadiation}
\alias{radiation_solarRadiation}
\alias{radiation_solarConstant}
\alias{radiation_solarElevation}
\alias{radiation_solarDeclination}
\alias{radiation_sunRiseSet}
\title{Solar radiation utility functions}
\usage{
radiation_julianDay(year, month, day)

radiation_dateStringToJulianDays(dateStrings)

radiation_solarDeclination(J)

radiation_solarConstant(J)

radiation_sunRiseSet(latrad, slorad, asprad, delta)

radiation_solarElevation(latrad, delta, hrad)

radiation_daylength(latrad, slorad, asprad, delta)

radiation_daylengthseconds(latrad, slorad, asprad, delta)

radiation_potentialRadiation(solarConstant, latrad, slorad, asprad, delta)

radiation_solarRadiation(
  solarConstant,
  latrad,
  elevation,
  slorad,
  asprad,
  delta,
  diffTemp,
  diffTempMonth,
  vpa,
  precipitation
)

radiation_directDiffuseInstant(
  solarConstant,
  latrad,
  slorad,
  asprad,
  delta,
  hrad,
  R_s,
  clearday
)

radiation_directDiffuseDay(
  solarConstant,
  latrad,
  slorad,
  asprad,
  delta,
  R_s,
  clearday,
  nsteps = 24L
)

radiation_skyLongwaveRadiation(Tair, vpa, c = 0)

radiation_outgoingLongwaveRadiation(
  solarConstant,
  latrad,
  elevation,
  slorad,
  asprad,
  delta,
  vpa,
  tmin,
  tmax,
  R_s
)

radiation_netRadiation(
  solarConstant,
  latrad,
  elevation,
  slorad,
  asprad,
  delta,
  vpa,
  tmin,
  tmax,
  R_s,
  alpha = 0.08
)
}
\arguments{
\item{year, month, day}{Year, month and day as integers.}

\item{dateStrings}{A character vector with dates in format "YYYY-MM-DD".}

\item{J}{Julian day (integer), number of days since January 1, 4713 BCE at
noon UTC.}

\item{latrad}{Latitude (in radians North).}

\item{slorad}{Slope (in radians).}

\item{asprad}{Aspect (in radians from North).}

\item{delta}{Solar declination (in radians).}

\item{hrad}{Solar hour (in radians).}

\item{solarConstant}{Solar constant (in kW·m-2).}

\item{elevation}{Elevation above sea level (in m).}

\item{diffTemp}{Difference between maximum and minimum temperature (ºC).}

\item{diffTempMonth}{Difference between maximum and minimum temperature,
averaged over 30 days (ºC).}

\item{vpa}{Average daily vapor pressure (kPa).}

\item{precipitation}{Precipitation (in mm).}

\item{R_s}{Daily incident solar radiation (MJ·m-2).}

\item{clearday}{Boolean flag to indicate a clearsky day (vs. overcast).}

\item{nsteps}{Number of daily substeps.}

\item{Tair}{Air temperature (in degrees Celsius).}

\item{c}{Proportion of sky covered by clouds (0-1).}

\item{tmin, tmax}{Minimum and maximum daily temperature (ºC).}

\item{alpha}{Surface albedo (from 0 to 1).}
}
\value{
Values returned for each function are: \itemize{
\item\code{radiation_dateStringToJulianDays}: A vector of Julian days (i.e.
number of days since January 1, 4713 BCE at noon UTC).
\item\code{radiation_daylength}: Day length (in hours).
\item\code{radiation_daylengthseconds}: Day length (in seconds).
\item\code{radiation_directDiffuseInstant}: A vector with instantaneous
direct and diffusive radiation rates (for both SWR and PAR).
\item\code{radiation_directDiffuseDay}: A data frame with instantaneous
direct and diffusive radiation rates (for both SWR and PAR) for each
subdaily time step. \item\code{radiation_potentialRadiation}: Daily
(potential) solar radiation (in MJ·m-2). \item\code{radiation_julianDay}:
Number of days since January 1, 4713 BCE at noon UTC.
\item\code{radiation_skyLongwaveRadiation}: Instantaneous incoming (sky)
longwave radiation (W·m-2).
\item\code{radiation_outgoingLongwaveRadiation}: Daily outgoing longwave
radiation (MJ·m-2·day-1).  \item\code{radiation_netRadiation}: Daily net
solar radiation (MJ·m-2·day-1).  \item\code{radiation_solarConstant}: Solar
constant (in kW·m-2). \item\code{radiation_solarDeclination}: Solar
declination (in radians). \item\code{radiation_solarElevation}: Angle of
elevation of the sun with respect to the horizon (in radians).
\item\code{radiation_solarRadiation}: Daily incident solar radiation
(MJ·m-2·day-1). \item\code{radiation_sunRiseSet}: Sunrise and sunset hours
in hour angle (radians). }
}
\description{
Set of functions used in the calculation of incoming solar radiation and net
radiation.
}
\section{Functions}{
\itemize{
\item \code{radiation_dateStringToJulianDays()}: Date string to julian days

\item \code{radiation_solarDeclination()}: solar declination

\item \code{radiation_solarConstant()}: solar constant

\item \code{radiation_sunRiseSet()}: sun rise and set

\item \code{radiation_solarElevation()}: solar elevation

\item \code{radiation_daylength()}: Day length

\item \code{radiation_daylengthseconds()}: Day length seconds

\item \code{radiation_potentialRadiation()}: Potential radiation

\item \code{radiation_solarRadiation()}: solar Radiation

\item \code{radiation_directDiffuseInstant()}: Direct diffuse instant

\item \code{radiation_directDiffuseDay()}: Direct diffuse day

\item \code{radiation_skyLongwaveRadiation()}: Sky longwave radiation

\item \code{radiation_outgoingLongwaveRadiation()}: Outgoing longwave radiation

\item \code{radiation_netRadiation()}: Net radiation

}}
\note{
Code for \code{radiation_julianDay()},
\code{radiation_solarConstant()} and \code{radiation_solarDeclination()} was
translated to C++ from R code in package 'insol' (by J. G. Corripio).
}
\references{
Danby, J. M. Eqn. 6.16.4 in Fundamentals of Celestial Mechanics,
2nd ed. Richmond, VA: Willmann-Bell, p. 207, 1988.

Garnier, B.J., Ohmura, A., 1968. A method of calculating the direct
shortwave radiation income of slopes. J. Appl. Meteorol. 7: 796-800

McMahon, T. A., M. C. Peel, L. Lowe, R. Srikanthan, and T. R. McVicar. 2013.
Estimating actual, potential, reference crop and pan evaporation using
standard meteorological data: a pragmatic synthesis. Hydrology & Earth
System Sciences 17:1331–1363. See also:
http://www.fao.org/docrep/x0490e/x0490e06.htm.

Reda, I. and Andreas, A. 2003. Solar Position Algorithm for Solar Radiation
Applications. 55 pp.; NREL Report No. TP-560-34302, Revised January 2008.
http://www.nrel.gov/docs/fy08osti/34302.pdf

Spitters, C.J.T., Toussaint, H.A.J.M. and Goudriaan, J. (1986). Separating
the diffuse and direct components of global radiation and its implications
for modeling canopy photosynthesis. I. Components of incoming radiation.
Agricultural and Forest Meteorology, 38, 231–242.
}
\seealso{
\code{\link{interpolate_data}}
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{humidity_relative2dewtemperature}
\alias{humidity_relative2dewtemperature}
\alias{humidity_dewtemperature2relative}
\alias{humidity_specific2relative}
\alias{humidity_relative2specific}
\title{Humidity conversion tools}
\usage{
humidity_relative2dewtemperature(Tc, HR)

humidity_dewtemperature2relative(Tc, Td, allowSaturated = FALSE)

humidity_specific2relative(Tc, HS, allowSaturated = FALSE)

humidity_relative2specific(Tc, HR)
}
\arguments{
\item{Tc}{A numeric vector of temperature in degrees Celsius.}

\item{HR}{A numeric vector of relative humidity (in \%).}

\item{Td}{A numeric vector of dew temperature in degrees Celsius.}

\item{allowSaturated}{Logical flag to allow values over 100\%}

\item{HS}{A numeric vector of specific humidity (unitless).}
}
\value{
A numeric vector with specific or relative humidity.
}
\description{
Functions to transform relative humidity to specific humidity or dew point
temperature and viceversa.
}
\seealso{
\code{\link{complete_meteo}}
}
\author{
Nicholas Martin-StPaul, INRA

Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{meteoland_meteo_example}
\alias{meteoland_meteo_example}
\title{Example data set for meteo data from weather stations}
\format{
sf object
}
\source{
'Servei Meteorològic de Catalunya' (SMC)
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Example data set of spatial location, topography and daily meteorological
records from 189 weather stations in Catalonia (NE Spain) corresponding to
April 2022.
}
\examples{

data(meteoland_meteo_example)

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_utils.R
\name{precipitation_rainfall_erosivity}
\alias{precipitation_rainfall_erosivity}
\title{Precipitation rainfall erosivity}
\usage{
precipitation_rainfall_erosivity(
  meteo_data,
  longitude,
  scale = c("month", "year"),
  average = TRUE
)
}
\arguments{
\item{meteo_data}{A meteo tibble as with the dates and meteorological variables
as returned by \code{\link{interpolate_data}} in the "interpolated_data"
column.}

\item{longitude}{Longitude in degrees.}

\item{scale}{Character, either 'month' or 'year'. Default to 'month'}

\item{average}{Boolean flag to calculate multi-year averages before applying
MedREM's formula.}
}
\value{
A vector of values for each month (in MJ·mm·ha-1·h-1·month-1) or each
year (in MJ·mm·ha-1·h-1·yr-1), depending on the scale
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Function \code{precipitation_rainfall_erosivity()} calculates a multi-year
average of monthly rainfall erosivity using the MedREM model proposed by
Diodato and Bellochi (2010) for the Mediterranean area (see also Guerra et
al. 2016).
}
\details{
MedREM model is: Rm = b0·P·sqrt(d)·(alpha + b1*longitude), where P is
accumulated precipitation and d is maximum daily precipitation. Parameters
used for the MedREM model are b0 = 0.117, b1 = -0.015, alpha = 2. Note that
there is a mistake in Guerra et al. (2016) regarding parameters b1 and a.
}
\examples{
\donttest{
interpolated_example <-
  interpolate_data(points_to_interpolate_example, meteoland_interpolator_example)

precipitation_rainfall_erosivity(
  meteo_data = interpolated_example$interpolated_data[[1]],
  longitude = 2.32,
  scale = "month",
  average = TRUE
)
}

}
\references{
Diodato, N., Bellocchi, G., 2010. MedREM, a rainfall erosivity
model for the Mediterranean region. J. Hydrol. 387, 119–127,
doi:10.1016/j.jhydrol.2010.04.003.

Guerra CA, Maes J, Geijzendorffer I, Metzger MJ (2016) An assessment of soil
erosion prevention by vegetation in Mediterranean Europe: Current trends of
ecosystem service provision. Ecol Indic 60:213–222. doi:
10.1016/j.ecolind.2015.06.043.
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF.

\enc{Víctor}{Victor} Granda \enc{García}{Garcia}, CREAF.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_reshape_meteo.R
\name{meteospain2meteoland}
\alias{meteospain2meteoland}
\title{From meteospain to meteoland meteo objects}
\usage{
meteospain2meteoland(
  meteo,
  complete = FALSE,
  params = defaultInterpolationParams()
)
}
\arguments{
\item{meteo}{meteospain meteo object.}

\item{complete}{logical indicating if the meteo data missing variables
should be calculated (if possible). Default to FALSE.}

\item{params}{A list containing parameters for PET estimation. By default the result of \code{\link{defaultInterpolationParams}}.}
}
\value{
a compatible meteo object to use with meteoland.
}
\description{
Adapting meteospain meteo objects to meteoland meteo objects
}
\details{
This function converts \code{meteospain} R package meteo objects to
compatible meteoland meteo objects by selecting the needed variables and
adapting the names to comply with meteoland requirements.
}
\examples{

if (interactive()) {
  # meteospain data
  library(meteospain)
  mg_april_2022_data <- get_meteo_from(
    "meteogalicia",
    meteogalicia_options("daily", as.Date("2022-04-01"), as.Date("2022-04-30"))
  )

  # just convert
  meteospain2meteoland(mg_april_2022_data)
  # convert and complete
  meteospain2meteoland(mg_april_2022_data, complete = TRUE)

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{penman}
\alias{penman}
\alias{penmanmonteith}
\title{Potential evapotranspiration}
\usage{
penman(
  latrad,
  elevation,
  slorad,
  asprad,
  J,
  Tmin,
  Tmax,
  RHmin,
  RHmax,
  R_s,
  u,
  z = 10,
  z0 = 0.001,
  alpha = 0.25,
  windfun = "1956"
)

penmanmonteith(rc, elevation, Tmin, Tmax, RHmin, RHmax, Rn, u = NA_real_)
}
\arguments{
\item{latrad}{Latitude in radians.}

\item{elevation}{Elevation (in m).}

\item{slorad}{Slope (in radians).}

\item{asprad}{Aspect (in radians from North).}

\item{J}{Julian day, number of days since January 1, 4713 BCE at noon UTC.}

\item{Tmin}{Minimum temperature (degrees Celsius).}

\item{Tmax}{Maximum temperature (degrees Celsius).}

\item{RHmin}{Minimum relative humidity (percent).}

\item{RHmax}{Maximum relative humidity (percent).}

\item{R_s}{Solar radiation (MJ/m2).}

\item{u}{With wind speed (m/s).}

\item{z}{Wind measuring height (m).}

\item{z0}{Roughness height (m).}

\item{alpha}{Albedo.}

\item{windfun}{Wind speed function version, either "1948" or "1956".}

\item{rc}{Canopy vapour flux (stomatal) resistance (s·m-1).}

\item{Rn}{Daily net radiation (MJ·m-2·day-1).}
}
\value{
Potential evapotranspiration (in mm of water).
}
\description{
Functions to calculate potential evapotranspiration using Penman or
Penman-Monteith.
}
\details{
The code was adapted from package `Evapotranspiration', which follows
McMahon et al. (2013). If wind speed is not available, an alternative
formulation for potential evapotranspiration is used as an approximation
(Valiantzas 2006)
}
\section{Functions}{
\itemize{
\item \code{penmanmonteith()}: Penman Monteith method

}}
\references{
Penman, H. L. 1948. Natural evaporation from open water, bare
soil and grass. Proceedings of the Royal Society of London. Series A.
Mathematical and Physical Sciences, 193, 120-145.

Penman, H. L. 1956. Evaporation: An introductory survey. Netherlands Journal
of Agricultural Science, 4, 9-29.

McMahon, T.A., Peel, M.C., Lowe, L., Srikanthan, R., McVicar, T.R. 2013.
Estimating actual, potential, reference crop and pan evaporation using
standard meteorological data: a pragmatic synthesis. Hydrology & Earth
System Sciences 17, 1331–1363. doi:10.5194/hess-17-1331-2013.
}
\seealso{
\code{\link{interpolate_data}}
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolation_cross_validation.R,
%   R/tidy_interpolator.R
\name{interpolation_cross_validation}
\alias{interpolation_cross_validation}
\alias{interpolator_calibration}
\title{Calibration and validation of interpolation procedures}
\usage{
interpolation_cross_validation(
  interpolator,
  stations = NULL,
  verbose = getOption("meteoland_verbosity", TRUE)
)

interpolator_calibration(
  interpolator,
  stations = NULL,
  update_interpolation_params = FALSE,
  variable = "MinTemperature",
  N_seq = seq(10, 30, by = 5),
  alpha_seq = seq(0.25, 10, by = 0.25),
  verbose = getOption("meteoland_verbosity", TRUE)
)
}
\arguments{
\item{interpolator}{A meteoland interpolator object, as created by
\code{\link{create_meteo_interpolator}}}

\item{stations}{A vector with the stations (numeric for station indexes or
character for stations id) to be used to calculate \code{"MAE"}. All
stations with data are included in the training set but predictive
\code{"MAE"} are calculated for the stations subset indicated in
\code{stations} param only. If \code{NULL} all stations are used in the
predictive \code{"MAE"} calculation.}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}

\item{update_interpolation_params}{Logical indicating if the interpolator
object must be updated with the calculated parameters. Default to FALSE}

\item{variable}{A string indicating the meteorological variable for which
interpolation parameters \code{"N"} and \code{"alpha"} will be calibrated.
Accepted values are:
\itemize{
\item{\code{MinTemperature} (kernel for minimum temperature)}
\item{\code{MaxTemperature} (kernel for maximum temperature)}
\item{\code{DewTemperature} (kernel for dew-temperature (i.e. relative humidity))}
\item{\code{Precipitation} (to calibrate the same
kernel for both precipitation events and regression of precipitation amounts;
not recommended)}
\item{\code{PrecipitationAmount} (kernel for regression of precipitation amounts)}
\item{\code{PrecipitationEvent} (kernel for precipitation events)}
}}

\item{N_seq}{Numeric vector with \code{"N"} values to be tested}

\item{alpha_seq}{Numeric vector with \code{"alpha"}}
}
\value{
\code{interpolation_cross_validation} returns a list with the
following items
\itemize{
\item{errors: Data frame with each combination of station and date with
observed variables, predicated variables and the total error
(predicted - observed) calculated for each variable}
\item{station_stats: Data frame with error and bias statistics aggregated by
station}
\item{dates_stats: Data frame with error and bias statistics aggregated by
date}
\item{r2: correlation indexes between observed and predicted values for each
meteorological variable}
}

If \code{update_interpolation_params} is FALSE (default),
\code{interpolator_calibration} returns a list with the following items
\itemize{ \item{MAE: A numeric matrix with the mean absolute error values,
averaged across stations, for each combination of parameters \code{"N"} and
\code{"alpha"}} \item{minMAE: Minimum MAE value} \item{N: Value of parameter
\code{"N"} corresponding to the minimum MAE} \item{alpha: Value of parameter
\code{"alpha"} corresponding the the minimum MAE} \item{observed: matrix with
observed values (meteorological measured values)} \item{predicted: matrix with
interpolated values for the optimum parameter combination} } If
\code{update_interpolation_params} is FALSE, \code{interpolator_calibration}
returns the interpolator provided with the parameters updated
}
\description{
Calibration and validation of interpolation procedures
}
\details{
Function \code{interpolator_calibration} determines optimal interpolation
parameters \code{"N"} and \code{"alpha"} for a given meteorological
variable. Optimization is done by minimizing mean absolute error ("MAE")
(Thornton \emph{et al.} 1997). Function
\code{interpolation_cross_validation} calculates average mean absolute
errors ("MAE") for the prediction period of the interpolator object. In both
calibration and cross validation procedures, predictions for each
meteorological station are made using a \emph{leave-one-out} procedure (i.e.
after excluding the station from the predictive set).
}
\examples{

\donttest{
# example interpolator
data("meteoland_interpolator_example")

# As the cross validation for all stations can be time consuming, we are
# gonna use only for the first 5 stations of the 198
cv <- interpolation_cross_validation(meteoland_interpolator_example, stations = 1:5)

# Inspect the results
cv$errors
cv$station_stats
cv$dates_stats
cv$r2
}


\donttest{
# example interpolator
data("meteoland_interpolator_example")

# As the calibration for all stations can be time consuming, we are gonna
# interpolate only for the first 5 stations of the 198 and only a handful
# of parameter combinations
calibration <- interpolator_calibration(
  meteoland_interpolator_example,
  stations = 1:5,
  variable = "MaxTemperature",
  N_seq = seq(10, 20, by = 5),
  alpha_seq = seq(8, 9, by = 0.25)
)

# we can update the interpolator params directly:
updated_interpolator <- interpolator_calibration(
  meteoland_interpolator_example,
  stations = 1:5,
  update_interpolation_params = TRUE,
  variable = "MaxTemperature",
  N_seq = seq(10, 20, by = 5),
  alpha_seq = seq(8, 9, by = 0.25)
)


# check the new interpolator have the parameters updated
get_interpolation_params(updated_interpolator)$N_MaxTemperature
get_interpolation_params(updated_interpolator)$alpha_MaxTemperature
}

}
\references{
Thornton, P.E., Running, S.W., 1999. An improved algorithm for
estimating incident daily solar radiation from measurements of temperature,
humidity, and precipitation. Agric. For. Meteorol. 93, 211–228.
doi:10.1016/S0168-1923(98)00126-9.

De Caceres M, Martin-StPaul N, Turco M, Cabon A, Granda V (2018) Estimating
daily meteorological data and downscaling climate models over landscapes.
Environmental Modelling and Software 108: 186-196.
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF

Victor Granda \enc{García}{Garcia}, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meteoland-package.R
\docType{package}
\name{meteoland-package}
\alias{meteoland-package}
\alias{meteoland}
\title{meteoland: Landscape Meteorology Tools}
\description{
Functions to estimate weather variables at any position of a landscape
}
\references{
De Caceres et al., 2018 (\doi{10.1016/j.envsoft.2018.08.003}))
}
\seealso{
Useful links: \itemize{ \item{
\url{https://emf-creaf.github.io/meteoland/index.html}} }
}
\author{
\strong{Maintainer}: Miquel De Cáceres
\email{miquelcaceres@gmail.com}
\href{https://orcid.org/0000-0001-7132-2080}{ORCID}

Authors: \itemize{
\item{ Víctor Granda
\href{https://orcid.org/0000-0002-0469-1991}{ORCID}}
\item{ Nicolas Martin
\href{https://orcid.org/0000-0001-7574-0108}{ORCID}}
\item{ Antoine Cabon
\href{https://orcid.org/0000-0001-6426-1726}{ORCID}}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaultInterpolationParams.R
\name{defaultInterpolationParams}
\alias{defaultInterpolationParams}
\title{Default interpolation parameters}
\usage{
defaultInterpolationParams()
}
\value{
A list with the following items (default values in brackets):
\itemize{ \item\code{initial_Rp [= 140000]}: Initial truncation radius.
\item\code{iterations [= 3]}: Number of station density iterations.
\item\code{alpha_MinTemperature [= 3.0]}: Gaussian shape parameter for
minimum temperature. \item\code{alpha_MaxTemperature [= 3.0]}: Gaussian
shape parameter for maximum temperature. \item\code{alpha_DewTemperature [=
3.0]}: Gaussian shape parameter for dew-point temperature.
\item\code{alpha_PrecipitationEvent [= 5.0]}: Gaussian shape parameter for
precipitation events. \item\code{alpha_PrecipitationAmount [= 5.0]}:
Gaussian shape parameter for the regression of precipitation amounts.
\item\code{alpha_Wind [= 3.0]}: Gaussian shape parameter for wind.
\item\code{N_MinTemperature [= 30]}: Average number of stations with
non-zero weights for minimum temperature. \item\code{N_MaxTemperature [=
30]}: Average number of stations with non-zero weights for maximum
temperature. \item\code{N_DewTemperature [= 30]}: Average number of stations
with non-zero weights for dew-point temperature.
\item\code{N_PrecipitationEvent [= 5]}: Average number of stations with
non-zero weights for precipitation events. \item\code{N_PrecipitationAmount
[= 20]}: Average number of stations with non-zero weights for the regression
of precipitation amounts. \item\code{N_Wind [= 2]}: Average number of
stations with non-zero weights for wind. \item\code{St_Precipitation [= 5]}:
Number of days for the temporal smoothing of precipitation.
\item\code{St_TemperatureRange [= 15]}: Number of days for the temporal
smoothing of temperature range. \item\code{pop_crit [= 0.50]}: Critical
precipitation occurrence parameter. \item\code{f_max [= 0.6]}: Maximum value
for precipitation regression extrapolations (0.6 equals to a maximum of 4
times extrapolation). \item\code{wind_height [= 10]}: Wind measurement
height (in m). \item\code{wind_roughness_height [= 0.001]}: Wind roughness
height (in m), for PET calculations. \item\code{penman_albedo [= 0.25]}: Albedo
for PET calculations. \item\code{penman_windfun [= "1956"]}: Wind speed function version,
either "1948" or "1956", for PET calculation. \item\code{debug [= FALSE]}: Boolean flag to show extra
console output. }
}
\description{
Returns a list with the default parameterization for interpolation. Most
parameter values are set according to Thornton et al. (1997).
}
\references{
Thornton, P.E., Running, S.W., White, M. A., 1997. Generating
surfaces of daily meteorological variables over large regions of complex
terrain. J. Hydrol. 190, 214–251. doi:10.1016/S0022-1694(96)03128-9.

De Caceres M, Martin-StPaul N, Turco M, Cabon A, Granda V (2018) Estimating
daily meteorological data and downscaling climate models over landscapes.
Environmental Modelling and Software 108: 186-196.
}
\seealso{
\code{\link{interpolate_data}}
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/precipitation_concentration.R
\name{precipitation_concentration}
\alias{precipitation_concentration}
\title{Precipitation daily concentration}
\usage{
precipitation_concentration(p)
}
\arguments{
\item{p}{A numeric vector with daily precipitation values.}
}
\value{
Function \code{precipitation_concentration()} returns a value
between 0 (equal distribution of rainfall) and 1 (one day concentrates all
rainfall).
}
\description{
Function \code{precipitation_concentration()} calculates daily precipitation
concentration (Martin-Vide et al. 2004).
}
\references{
Martin-Vide J (2004) Spatial distribution of a daily precipitation
concentration index in peninsular Spain. International Journal of
Climatology 24, 959–971. doi:10.1002/joc.1030.
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_summary_interpolated_data.R
\name{summarise_interpolator}
\alias{summarise_interpolator}
\title{Summarise interpolator objects by temporal dimension}
\usage{
summarise_interpolator(
  interpolator,
  fun = "mean",
  frequency = NULL,
  vars_to_summary = c("Temperature", "MinTemperature", "MaxTemperature", "Precipitation",
    "RelativeHumidity", "MinRelativeHumidity", "MaxRelativeHumidity", "Radiation",
    "WindSpeed", "WindDirection", "PET", "SmoothedPrecipitation",
    "SmoothedTemperatureRange", "elevation", "slope", "aspect"),
  dates_to_summary = NULL,
  months_to_summary = 1:12,
  verbose = getOption("meteoland_verbosity", TRUE),
  ...
)
}
\arguments{
\item{interpolator}{An interpolator object as created by \code{\link{create_meteo_interpolator}}.}

\item{fun}{The function to use for summarising the data.}

\item{frequency}{A string indicating the interval specification (allowed ones
are "week", "month", "quarter" and "year"). If NULL (default), aggregation
is done in one interval for all the dates present.}

\item{vars_to_summary}{A character vector with one or more variable names to
summarise. By default, all interpolated variables are summarised.}

\item{dates_to_summary}{A Date object to define the dates to be summarised.
If NULL (default), all dates in the interpolated data are processed.}

\item{months_to_summary}{A numeric vector with the month numbers to subset
the data before summarising. (e.g. \code{c(7,8)} for July and August).
This parameter allows studying particular seasons, when combined with
\code{frequency}. For example \code{frequency = "years"} and
\code{months = 6:8} leads to summarizing summer months of each year.}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}

\item{...}{Arguments needed for \code{fun}}
}
\value{
\code{summarise_interpolator} function returns the same interpolator object provided with the
temporal dimension aggregated to desired frequency.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Summarises an interpolator object by the desired temporal scale.
}
\examples{
\donttest{
# example interpolator
meteoland_interpolator_example

# aggregate all dates in the interpolator, calculating the maximum values
summarise_interpolator(meteoland_interpolator_example, fun = "max")

# aggregate weekly, calculating mean values
summarise_interpolator(meteoland_interpolator_example, frequency = "week")

}

}
\author{
\enc{Víctor}{Victor} Granda \enc{García}{Garcia}, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_meteo_and_topo.R
\name{with_meteo}
\alias{with_meteo}
\title{Ensure meteo object is ready to create an interpolator object}
\usage{
with_meteo(meteo, verbose = getOption("meteoland_verbosity", TRUE))
}
\arguments{
\item{meteo}{meteo object}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
invisible meteo object ready to pipe in the interpolator creation
}
\description{
Check integrity of meteo objects
}
\details{
This function is the first step in the creation of a meteoland interpolator,
ensuring the meteo provided contains all the required elements
}
\examples{
# example meteo
data(meteoland_meteo_example)
with_meteo(meteoland_meteo_example)

}
\seealso{
Other interpolator functions: \code{\link{add_topo}()},
\code{\link{create_meteo_interpolator}()},
\code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
\code{\link{set_interpolation_params}()}, \code{\link{write_interpolator}()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolator.R
\name{read_interpolator}
\alias{read_interpolator}
\title{Read interpolator files}
\usage{
read_interpolator(filename)
}
\arguments{
\item{filename}{interpolator file name}
}
\value{
an interpolator (stars) object
}
\description{
Read interpolator files created with \code{\link{write_interpolator}}
}
\details{
This function takes the file name of the nc file storing an interpolator
object and load it into the work environment
}
\examples{

\donttest{
# example interpolator
data(meteoland_interpolator_example)

# temporal folder
tmp_dir <- tempdir()

# write interpolator
write_interpolator(
  meteoland_interpolator_example,
  file.path(tmp_dir, "meteoland_interpolator_example.nc")
)

# check file exists
file.exists(file.path(tmp_dir, "meteoland_interpolator_example.nc"))

# read it again
read_interpolator(file.path(tmp_dir, "meteoland_interpolator_example.nc"))
}

}
\seealso{
Other interpolator functions: \code{\link{add_topo}()},
\code{\link{create_meteo_interpolator}()},
\code{\link{get_interpolation_params}()},
\code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()},
\code{\link{write_interpolator}()}
}
\author{
Victor Granda \enc{García}{Garcia}, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{utils_saturationVP}
\alias{utils_saturationVP}
\alias{utils_airDensity}
\alias{utils_atmosphericPressure}
\alias{utils_averageDailyVP}
\alias{utils_averageDaylightTemperature}
\alias{utils_latentHeatVaporisation}
\alias{utils_latentHeatVaporisationMol}
\alias{utils_psychrometricConstant}
\alias{utils_saturationVaporPressureCurveSlope}
\title{Physical utility functions}
\usage{
utils_saturationVP(temperature)

utils_averageDailyVP(Tmin, Tmax, RHmin, RHmax)

utils_atmosphericPressure(elevation)

utils_airDensity(temperature, Patm)

utils_averageDaylightTemperature(Tmin, Tmax)

utils_latentHeatVaporisation(temperature)

utils_latentHeatVaporisationMol(temperature)

utils_psychrometricConstant(temperature, Patm)

utils_saturationVaporPressureCurveSlope(temperature)
}
\arguments{
\item{temperature}{Air temperature (ºC).}

\item{Tmin, Tmax}{Minimum and maximum daily temperature (ºC).}

\item{RHmin, RHmax}{Minimum and maximum relative humidity (\%).}

\item{elevation}{Elevation above sea level (in m).}

\item{Patm}{Atmospheric air pressure (in kPa).}
}
\value{
Values returned for each function are: \itemize{
\item{\code{utils_airDensity}: air density (in kg·m-3).}
\item{\code{utils_atmosphericPressure}: Air atmospheric pressure (in kPa).}
\item{\code{utils_averageDailyVP}: average (actual) vapour pressure (in kPa).}
\item{\code{utils_averageDaylightTemperature}: average daylight air
temperature (in ºC). \item\code{utils_latentHeatVaporisation}: Latent heat
of vaporisation (MJ·kg-1).  \item\code{utils_latentHeatVaporisationMol}:
Latent heat of vaporisation (J·mol-1).}
\item{\code{utils_psychrometricConstant}: Psychrometric constant (kPa·ºC-1).}
\item{\code{utils_saturationVP}: saturation vapour pressure (in kPa).}
\item{\code{utils_saturationVaporPressureCurveSlope}: Slope of the saturation
vapor pressure curve (kPa·ºC-1).}  }
}
\description{
Set of functions used in the calculation of physical variables.
}
\section{Functions}{
\itemize{
\item \code{utils_averageDailyVP()}: Average daily VP

\item \code{utils_atmosphericPressure()}: Atmospheric pressure

\item \code{utils_airDensity()}: Air density

\item \code{utils_averageDaylightTemperature()}: Daylight temperature

\item \code{utils_latentHeatVaporisation()}: latent heat vaporisation

\item \code{utils_latentHeatVaporisationMol()}: Heat vaporisation mol

\item \code{utils_psychrometricConstant()}: psychrometric constant

\item \code{utils_saturationVaporPressureCurveSlope()}: Saturation VP curve slope

}}
\references{
McMurtrie, R. E., D. A. Rook, and F. M. Kelliher. 1990.
Modelling the yield of Pinus radiata on a site limited by water and
nitrogen. Forest Ecology and Management 30:381–413.

McMahon, T. A., M. C. Peel, L. Lowe, R. Srikanthan, and T. R. McVicar. 2013.
Estimating actual, potential, reference crop and pan evaporation using
standard meteorological data: a pragmatic synthesis. Hydrology & Earth
System Sciences 17:1331–1363. See also:
http://www.fao.org/docrep/x0490e/x0490e06.htm
}
\author{
Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolator.R
\name{get_interpolation_params}
\alias{get_interpolation_params}
\title{Retrieving interpolation parameters from interpolator object}
\usage{
get_interpolation_params(interpolator)
}
\arguments{
\item{interpolator}{interpolator object as returned by
\code{\link{create_meteo_interpolator}}}
}
\value{
The complete parameter list from the interpolator object
}
\description{
Retrieve the parameter list from and interpolator object
}
\examples{
# example interpolator
data(meteoland_interpolator_example)
# get the params from the interpolator
get_interpolation_params(meteoland_interpolator_example)

}
\seealso{
Other interpolator functions: \code{\link{add_topo}()},
\code{\link{create_meteo_interpolator}()},
\code{\link{read_interpolator}()}, \code{\link{set_interpolation_params}()},
\code{\link{with_meteo}()}, \code{\link{write_interpolator}()}
}
\author{
Victor Granda \enc{García}{Garcia}, EMF-CREAF
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidy_interpolator.R
\name{create_meteo_interpolator}
\alias{create_meteo_interpolator}
\title{Meteoland interpolator creation}
\usage{
create_meteo_interpolator(
  meteo_with_topo,
  params = NULL,
  verbose = getOption("meteoland_verbosity", TRUE)
)
}
\arguments{
\item{meteo_with_topo}{Meteo object, as returned by \code{\link{with_meteo}}}

\item{params}{Interpolation parameters as a list. Typically the result of
\code{\link{defaultInterpolationParams}}.}

\item{verbose}{Logical indicating if the function must show messages and info.
Default value checks \code{"meteoland_verbosity"} option and if not set, defaults
to TRUE. It can be turned off for the function with FALSE, or session wide with
\code{options(meteoland_verbosity = FALSE)}}
}
\value{
an interpolator object (stars)
}
\description{
Function to create the meteoland interpolator
}
\details{
This function takes meteorology information and a list of interpolation
parameters and creates the interpolator object to be ready to use.
}
\examples{

# example meteo data
data(meteoland_meteo_example)

# create the interpolator with default params
with_meteo(meteoland_meteo_example) |>
  create_meteo_interpolator()

# create the interpolator with some params changed
with_meteo(meteoland_meteo_example) |>
  create_meteo_interpolator(params = list(debug = TRUE))

}
\seealso{
Other interpolator functions: \code{\link{add_topo}()},
\code{\link{get_interpolation_params}()}, \code{\link{read_interpolator}()},
\code{\link{set_interpolation_params}()}, \code{\link{with_meteo}()},
\code{\link{write_interpolator}()}
}
\author{
Victor Granda \enc{García}{Garcia}, EMF-CREAF

Miquel De \enc{Cáceres}{Caceres} Ainsa, EMF-CREAF
}
